@@ -4,7 +4,7 @@
 # Purpose: Implements Editras App object and the Main method                  #
 # Author: Cody Precord <cprecord@editra.org>                                  #
 # Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
@@ -21,8 +21,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 49556 2007-10-31 13:03:26Z CJP $"
-__revision__ = "$Revision: 49556 $"
+__svnid__ = "$Id: Editra.py 51275 2008-01-18 09:53:04Z CJP $"
+__revision__ = "$Revision: 51275 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -33,7 +33,7 @@ import gettext
 import wx
 import ed_glob
 import ed_i18n
-from profiler import Profile_Del, Profile_Get, Profile_Set
+import profiler
 import util
 import dev_tool
 import ed_main
@@ -54,7 +54,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
     """
     def __init__(self, *args, **kargs):
         """Initialize that main app and its attributes
-        @postcondition: application is created and ready to be run in mainloop"
+        @postcondition: application is created and ready to be run in mainloop
 
         """
         wx.App.__init__(self, *args, **kargs)
@@ -79,22 +79,24 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._log = dev_tool.DEBUGP
         self._log("[app][info] Editra is Initializing")
 
-        if Profile_Get('REPORTER', 'bool', True):
+        if profiler.Profile_Get('REPORTER', 'bool', True):
             sys.excepthook = dev_tool.ExceptionHook
 
         #---- Bind Events ----#
         self.Bind(wx.EVT_ACTIVATE_APP, self.OnActivate)
         self.Bind(wx.EVT_MENU, self.OnNewWindow, id=ed_glob.ID_NEW_WINDOW)
+        self.Bind(wx.EVT_MENU, self.OnCloseWindow)
  
         return True
 
     def Exit(self, force=False):
         """Exit the program
-        @postcondition: If no toplevel windows are precent program will exit.
+        @postcondition: If no toplevel windows are present program will exit.
         @postcondition: Program may remain open if an open window is locking.
 
         """
         self._pluginmgr.WritePluginConfig()
+        profiler.Profile().Write(profiler.Profile_Get('MYPROFILE'))
         if not self._lock or force:
             wx.App.Exit(self)
 
@@ -263,12 +265,27 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         else:
             evt.Skip()
 
+    def OnCloseWindow(self, evt):
+        """Close the currently active window
+        @param evt: wx.MenuEvent
+
+        """
+        if evt.GetId() in [ed_glob.ID_CLOSE, ed_glob.ID_CLOSE_WINDOW]:
+            for window in self._windows.values():
+                if hasattr(window[0], 'IsActive') and window[0].IsActive():
+                    if hasattr(window[0], 'Close'):
+                        window[0].Close()
+                    break
+        else:
+            evt.Skip()
+
     def OpenNewWindow(self, fname=u'', caller=None):
         """Open a new window
         @keyword fname: Open a file in the new window
 
         """
-        frame = ed_main.MainWindow(None, wx.ID_ANY, Profile_Get('WSIZE'), 
+        frame = ed_main.MainWindow(None, wx.ID_ANY,
+                                   profiler.Profile_Get('WSIZE'), 
                                    ed_glob.PROG_NAME)
         if caller:
             pos = caller.GetPosition()
@@ -282,7 +299,8 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         
         # Ensure frame gets an Activate event when shown
         # this doesn't happen automatically on windows
-        wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
+        if wx.Platform == '__WXMSW__':
+            wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
 
     def RegisterWindow(self, name, window, can_lock=False):
         """Registers winows with the app. The name should be the
@@ -328,6 +346,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
             if not len(self._windows):
                 self._log("[app][info] No more open windows shutting down")
                 self.Exit()
+                return
 
             if name == repr(cur_top):
                 found = False
@@ -348,7 +367,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
             else:
                 self._log("[app][info] UnRegistered %s" % name)
         else:
-            self._log("[app][warning] The window %s is not registered" % name)
+            self._log("[app][warn] The window %s is not registered" % name)
 
     def WindowCanLock(self, winname):
         """Checks if a named window can lock the application or
@@ -361,7 +380,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         if self._windows.has_key(winname):
             return self._windows[winname][1]
         else:
-            self._log("[app][warning] the window %s has "
+            self._log("[app][warn] the window %s has "
                       "not been registered" % winname)
             return False
 
@@ -373,31 +392,31 @@ def InitConfig():
 
     """
     ed_glob.CONFIG['PROFILE_DIR'] = util.ResolvConfigDir("profiles")
-    import profiler
     profile_updated = False
     if util.HasConfigDir():
         if profiler.ProfileIsCurrent():
             profiler.Profile().Load(profiler.GetProfileStr())
         else:
-            dev_tool.DEBUGP("[main_info] Updating Profile to current version")
-            pstr = profiler.GetProfileStr()
-            # upgrade earlier profiles to current 
-            if len(pstr) > 3 and pstr[-2:] == "pp":
-                pstr = pstr + u'b'
+            dev_tool.DEBUGP("[InitConfig][info] Updating Profile to current version")
 
             # Load and update profile
+            pstr = profiler.GetProfileStr()
             profiler.Profile().Load(pstr)
             profiler.Profile().Update()
 
-            ## Force some default values to be set on upgrade
+            #---- Temporary Profile Adaptions ----#
+            ## Force some default values to be set on an upgrade
             if wx.Platform == '__WXGTK__':
-                Profile_Set('ICONS', 'Default')
+                profiler.Profile_Set('ICONS', 'Default')
             else:
-                Profile_Set('ICONS', 'Tango')
+                profiler.Profile_Set('ICONS', 'Tango')
 
-            # NOTE: currently turned off by default due to performance issues
-            if wx.Platform == '__WXMAC__':
-                Profile_Set('WRAP', False)
+            # GUI_DEBUG mode removed in 0.2.5
+            mode = profiler.Profile_Get('MODE')
+            if mode == 'GUI_DEBUG':
+                profiler.Profile_Set('MODE', 'DEBUG')
+
+            #---- End Temporary Profile Adaptions ----#
 
             # Write out updated profile
             profiler.Profile().Write(pstr)
@@ -413,13 +432,16 @@ def InitConfig():
         # Fresh install
         util.CreateConfigDir()
 
+        # Set default eol for windows
+        if wx.Platform == '__WXMSW__':
+            profiler.Profile_Set('EOL', 'Windows (\\r\\n)')
+
     # Set debug mode
-    if 'DEBUG' in Profile_Get('MODE'):
+    if 'DEBUG' in profiler.Profile_Get('MODE'):
         ed_glob.DEBUG = True
 
     # Resolve resource locations
     ed_glob.CONFIG['CONFIG_DIR'] = util.ResolvConfigDir("")
-    ed_glob.CONFIG['PIXMAPS_DIR'] = util.ResolvConfigDir("pixmaps")
     ed_glob.CONFIG['SYSPIX_DIR'] = util.ResolvConfigDir("pixmaps", True)
     ed_glob.CONFIG['PLUGIN_DIR'] = util.ResolvConfigDir("plugins")
     ed_glob.CONFIG['THEME_DIR'] = util.ResolvConfigDir(os.path.join("pixmaps",
@@ -428,7 +450,9 @@ def InitConfig():
     ed_glob.CONFIG['STYLES_DIR'] = util.ResolvConfigDir("styles")
     ed_glob.CONFIG['SYS_PLUGIN_DIR'] = util.ResolvConfigDir("plugins", True)
     ed_glob.CONFIG['SYS_STYLES_DIR'] = util.ResolvConfigDir("styles", True)
-    ed_glob.CONFIG['TEST_DIR'] = util.ResolvConfigDir("tests", True)
+    ed_glob.CONFIG['TEST_DIR'] = util.ResolvConfigDir(os.path.join("tests", 
+                                                                   "syntax"),
+                                                                   True)
     if not util.HasConfigDir("cache"):
         util.MakeConfigDir("cache")
     ed_glob.CONFIG['CACHE_DIR'] = util.ResolvConfigDir("cache")
@@ -445,7 +469,7 @@ def Main():
     """
     try:
         opts, args = getopt.getopt(sys.argv[1:], "dhv", 
-                                   ['debug', 'help', 'oldPath=', 'version'])
+                                   ['debug', 'help', 'version'])
     except getopt.GetoptError, msg:
         dev_tool.DEBUGP("[main][err] %s" % str(msg))
         opts = list()
@@ -453,11 +477,6 @@ def Main():
 
     # Process command line options
     if len(opts):
-        if opts[0][0] == "--oldPath":
-            oldpath = opts[0][1]
-            os.chdir(oldpath)
-            opts.pop(0)
-
         if True in [x[0] in ['-h', '--help'] for x in opts]:
             print ("Editra - %s - Developers Text Editor\n"
                    "Cody Precord (2005-2007)\n\n"
@@ -469,15 +488,14 @@ def Main():
                    "\nLong Arguments:\n"
                    "  --debug    Turn on console debugging\n"
                    "  --help     Show this help message\n"
-                   "  --oldPath  Don't use this!!\n"
                    "  --version  Print version number and exit\n"
                   ) % ed_glob.VERSION
-            exit(0)
+            os._exit(0)
 
         if True in [x[0] in ['-v', '--version'] for x in opts]:
             print "%s - v%s - Developers Editor" % (ed_glob.PROG_NAME, \
                                                     ed_glob.VERSION)
-            exit(0)
+            os._exit(0)
 
         if True in [x[0] in ['-d', '--debug'] for x in opts]:
             ed_glob.DEBUG = True
@@ -486,48 +504,55 @@ def Main():
     profile_updated = InitConfig()
 
     # 1. Create Application
-    dev_tool.DEBUGP("[main_info] Initializing Application...")
-    editra_app = Editra(Profile_Get('MODE') == u"GUI_DEBUG")
+    dev_tool.DEBUGP("[main][info] Initializing Application...")
+    editra_app = Editra(False)
 
     # 2. Initialize the Language Settings
-    the_locale = wx.Locale(ed_i18n.GetLangId(Profile_Get('LANG')))
-    the_locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
-    the_locale.AddCatalog(ed_glob.PROG_NAME)
-    language = gettext.translation(ed_glob.PROG_NAME, 
-                                   ed_glob.CONFIG['LANG_DIR'],
-                                   [the_locale.GetCanonicalName()], 
-                                   fallback=True)
-    language.install()
+    the_locale = wx.Locale(ed_i18n.GetLangId(profiler.Profile_Get('LANG')))
+    if the_locale.GetCanonicalName() in ed_i18n.GetAvailLocales():
+        the_locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
+        the_locale.AddCatalog(ed_glob.PROG_NAME)
+        language = gettext.translation(ed_glob.PROG_NAME, 
+                                       ed_glob.CONFIG['LANG_DIR'],
+                                       [the_locale.GetCanonicalName()], 
+                                       fallback=True)
+        language.install()
+    else:
+        del the_locale
 
     if profile_updated:
         # Make sure window iniliazes to default position
-        Profile_Del('WPOS')
+        profiler.Profile_Del('WPOS')
         wx.MessageBox(_("Your profile has been updated to the latest "
                         "version") + u"\n" + \
                       _("Please check the preferences dialog to reset "
-                        "you preferences"),
+                        "your preferences"),
                       _("Profile Updated"))
 
     # Splash a warning if version is not a final version
-    if Profile_Get('APPSPLASH') and int(ed_glob.VERSION[0]) < 1:
+    if profiler.Profile_Get('APPSPLASH') and int(ed_glob.VERSION[0]) < 1:
         import edimage
-        splash_img = edimage.getsplashwarnBitmap()
+        splash_img = edimage.splashwarn.GetBitmap()
         splash = wx.SplashScreen(splash_img, wx.SPLASH_CENTRE_ON_PARENT | \
                                  wx.SPLASH_NO_TIMEOUT, 0, None, wx.ID_ANY)
         splash.Show()
 
-    frame = ed_main.MainWindow(None, wx.ID_ANY, Profile_Get('WSIZE'), 
+    frame = ed_main.MainWindow(None, wx.ID_ANY, profiler.Profile_Get('WSIZE'), 
                                ed_glob.PROG_NAME)
     editra_app.RegisterWindow(repr(frame), frame, True)
     editra_app.SetTopWindow(frame)
 
     # Load Session Data
     # But not if there are command line args for files to open
-    if Profile_Get('SAVE_SESSION', 'bool', False) and not len(args):
+    if profiler.Profile_Get('SAVE_SESSION', 'bool', False) and not len(args):
         frame.GetNotebook().LoadSessionFiles()
 
     frame.Show(True)
-    wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
+    # Unlike wxMac/wxGTK Windows doesn't post an activate event when a window
+    # is first shown, so do it manually to make sure all event handlers get
+    # pushed.
+    if wx.Platform == '__WXMSW__':
+        wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
 
     if 'splash' in locals():
         splash.Destroy()
@@ -540,8 +565,10 @@ def Main():
             dev_tool.DEBUGP("[main][err] IndexError on commandline args")
 
     # 3. Start Applications Main Loop
-    dev_tool.DEBUGP("[main_info] Starting MainLoop...")
+    dev_tool.DEBUGP("[main][info] Starting MainLoop...")
     editra_app.MainLoop()
+    dev_tool.DEBUGP("[main][info] MainLoop finished exiting application")
+    os._exit(0)
 
 #-----------------------------------------------------------------------------#
 if __name__ == '__main__':
