@@ -1,57 +1,41 @@
-############################################################################
-#    Copyright (C) 2007 Cody Precord                                       #
-#    cprecord@editra.org                                                   #
-#                                                                          #
-#    Editra is free software; you can redistribute it and#or modify        #
-#    it under the terms of the GNU General Public License as published by  #
-#    the Free Software Foundation; either version 2 of the License, or     #
-#    (at your option) any later version.                                   #
-#                                                                          #
-#    Editra is distributed in the hope that it will be useful,             #
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
-#    GNU General Public License for more details.                          #
-#                                                                          #
-#    You should have received a copy of the GNU General Public License     #
-#    along with this program; if not, write to the                         #
-#    Free Software Foundation, Inc.,                                       #
-#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
-############################################################################
+###############################################################################
+# Name: doctools.py                                                           #
+# Purpose: Tools for managing document services                               #
+# Author: Cody Precord <cprecord@editra.org>                                  #
+# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
+# Licence: wxWindows Licence                                                  #
+###############################################################################
 
 """
 #--------------------------------------------------------------------------#
 # FILE: doctools.py                                                        #
-# @author: Cody Precord                                                    #
+# AUTHOR: Cody Precord                                                     #
 # LANGUAGE: Python                                                         #
-# @summary:                                                                #
+# SUMMARY:                                                                 #
 #  Provides helper functions and classes for managing documents and        #
 # and their services.                                                      #
 #                                                                          #
-# METHODS:
-#
-#
-#
 #--------------------------------------------------------------------------#
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: doctools.py 310 2007-07-22 02:29:01Z CodyPrecord $"
-__revision__ = "$Revision: 310 $"
+__svnid__ = "$Id: doctools.py 51096 2008-01-08 08:03:46Z CJP $"
+__revision__ = "$Revision: 51096 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
-import profiler
 import util
-import dev_tool
+from profiler import Profile_Get
 
 #--------------------------------------------------------------------------#
 
 class DocPositionMgr(object):
-    """Object for managing the saving and settign of a
-    documents position between sessions. Through the use
-    of an in memory dictionary during run time and on 
-    disk dictionary to use when starting and stopping the editor.
+    """Object for managing the saving and setting of a collection of
+    documents positions between sessions. Through the use of an in memory 
+    dictionary during run time and on disk dictionary to use when starting 
+    and stopping the editor.
+    @note: saves config to ~/.Editra/cache/
 
     """
     def __init__(self, book_path):
@@ -62,11 +46,8 @@ class DocPositionMgr(object):
         object.__init__(self)
         self._book = book_path
         self._records = dict()
-        if profiler.Profile_Get('SAVE_POS'):
-            if self.LoadBook(book_path):
-                dev_tool.DEBUGP("[docpositionmgr] successfully loaded book")
-            else:
-                dev_tool.DEBUGP("[docpositionmgr] failed to load book")
+        if Profile_Get('SAVE_POS'):
+            self.LoadBook(book_path)
 
     def AddRecord(self, vals):
         """Adds a record to the dictionary from a list of the
@@ -94,10 +75,7 @@ class DocPositionMgr(object):
         @return: postion value for the given filename
 
         """
-        if self._records.has_key(name):
-            return self._records[name]
-        else:
-            return 0
+        return self._records.get(name, 0)
 
     def LoadBook(self, book):
         """Loads a set of records from an on disk dictionary
@@ -113,6 +91,7 @@ class DocPositionMgr(object):
                 tfile = util.GetFileWriter(book)
                 tfile.close()
             except (IOError, OSError):
+                util.Log("[docpositionmgr][err] failed to load book")
                 return False
 
         reader = util.GetFileReader(book)
@@ -123,18 +102,21 @@ class DocPositionMgr(object):
             vals = line.split(u'=')
             if len(vals) != 2 or not os.path.exists(vals[0]):
                 continue
+
             try:
                 vals[1] = int(vals[1])
             except (TypeError, ValueError), msg:
-                dev_tool.DEBUGP("[docpositionmgr] %s" % str(msg))
+                util.Log("[docpositionmgr][err] %s" % str(msg))
                 continue
             else:
                 self.AddRecord(vals)
+
+        util.Log("[docpositionmgr][info] successfully loaded book")
         return True
 
     def WriteBook(self):
-        """Writes the in memory dictionary to the on disk one
-        @postcondtion: in memory doc data is written out to disk
+        """Writes the collection of files=pos to the config file
+        @postcondition: in memory doc data is written out to disk
 
         """
         writer = util.GetFileWriter(self.GetBook())
@@ -143,5 +125,4 @@ class DocPositionMgr(object):
                 writer.write(u"%s=%d\n" % (key, self._records[key]))
             writer.close()
         except (IOError, AttributeError), msg:
-            dev_tool.DEBUGP("[docpositionmgr] %s" % str(msg))
-
+            util.Log("[docpositionmgr][err] %s" % str(msg))
