@@ -21,38 +21,39 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_glob.py 49579 2007-11-01 08:46:15Z CJP $"
-__revision__ = "$Revision: 49579 $"
+__svnid__ = "$Id: ed_glob.py 51248 2008-01-16 13:32:31Z CJP $"
+__revision__ = "$Revision: 51248 $"
 
-__all__ = [ 'CONFIG', 'SB_INFO', 'SB_ROWCOL', 'VERSION', 'PROG_NAME',
-            'ID_NEW', 'ID_OPEN', 'ID_CLOSE', 'ID_CLOSEALL', 'ID_SAVE',
-            'ID_SAVEAS', 'ID_SAVEALL', 'ID_SAVE_PROFILE', 'ID_LOAD_PROFILE',
-            'ID_PRINT', 'ID_PRINT_PRE', 'ID_PRINT_SU', 'ID_EXIT', 'ID_UNDO',
-            'ID_REDO', 'ID_CUT', 'ID_COPY', 'ID_PASTE', 'ID_SELECTALL',
-            'ID_ADD_BM', 'ID_DEL_BM',  'ID_DEL_ALL_BM', 'ID_LINE_AFTER',
-            'ID_LINE_BEFORE', 'ID_CUT_LINE', 'ID_COPY_LINE', 'ID_JOIN_LINES',
-            'ID_TRANSPOSE', 'ID_FIND', 'ID_FIND_REPLACE', 'ID_QUICK_FIND',
-            'ID_PREF', 'ID_ZOOM_OUT', 'HOME_PAGE', 'CONTACT_MAIL',
-            'ID_ZOOM_IN', 'ID_ZOOM_NORMAL', 'ID_SHOW_EDGE', 'ID_SHOW_EOL',
-            'ID_SHOW_LN', 'ID_SHOW_WS', 'ID_PERSPECTIVES', 'ID_INDENT_GUIDES', 
-            'ID_VIEW_TOOL', 'ID_GOTO_LINE', 'ID_NEXT_MARK', 'ID_PRE_MARK', 
-            'ID_FONT', 'ID_EOL_MODE', 'ID_EOL_MAC', 'ID_EOL_UNIX', 'ID_EOL_WIN',
-            'ID_WORD_WRAP', 'ID_INDENT',  'ID_UNINDENT', 'ID_TO_UPPER', 
-            'ID_TO_LOWER', 'ID_SPACE_TO_TAB', 'ID_TAB_TO_SPACE', 'ID_TRIM_WS', 
-            'ID_COMMENT', 'ID_UNCOMMENT', 'ID_AUTOCOMP', 'ID_AUTOINDENT', 
-            'ID_SYNTAX', 'ID_FOLDING', 'ID_BRACKETHL', 'ID_LEXER', 
-            'ID_KWHELPER', 'ID_PLUGMGR', 'ID_STYLE_EDIT', 'ID_MACRO_START', 
-            'ID_MACRO_STOP', 'ID_MACRO_PLAY', 'ID_ABOUT', 'ID_HOMEPAGE', 
-            'ID_CONTACT', 'ID_COMMAND_BAR', 'ID_DOCUMENTATION', 'ID_COMMAND',
-            'ID_CLOSE_WINDOW' 
+__all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN', 
+            'ID_CLOSE', 'ID_CLOSEALL', 'ID_SAVE', 'ID_SAVEAS', 'ID_SAVEALL', 
+            'ID_SAVE_PROFILE', 'ID_LOAD_PROFILE', 'ID_PRINT', 'ID_PRINT_PRE', 
+            'ID_PRINT_SU', 'ID_EXIT', 'ID_UNDO', 'ID_REDO', 'ID_CUT', 
+            'ID_COPY', 'ID_PASTE', 'ID_SELECTALL', 'ID_ADD_BM', 'ID_DEL_BM',
+            'ID_DEL_ALL_BM', 'ID_LINE_AFTER', 'ID_LINE_BEFORE', 'ID_CUT_LINE',
+            'ID_COPY_LINE', 'ID_JOIN_LINES', 'ID_TRANSPOSE', 'ID_FIND',
+            'ID_FIND_REPLACE', 'ID_QUICK_FIND', 'ID_PREF', 'ID_ZOOM_OUT',
+            'HOME_PAGE', 'CONTACT_MAIL', 'ID_ZOOM_IN', 'ID_ZOOM_NORMAL',
+            'ID_SHOW_EDGE', 'ID_SHOW_EOL', 'ID_SHOW_LN', 'ID_SHOW_WS',
+            'ID_PERSPECTIVES', 'ID_INDENT_GUIDES', 'ID_VIEW_TOOL',
+            'ID_GOTO_LINE', 'ID_NEXT_MARK', 'ID_PRE_MARK', 'ID_FONT',
+            'ID_EOL_MAC', 'ID_EOL_UNIX', 'ID_EOL_WIN', 'ID_WORD_WRAP',
+            'ID_INDENT', 'ID_UNINDENT', 'ID_TO_UPPER', 'ID_TO_LOWER',
+            'ID_SPACE_TO_TAB', 'ID_TAB_TO_SPACE', 'ID_TRIM_WS', 'ID_COMMENT',
+            'ID_UNCOMMENT', 'ID_AUTOCOMP', 'ID_AUTOINDENT', 'ID_SYNTAX',
+            'ID_FOLDING', 'ID_BRACKETHL', 'ID_LEXER', 'ID_KWHELPER',
+            'ID_PLUGMGR', 'ID_STYLE_EDIT', 'ID_MACRO_START', 'ID_MACRO_STOP',
+            'ID_MACRO_PLAY', 'ID_ABOUT', 'ID_HOMEPAGE', 'ID_CONTACT',
+            'ID_COMMAND_BAR', 'ID_DOCUMENTATION', 'ID_COMMAND',
+            'ID_CLOSE_WINDOW', 'ID_USE_SOFTTABS', 'ID_DUP_LINE'
 ]
 
 #---- Project Info ----#
-AUTHOR = u'Cody Precord'
-VERSION = u'0.2.15'
-PROG_NAME = u'Editra'
-HOME_PAGE = u"http://editra.org"
-CONTACT_MAIL = u"staff@editra.org"
+# The project info was moved to another module so it could be accessed
+# externally without needing to import anything else.  It's imported
+# here with a * until there isn't anyplace left that expects to find
+# these values in this module.
+from info import *
+
 #---- End Project Info ----#
 
 #---- Imported Libs/Objects ----#
@@ -62,8 +63,7 @@ import wx
 # Values set when main loads
 CONFIG = {
           'CACHE_DIR'   : "",   # Holds temp data about documents
-          'PROFILE_DIR' : "",   # User Profile Direcotry
-          'PIXMAPS_DIR' : "",   # Graphics Directory
+          'PROFILE_DIR' : "",   # User Profile Directory
           'PLUGIN_DIR'  : "",   # User Plugin Dir
           'SYSPIX_DIR'  : "",   # Editras non user graphics
           'THEME_DIR'   : "",   # Theme Directory
@@ -110,6 +110,7 @@ ID_LINE_AFTER    = wx.NewId()
 ID_LINE_BEFORE   = wx.NewId()
 ID_CUT_LINE      = wx.NewId()
 ID_COPY_LINE     = wx.NewId()
+ID_DUP_LINE      = wx.NewId()
 ID_JOIN_LINES    = wx.NewId()
 ID_TRANSPOSE     = wx.NewId()
 ID_FIND          = wx.ID_FIND
@@ -130,7 +131,6 @@ ID_PREF_EDGE     = wx.NewId()
 ID_PREF_SYNTHEME = wx.NewId()
 ID_PREF_TABS     = wx.NewId()
 ID_PREF_TABW     = wx.NewId()
-ID_PREF_METAL    = wx.NewId()
 ID_PREF_FHIST    = wx.NewId()
 ID_PREF_WSIZE    = wx.NewId()
 ID_PREF_WPOS     = wx.NewId()
@@ -166,6 +166,7 @@ ID_EOL_MODE      = wx.NewId()
 ID_EOL_MAC       = wx.NewId()
 ID_EOL_UNIX      = wx.NewId()
 ID_EOL_WIN       = wx.NewId()
+ID_USE_SOFTTABS  = wx.NewId()
 ID_WORD_WRAP     = wx.NewId()
 ID_INDENT        = wx.ID_INDENT
 ID_UNINDENT      = wx.ID_UNINDENT
@@ -265,7 +266,6 @@ ID_2_PROF = {
              ID_TRANSPARENCY      : 'ALPHA',
              ID_WORD_WRAP         : 'WRAP',
              ID_PREF_EDGE         : 'EDGE',
-             ID_PREF_METAL        : 'METAL',
              ID_PREF_SPOS         : 'SAVE_POS',
              ID_PREF_WSIZE        : 'SET_WSIZE',
              ID_PREF_WPOS         : 'SET_WPOS',
@@ -279,4 +279,4 @@ ID_2_PROF = {
 
 # Default Plugins
 DEFAULT_PLUGINS = ("generator.Html", "generator.LaTeX", "generator.Rtf",
-                   "iface.Shelf", "ed_theme.TangoTheme")
+                   "iface.Shelf", "ed_theme.TangoTheme", "ed_log.EdLogViewer")
