@@ -21,8 +21,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 49452 2007-10-26 10:38:11Z CJP $"
-__revision__ = "$Revision: 49452 $"
+__svnid__ = "$Id: ed_main.py 51096 2008-01-08 08:03:46Z CJP $"
+__revision__ = "$Revision: 51096 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -34,8 +34,6 @@ import wx
 import wx.aui
 from ed_glob import *
 import util
-from profiler import Profile_Get as _PGET
-from profiler import Profile_Set as _PSET
 import profiler
 import ed_toolbar
 import ed_event
@@ -51,6 +49,9 @@ import iface
 
 # Function Aliases
 _ = wx.GetTranslation
+_PGET = profiler.Profile_Get
+_PSET = profiler.Profile_Set
+
 #--------------------------------------------------------------------------#
 
 class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
@@ -78,13 +79,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         self.SetTitle()
         util.SetWindowIcon(self)
 
-        # Check if user wants Metal Style under OS X
-        # NOTE: soon to be deprecated
-        if wx.Platform == '__WXMAC__' and _PGET('METAL'):
-            self.SetExtraStyle(wx.FRAME_EX_METAL)
-
         # Attributes
         self.LOG = wx.GetApp().GetLog()
+        self._exiting = False
         self._handlers = dict(menu=list(), ui=list())
 
         #---- Sizers to hold subapplets ----#
@@ -115,31 +112,21 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         #---- Setup Toolbar ----#
         self.SetToolBar(ed_toolbar.EdToolBar(self))
-        if not _PGET('TOOLBAR'):
-            self.GetToolBar().Hide()
+        self.GetToolBar().Show(_PGET('TOOLBAR'))
         #---- End Toolbar Setup ----#
 
         #---- Menus ----#
         menbar = ed_menu.EdMenuBar()
-        self._menus = dict(file=menbar.GetMenuByName("file"),
-                           edit=menbar.GetMenuByName("edit"),
-                           view=menbar.GetMenuByName("view"),
-                           viewedit=menbar.GetMenuByName("viewedit"),
-                           format=menbar.GetMenuByName("format"),
-                           settings=menbar.GetMenuByName("settings"),
-                           tools=menbar.GetMenuByName("tools"),
-                           lineformat=menbar.GetMenuByName("lineformat"),
-                           language=syntax.GenLexerMenu())
 
         # Todo this should not be hard coded
-        self._menus['view'].InsertMenu(5, ID_PERSPECTIVES, _("Perspectives"), 
-                                       self.GetPerspectiveControls())
+        menbar.GetMenuByName("view").InsertMenu(5, ID_PERSPECTIVES,
+                             _("Perspectives"), self.GetPerspectiveControls())
 
         ## Setup additional menu items
         self.filehistory.UseMenu(menbar.GetMenuByName("filehistory"))
-        self._menus['settings'].AppendMenu(ID_LEXER, _("Lexers"), 
-                                           self._menus['language'],
-                                           _("Manually Set a Lexer/Syntax"))
+        menbar.GetMenuByName("settings").AppendMenu(ID_LEXER, _("Lexers"), 
+                                                    syntax.GenLexerMenu(),
+                                              _("Manually Set a Lexer/Syntax"))
 
         # On mac, do this to make help menu appear in correct location
         # Note it must be done before setting the menu bar and after the
@@ -151,19 +138,12 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         self.SetMenuBar(menbar)
 
         #---- Actions to take on menu events ----#
-        self.Bind(wx.EVT_MENU_OPEN, self.UpdateMenu)
-        if wx.Platform == '__WXGTK__':
-            self.Bind(wx.EVT_MENU_HIGHLIGHT, \
-                      self.OnMenuHighlight, id=ID_LEXER)
-            self.Bind(wx.EVT_MENU_HIGHLIGHT, \
-                      self.OnMenuHighlight, id=ID_EOL_MODE)
 
         # Collect Menu Event handler pairs
         self._handlers['menu'].extend([# File Menu
                                        (ID_NEW, self.OnNew),
                                        (ID_OPEN, self.OnOpen),
                                        (ID_CLOSE, self.OnClosePage),
-                                       (ID_CLOSE_WINDOW, self.OnClose),
                                        (ID_CLOSEALL, self.OnClosePage),
                                        (ID_SAVE, self.OnSave),
                                        (ID_SAVEAS, self.OnSaveAs),
@@ -181,7 +161,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                        (ID_FIND_REPLACE, 
                                         self.nb.FindService.OnShowFindDlg),
                                        (ID_QUICK_FIND, self.OnCommandBar),
-                                       (ID_PREF, self.OnPreferences),
+                                       (ID_PREF, OnPreferences),
 
                                        # View Menu
                                        (ID_GOTO_LINE, self.OnCommandBar),
@@ -196,20 +176,58 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                        (ID_PLUGMGR, self.OnPluginMgr),
 
                                        # Help Menu
-                                       (ID_ABOUT, self.OnAbout),
-                                       (ID_HOMEPAGE, self.OnHelp),
-                                       (ID_DOCUMENTATION, self.OnHelp),
-                                       (ID_CONTACT, self.OnHelp)])
+                                       (ID_ABOUT, OnAbout),
+                                       (ID_HOMEPAGE, OnHelp),
+                                       (ID_DOCUMENTATION, OnHelp),
+                                       (ID_CONTACT, OnHelp)])
 
         self._handlers['menu'].extend([(l_id, self.DispatchToControl) 
                                        for l_id in syntax.SyntaxIds()])
 
-        # Extra menu handlers (need to work these into above system yet
+        # Extra menu handlers (need to work these into above system yet)
         self.Bind(wx.EVT_MENU, self.DispatchToControl)
         self.Bind(wx.EVT_MENU, self.OnGenerate)
         self.Bind(wx.EVT_MENU_RANGE, self.OnFileHistory, 
                   id=wx.ID_FILE1, id2=wx.ID_FILE9)
 
+        # Update UI Handlers
+        self._handlers['ui'].extend([# Edit Menu
+                                     (ID_COPY, self.OnUpdateClipboardUI),
+                                     (ID_CUT, self.OnUpdateClipboardUI),
+                                     (ID_PASTE, self.OnUpdateClipboardUI),
+                                     (ID_UNDO, self.OnUpdateClipboardUI),
+                                     (ID_REDO, self.OnUpdateClipboardUI),
+                                     # Format Menu
+                                     (ID_USE_SOFTTABS, self.OnUpdateFormatUI),
+                                     (ID_WORD_WRAP, self.OnUpdateFormatUI),
+                                     (ID_EOL_MAC, self.OnUpdateFormatUI),
+                                     (ID_EOL_WIN, self.OnUpdateFormatUI),
+                                     (ID_EOL_UNIX, self.OnUpdateFormatUI),
+                                     # Settings Menu
+                                     (ID_AUTOCOMP, self.OnUpdateSettingsUI),
+                                     (ID_AUTOINDENT, self.OnUpdateSettingsUI),
+                                     (ID_SYNTAX, self.OnUpdateSettingsUI),
+                                     (ID_FOLDING, self.OnUpdateSettingsUI),
+                                     (ID_BRACKETHL, self.OnUpdateSettingsUI),
+                                     # View Menu
+                                     (ID_ZOOM_NORMAL, self.OnUpdateViewUI),
+                                     (ID_ZOOM_IN, self.OnUpdateViewUI),
+                                     (ID_ZOOM_OUT, self.OnUpdateViewUI),
+                                     (ID_VIEW_TOOL, self.OnUpdateViewUI),
+                                     (ID_SHOW_WS, self.OnUpdateViewUI),
+                                     (ID_SHOW_EDGE, self.OnUpdateViewUI),
+                                     (ID_SHOW_EOL, self.OnUpdateViewUI),
+                                     (ID_SHOW_LN, self.OnUpdateViewUI),
+                                     (ID_INDENT_GUIDES, self.OnUpdateViewUI)
+                                    ])
+
+        # Lexer Menu
+        self._handlers['ui'].extend([(l_id, self.OnUpdateLexerUI) 
+                                     for l_id in syntax.SyntaxIds()])
+
+        # Perspectives
+        self._handlers['ui'].extend(self.GetPersectiveHandlers())
+
         #---- End Menu Setup ----#
 
         #---- Other Event Handlers ----#
@@ -228,12 +246,10 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         #---- End other event actions ----#
 
         #---- Final Setup Calls ----#
-        self._exiting = False
         self.LoadFileHistory(_PGET('FHIST_LVL', fmt='int'))
-        self.UpdateToolBar()
 
         # Call add on plugins
-        self.LOG("[main][info] Loading MainWindow Plugins ")
+        self.LOG("[ed_main][info] Loading MainWindow Plugins")
         plgmgr = wx.GetApp().GetPluginManager()
         addons = MainWindowAddOn(plgmgr)
         addons.Init(self)
@@ -241,8 +257,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         self._handlers['ui'].extend(addons.GetEventHandlers(ui_evt=True))
         self._shelf = iface.Shelf(plgmgr)
         self._shelf.Init(self)
-        self.LOG("[main][info] Loading Generator plugins")
-        generator.Generator(plgmgr).InstallMenu(self._menus['tools'])
+        self.LOG("[ed_main][info] Loading Generator plugins")
+        generator.Generator(plgmgr).InstallMenu(menbar.GetMenuByName("tools"))
 
         # Set Perspective
         self.SetPerspective(_PGET('DEFAULT_VIEW'))
@@ -261,12 +277,26 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         app = wx.GetApp()
         if evt.GetActive():
+            self.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
             for handler in self._handlers['menu']:
                 app.AddHandlerForID(*handler)
 
             for handler in self._handlers['ui']:
                 app.AddUIHandlerForID(*handler)
         else:
+            self.SetExtraStyle(0)
+
+            # Hack set update ui events back to proccess all here in case
+            # opened dialog needs them. Not sure why this is necessary but it
+            # is the only solution I could find to fix the external find
+            # dialogs so that there buttons become enabled when typing in the
+            # text control.
+            #
+            # If the windows that took the active position is another mainwindow
+            # it will set the events back to UPDATE_UI_PROCESS_SPECIFED to
+            # prevent all the toolbars/menu items of each window from updating
+            # when they dont need to.
+            wx.UpdateUIEvent().SetMode(wx.UPDATE_UI_PROCESS_ALL)
             for handler in self._handlers['menu']:
                 app.RemoveHandlerForID(handler[0])
 
@@ -294,7 +324,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         @type fname: string
 
         """
-        result = wx.ID_CANCEL
         try:
             e_id = evt.GetId()
         except AttributeError:
@@ -307,7 +336,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             dlg.SetFilterIndex(_PGET('FFILTER', 'int', 0))
 
             if dlg.ShowModal() != wx.ID_OK:
-                self.LOG('[mainw][info] Canceled Opening File')
+                self.LOG('[ed_main][info] Canceled Opening File')
             else:
                 _PSET('FFILTER', dlg.GetFilterIndex())
                 paths = dlg.GetPaths()
@@ -322,7 +351,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
             dlg.Destroy()
         else:
-            self.LOG("[mainw][info] CMD Open File: %s" % fname)
+            self.LOG("[ed_main][info] CMD Open File: %s" % fname)
             filename = util.GetFileName(fname)
             dirname = util.GetPathName(fname)
             self.nb.OpenPage(dirname, filename)
@@ -341,6 +370,16 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         return self.nb
 
+    def GetShelf(self):
+        """Get this windows Shelf
+        @return: reference to L{iface.Shelf} instance
+        @note: returns the plugin instance not the actual notebook, if
+               a reference to the notebook is needed for parenting call
+               GetWindow on the object returned by this function.
+
+        """
+        return self._shelf
+
     def IsExiting(self):
         """Returns whether the windows is in the process of exiting
         or not.
@@ -363,7 +402,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 if isinstance(fname, basestring) and fname:
                     self.filehistory.AddFileToHistory(fname)
         except UnicodeEncodeError, msg:
-            self.LOG("[main][err] Filehistory load failed: %s" % str(msg))
+            self.LOG("[ed_main][err] Filehistory load failed: %s" % str(msg))
 
     def OnNew(self, evt):
         """Start a New File in a new tab
@@ -374,7 +413,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         if evt.GetId() == ID_NEW:
             self.nb.NewPage()
             self.nb.GoCurrentPage()
-            self.UpdateToolBar()
         else:
             evt.Skip()
 
@@ -386,7 +424,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         if evt.GetId() == ID_OPEN:
             self.DoOpen(evt)
-            self.UpdateToolBar()
         else:
             evt.Skip()
 
@@ -423,8 +460,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         if e_id == ID_CLOSE:
             self.nb.ClosePage()
         elif e_id == ID_CLOSEALL:
-            # XXX maybe warn and ask if they really want to close
-            #     all pages before doing it.
+            # TODO maybe warn and ask if they really want to close
+            #      all pages before doing it.
             self.nb.CloseAllPages()
         else:
             evt.Skip()
@@ -469,7 +506,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 ret_val = self.OnSaveAs(ID_SAVEAS, ctrl[0], ctrl[1])
                 if ret_val:
                     self.AddFileToHistory(ctrl[1].GetFileName())
-        self.UpdateToolBar()
 
     def OnSaveAs(self, evt, title=u'', page=None):
         """Save File Using a new/different name
@@ -611,21 +647,18 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         _PSET('LAST_SESSION', self.nb.GetFileNames())
         self._exiting = True
         controls = self.nb.GetPageCount()
-        self.LOG("[main_evt][exit] Number of controls: %d" % controls)
+        self.LOG("[ed_main][evt] OnClose: Number of controls: %d" % controls)
         while controls:
             if controls <= 0:
                 self.Close(True) # Force exit since there is a problem
 
-            self.LOG("[main_evt][exit] Requesting Page Close")
+            self.LOG("[ed_main][evt] OnClose: Requesting Page Close")
             result = self.nb.ClosePage()
             if result == wx.ID_CANCEL:
-                break
+                self._exiting = False
+                return True
             controls -= 1
 
-        if result == wx.ID_CANCEL:
-            self._exiting = False
-            return True
-
         ### If we get to here there is no turning back so cleanup
         ### additional items and save user settings
 
@@ -637,13 +670,15 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         _PSET('SHELF_ITEMS', self._shelf.GetItemStack())
 
         # Save Window Size/Position for next launch
+        self.UpdateAutoPerspective()
+
         # XXX On wxMac the window size doesnt seem to take the toolbar
         #     into account so destroy it so that the window size is accurate.
         if wx.Platform == '__WXMAC__' and self.GetToolBar():
             self.GetToolBar().Destroy()
         _PSET('WSIZE', self.GetSizeTuple())
         _PSET('WPOS', self.GetPositionTuple())
-        self.LOG("[main_evt] [exit] Closing editor at pos=%s size=%s" % \
+        self.LOG("[ed_main][evt] OnClose: Closing editor at pos=%s size=%s" % \
                  (_PGET('WPOS', 'str'), _PGET('WSIZE', 'str')))
         
         # Update profile
@@ -654,7 +689,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         try:
             del self.filehistory
         except AttributeError:
-            self.LOG("[main][exit][err] Trapped AttributeError OnExit")
+            self.LOG("[ed_main][err] OnClose: Trapped AttributeError OnExit")
 
         # Post exit notice to all aui panes
         panes = self._mgr.GetAllPanes()
@@ -664,34 +699,12 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             wx.PostEvent(pane.window, exit_evt)
 
         # Finally close the window
-        self.LOG("[main_info] Closing Main Frame")
+        self.LOG("[ed_main][evt] OnClose: Closing Main Frame")
         wx.GetApp().UnRegisterWindow(repr(self))
         self.Destroy()
 
     #---- End File Menu Functions ----#
 
-    #---- Edit Menu Functions ----#
-    def OnPreferences(self, evt):
-        """Open the Preference Panel
-        @note: The dialogs module is not imported until this is 
-               first called so the first open may lag a little.
-        @param evt: Event fired that called this handler
-        @type evt: wxMenuEvent
-
-        """
-        if evt.GetId() == ID_PREF:
-            import prefdlg
-            win = wx.GetApp().GetWindowInstance(prefdlg.PreferencesDialog)
-            if win is not None:
-                win.Raise()
-                return
-            dlg = prefdlg.PreferencesDialog(None)
-            dlg.CenterOnParent()
-            dlg.Show()
-        else:
-            evt.Skip()
-    #---- End Edit Menu Functions ----#
-
     #---- View Menu Functions ----#
     def OnViewTb(self, evt):
         """Toggles visibility of toolbar
@@ -715,7 +728,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 toolbar.Show()
                 if wx.Platform != '__WXMAC__':
                     self.SetSize((size[0], size[1] + toolbar.GetSize()[1]))
-                self.UpdateToolBar()
 
             self.SendSizeEvent()
             self.Refresh()
@@ -808,53 +820,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         else:
             evt.Skip()
 
-    #---- Help Menu Functions ----#
-    def OnAbout(self, evt):
-        """Show the About Dialog
-        @param evt: Event fired that called this handler
-        @type evt: wxMenuEvent
-
-        """
-        if evt.GetId() == ID_ABOUT:
-            info = wx.AboutDialogInfo()
-            year = time.localtime()
-            desc = ["Editra is a programmers text editor.",
-                    "Written in 100%% Python.",
-                    "Homepage: " + HOME_PAGE + "\n",
-                    "Platform Info: (%s,%s)", 
-                    "License: wxWindows (see COPYING.txt for full license)"]
-            desc = "\n".join(desc)
-            py_version = sys.platform + ", python " + sys.version.split()[0]
-            platform = list(wx.PlatformInfo[1:])
-            platform[0] += (" " + wx.VERSION_STRING)
-            wx_info = ", ".join(platform)
-            info.SetCopyright("Copyright(C) 2005-%d Cody Precord" % year[0])
-            info.SetName(PROG_NAME.title())
-            info.SetDescription(desc % (py_version, wx_info))
-            info.SetVersion(VERSION)
-            wx.AboutBox(info)
-        else:
-            evt.Skip()
-
-    def OnHelp(self, evt):
-        """Handles help related menu events
-        @param evt: Event fired that called this handler
-        @type evt: wxMenuEvent
-
-        """
-        import webbrowser
-        e_id = evt.GetId()
-        if e_id == ID_HOMEPAGE:
-            webbrowser.open(HOME_PAGE, 1)
-        elif e_id == ID_DOCUMENTATION:
-            webbrowser.open_new_tab(HOME_PAGE + "/?page=doc")
-        elif e_id == ID_CONTACT:
-            webbrowser.open(u'mailto:%s' % CONTACT_MAIL)
-        else:
-            evt.Skip()
-
-    #---- End Help Menu Functions ----#
-
     #---- Misc Function Definitions ----#
     def DispatchToControl(self, evt):
         """Catches events that need to be passed to the current
@@ -865,6 +830,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         e_id = evt.GetId()
         if not self.IsActive() and e_id != ID_KWHELPER:
+            evt.Skip()
             return
 
         ctrl = self.nb.GetCurrentCtrl()
@@ -872,10 +838,25 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                         ID_JOIN_LINES, ID_CUT_LINE, ID_COPY_LINE, ID_INDENT, 
                         ID_UNINDENT, ID_TRANSPOSE, ID_COMMENT, ID_UNCOMMENT,
                         ID_SELECTALL, ID_UNDO, ID_REDO, ID_CUT, ID_COPY, 
-                        ID_PASTE, ID_LINE_BEFORE, ID_LINE_AFTER ]
+                        ID_PASTE, ID_LINE_BEFORE, ID_LINE_AFTER, ID_DUP_LINE ]
 
-        if self.FindFocus() != ctrl and e_id in active_only:
-            evt.Skip()
+        has_focus = self.FindFocus()
+        if has_focus != ctrl and e_id in active_only:
+            if has_focus is not None:
+                if e_id == ID_PASTE and hasattr(has_focus, 'Paste'):
+                    has_focus.Paste()
+                elif e_id == ID_CUT and hasattr(has_focus, 'Cut'):
+                    has_focus.Cut()
+                elif e_id == ID_COPY and hasattr(has_focus, 'Copy'):
+                    has_focus.Copy()
+                elif e_id == ID_REDO and hasattr(has_focus, 'Redo'):
+                    has_focus.Redo()
+                elif e_id == ID_UNDO and hasattr(has_focus, 'Undo'):
+                    has_focus.Undo()
+                else:
+                    evt.Skip()
+            else:
+                evt.Skip()
             return
 
         menu_ids = syntax.SyntaxIds()
@@ -886,39 +867,136 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                          ID_SHOW_LN,  ID_AUTOINDENT, ID_TAB_TO_SPACE, 
                          ID_SPACE_TO_TAB, ID_TRIM_WS, ID_SHOW_EDGE, 
                          ID_MACRO_START, ID_MACRO_STOP, ID_MACRO_PLAY, 
-                         ID_TO_LOWER, ID_TO_UPPER, ID_KWHELPER
+                         ID_TO_LOWER, ID_TO_UPPER, ID_KWHELPER, ID_USE_SOFTTABS
                          ])
         menu_ids.extend(active_only)
 
-        if evt.GetId() in menu_ids:
+        if e_id in menu_ids:
             ctrl.ControlDispatch(evt)
-            self.UpdateToolBar()
         else:
             evt.Skip()
         return
 
-    def OnKeyUp(self, evt):
-        """Update Line/Column indicator based on position.
-        @param evt: Key Event
-        @type evt: wx.KeyEvent(EVT_KEY_UP)
+    # Menu Update Handlers
+    def OnUpdateClipboardUI(self, evt):
+        """Update clipboard related menu/toolbar items
+        @param evt: EVT_UPDATE_UI
 
         """
-        self.SetStatusText(_("Line: %d  Column: %d") % \
-                           self.nb.GetCurrentCtrl().GetPos(), SB_ROWCOL)
-        evt.Skip()
+        if not self.IsActive():
+            return
 
-    def OnMenuHighlight(self, evt):
-        """HACK for GTK, submenus dont seem to fire a EVT_MENU_OPEN
-        but do fire a MENU_HIGHLIGHT
-        @note: Only used on GTK
-        @param evt: Event fired that called this handler
-        @type evt: wx.MenuEvent(EVT_MENU_HIGHLIGHT)
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        # Slow the update interval to reduce overhead
+        evt.SetUpdateInterval(200)
+        ctrl = self.nb.GetCurrentCtrl()
+        if e_id == ID_UNDO:
+            evt.Enable(ctrl.CanUndo())
+        elif e_id == ID_REDO:
+            evt.Enable(ctrl.CanRedo())
+        elif e_id == ID_PASTE:
+            evt.Enable(ctrl.CanPaste())
+        elif e_id in [ID_COPY, ID_CUT]:
+            evt.Enable(ctrl.GetSelectionStart() != ctrl.GetSelectionEnd())
+        else:
+            evt.Skip()
+
+    def OnUpdateFormatUI(self, evt):
+        """Update status of format menu items
+        @param evt: wxEVT_UPDATE_UI
 
         """
-        if evt.GetId() == ID_LEXER:
-            self.UpdateMenu(self._menus['language'])
-        elif evt.GetId() == ID_EOL_MODE:
-            self.UpdateMenu(self._menus['lineformat'])
+        if not self.IsActive():
+            return
+
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        evt.SetUpdateInterval(350)
+        ctrl = self.nb.GetCurrentCtrl()
+        eol = ctrl.GetEOLModeId()
+        if e_id == ID_USE_SOFTTABS:
+            evt.Check(not bool(ctrl.GetUseTabs()))
+        elif e_id == ID_WORD_WRAP:
+            evt.Check(bool(ctrl.GetWrapMode()))
+        elif e_id in [ID_EOL_MAC, ID_EOL_WIN, ID_EOL_UNIX]:
+            evt.Check(eol == e_id)
+        else:
+            evt.Skip()
+
+    def OnUpdateLexerUI(self, evt):
+        """Update status of lexer menu
+        @param evt: wxEVT_UPDATE_UI
+
+        """
+        if not self.IsActive():
+            return
+
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        evt.SetUpdateInterval(400)
+        if e_id in syntax.SyntaxIds():
+            lang = self.nb.GetCurrentCtrl().GetLangId()
+            evt.Check(lang == evt.GetId())
+        else:
+            evt.Skip()
+
+    def OnUpdateSettingsUI(self, evt):
+        """Update settings menu items
+        @param evt: wxEVT_UPDATE_UI
+
+        """
+        if not self.IsActive():
+            return
+
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        evt.SetUpdateInterval(300)
+        ctrl = self.nb.GetCurrentCtrl()
+        if e_id == ID_AUTOCOMP:
+            evt.Check(ctrl.GetAutoComplete())
+        elif e_id == ID_AUTOINDENT:
+            evt.Check(ctrl.GetAutoIndent())
+        elif e_id == ID_SYNTAX:
+            evt.Check(ctrl.IsHighlightingOn())
+        elif e_id == ID_FOLDING:
+            evt.Check(ctrl.IsFoldingOn())
+        elif e_id == ID_BRACKETHL:
+            evt.Check(ctrl.IsBracketHlOn())
+        else:
+            evt.Skip()
+
+    def OnUpdateViewUI(self, evt):
+        """Update status of view menu items
+        @param evt: wxEVT_UPDATE_UI
+
+        """
+        if not self.IsActive():
+            return
+
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        evt.SetUpdateInterval(300)
+        ctrl = self.nb.GetCurrentCtrl()
+        zoom = ctrl.GetZoom()
+        if e_id == ID_ZOOM_NORMAL:
+            evt.Enable(zoom)
+        elif e_id == ID_ZOOM_IN:
+            evt.Enable(zoom < 18)
+        elif e_id == ID_ZOOM_OUT:
+            evt.Enable(zoom > -8)
+        elif e_id == ID_VIEW_TOOL:
+            evt.Check(self.GetToolBar().IsShown())
+        elif e_id == ID_SHOW_WS:
+            evt.Check(bool(ctrl.GetViewWhiteSpace()))
+        elif e_id == ID_SHOW_EDGE:
+            evt.Check(bool(ctrl.GetEdgeMode()))
+        elif e_id == ID_SHOW_EOL:
+            evt.Check(bool(ctrl.GetViewEOL()))
+        elif e_id == ID_SHOW_LN:
+            evt.Check(bool(ctrl.GetMarginWidth(1)))
+        elif e_id == ID_INDENT_GUIDES:
+            evt.Check(bool(ctrl.GetIndentationGuides()))
         else:
             evt.Skip()
 
@@ -947,100 +1025,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         self._cmdbar.Show(ed_cmdbar.ID_CMD_CTRL)
         self.sizer.Layout()
 
-    # Menu Helper Functions
-    #TODO this is fairly stupid, write a more general solution
-    def UpdateMenu(self, evt):
-        """Update Status of a Given Menu.
-        @status: very ugly (hopefully temporary) hack that I hope I
-                 can get rid of as soon as I can find a better way
-                 to handle updating menuitem status.
-        @param evt: Event fired that called this handler
-        @type evt: wxMenuEvent
-
-        """
-        ctrl = self.nb.GetCurrentCtrl()
-        menu2 = None
-        if evt.GetClassName() == "wxMenuEvent":
-            menu = evt.GetMenu()
-            # HACK MSW GetMenu returns None on submenu open events
-            if menu is None:
-                menu = self._menus['language']
-                menu2 = self._menus['lineformat']
-        else:
-            menu = evt
-
-        if menu == self._menus['language']:
-            self.LOG("[main_evt] Updating Settings/Lexer Menu")
-            lang_id = ctrl.GetLangId()
-            for menu_item in menu.GetMenuItems():
-                item_id = menu_item.GetId()
-                if menu.IsChecked(item_id):
-                    menu.Check(item_id, False)
-                if item_id == lang_id or \
-                   (menu_item.GetLabel() == 'Plain Text' and lang_id == 0):
-                    menu.Check(item_id, True)
-            # HACK needed for MSW
-            if menu2 != None:
-                self.LOG("[menu_evt] Updating EOL Mode Menu")
-                eol = ctrl.GetEOLModeId()
-                for menu_id in [ID_EOL_MAC, ID_EOL_UNIX, ID_EOL_WIN]:
-                    menu2.Check(menu_id, eol == menu_id)
-                menu = self._menus['viewedit']
-        if menu == self._menus['edit']:
-            self.LOG("[main_evt] Updating Edit Menu")
-            menu.Enable(ID_UNDO, ctrl.CanUndo())
-            menu.Enable(ID_REDO, ctrl.CanRedo())
-            menu.Enable(ID_PASTE, ctrl.CanPaste())
-            sel1, sel2 = ctrl.GetSelection()
-            menu.Enable(ID_COPY, sel1 != sel2)
-            menu.Enable(ID_CUT, sel1 != sel2)
-            menu.Enable(ID_FIND, True)
-            menu.Enable(ID_FIND_REPLACE, True)
-        elif menu == self._menus['settings']:
-            self.LOG("[menu_evt] Updating Settings Menu")
-            menu.Check(ID_AUTOCOMP, ctrl.GetAutoComplete())
-            menu.Check(ID_AUTOINDENT, ctrl.GetAutoIndent())
-            menu.Check(ID_SYNTAX, ctrl.IsHighlightingOn())
-            menu.Check(ID_FOLDING, ctrl.IsFoldingOn())
-            menu.Check(ID_BRACKETHL, ctrl.IsBracketHlOn())
-        elif menu == self._menus['view']:
-            zoom = ctrl.GetZoom()
-            self.LOG("[menu_evt] Updating View Menu: zoom = %d" % zoom)
-            self._menus['view'].Enable(ID_ZOOM_NORMAL, zoom)
-            self._menus['view'].Enable(ID_ZOOM_IN, zoom < 18)
-            self._menus['view'].Enable(ID_ZOOM_OUT, zoom > -8)
-            menu.Check(ID_VIEW_TOOL, self.GetToolBar().IsShown())
-        elif menu == self._menus['viewedit']:
-            menu.Check(ID_SHOW_WS, bool(ctrl.GetViewWhiteSpace()))
-            menu.Check(ID_SHOW_EDGE, bool(ctrl.GetEdgeMode()))
-            menu.Check(ID_SHOW_EOL, bool(ctrl.GetViewEOL()))
-            menu.Check(ID_SHOW_LN, bool(ctrl.GetMarginWidth(1)))
-            menu.Check(ID_INDENT_GUIDES, bool(ctrl.GetIndentationGuides()))
-        elif menu == self._menus['format']:
-            self.LOG("[menu_evt] Updating Format Menu")
-            menu.Check(ID_WORD_WRAP, bool(ctrl.GetWrapMode()))
-        elif menu == self._menus['lineformat']:
-            self.LOG("[menu_evt] Updating EOL Mode Menu")
-            eol = ctrl.GetEOLModeId()
-            for menu_id in [ID_EOL_MAC, ID_EOL_UNIX, ID_EOL_WIN]:
-                menu.Check(menu_id, eol == menu_id)
-        else:
-            pass
-        return 0
-
-    def UpdateToolBar(self):
-        """Update Tool Status
-        @status: Temporary fix for toolbar status updating
-
-        """
-        toolbar = self.GetToolBar()
-        if not hasattr(toolbar, 'IsShown') or not toolbar.IsShown():
-            return -1
-        ctrl = self.nb.GetCurrentCtrl()
-        toolbar.EnableTool(ID_UNDO, ctrl.CanUndo())
-        toolbar.EnableTool(ID_REDO, ctrl.CanRedo())
-        toolbar.EnableTool(ID_PASTE, ctrl.CanPaste())
-
     def ModifySave(self):
         """Called when document has been modified prompting
         a message dialog asking if the user would like to save
@@ -1078,11 +1062,77 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             name = u' - ' + name
         wx.Frame.SetTitle(self, title + name)
 
-    #---- End Misc Functions ----#
 
 #-----------------------------------------------------------------------------#
-# Plugin interface's to the MainWindow
-# For backwards compatibility soon to be moved
+# Event handlers that don't need to be part of the class
+
+def OnAbout(evt):
+    """Show the About Dialog
+    @param evt: Event fired that called this handler
+    @type evt: wxMenuEvent
+
+    """
+    if evt.GetId() == ID_ABOUT:
+        info = wx.AboutDialogInfo()
+        year = time.localtime()
+        desc = ["Editra is a programmers text editor.",
+                "Written in 100%% Python.",
+                "Homepage: " + HOME_PAGE + "\n",
+                "Platform Info: (%s,%s)", 
+                "License: wxWindows (see COPYING.txt for full license)"]
+        desc = "\n".join(desc)
+        py_version = sys.platform + ", python " + sys.version.split()[0]
+        platform = list(wx.PlatformInfo[1:])
+        platform[0] += (" " + wx.VERSION_STRING)
+        wx_info = ", ".join(platform)
+        info.SetCopyright("Copyright(C) 2005-%d Cody Precord" % year[0])
+        info.SetName(PROG_NAME.title())
+        info.SetDescription(desc % (py_version, wx_info))
+        info.SetVersion(VERSION)
+        wx.AboutBox(info)
+    else:
+        evt.Skip()
+
+def OnHelp(evt):
+    """Handles help related menu events
+    @param evt: Event fired that called this handler
+    @type evt: wxMenuEvent
+
+    """
+    import webbrowser
+    e_id = evt.GetId()
+    if e_id == ID_HOMEPAGE:
+        webbrowser.open(HOME_PAGE, 1)
+    elif e_id == ID_DOCUMENTATION:
+        webbrowser.open(HOME_PAGE + "/?page=doc", 1)
+    elif e_id == ID_CONTACT:
+        webbrowser.open(u'mailto:%s' % CONTACT_MAIL)
+    else:
+        evt.Skip()
+
+def OnPreferences(evt):
+    """Open the Preference Panel
+    @note: The dialogs module is not imported until this is 
+           first called so the first open may lag a little.
+    @param evt: Event fired that called this handler
+    @type evt: wxMenuEvent
+
+    """
+    if evt.GetId() == ID_PREF:
+        import prefdlg
+        win = wx.GetApp().GetWindowInstance(prefdlg.PreferencesDialog)
+        if win is not None:
+            win.Raise()
+        else:
+            dlg = prefdlg.PreferencesDialog(None)
+            dlg.CenterOnParent()
+            dlg.Show()
+    else:
+        evt.Skip()
+
+#-----------------------------------------------------------------------------#
+# Plugin interface to the MainWindow
+# For backwards compatibility soon to be removed
 MainWindowI = iface.MainWindowI
 
 class MainWindowAddOn(plugin.Plugin):
@@ -1097,7 +1147,7 @@ class MainWindowAddOn(plugin.Plugin):
             try:
                 observer.PlugIt(window)
             except Exception, msg:
-                util.Log("[main_addon][err] %s" % str(msg))
+                util.Log("[ed_main][err] MainWindowAddOn.Init: %s" % str(msg))
 
     def GetEventHandlers(self, ui_evt=False):
         """Get Event handlers and Id's from all observers
@@ -1113,7 +1163,7 @@ class MainWindowAddOn(plugin.Plugin):
                 else:
                     items = observer.GetMenuHandlers()
             except Exception, msg:
-                util.Log("[main_addon][err] %s" % str(msg))
+                util.Log("[ed_main][err] MainWindoAddOn.GetEventHandlers: %s" % str(msg))
                 continue
             handlers.extend(items)
         return handlers
