@@ -19,8 +19,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_menu.py 49416 2007-10-25 06:06:13Z CJP $"
-__revision__ = "$Revision: 49416 $"
+__svnid__ = "$Id: ed_menu.py 51110 2008-01-08 15:03:38Z CJP $"
+__revision__ = "$Revision: 51110 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -147,7 +147,7 @@ class EdMenu(wx.Menu):
             mitem = self.FindItemByPosition(item)
             if mitem.IsSeparator():
                 continue
-            mlabel = mitem.GetLabel()
+            mlabel = mitem.GetItemLabel()
             if after and mitem.GetId() == after:
                 start = True
                 continue
@@ -271,7 +271,7 @@ class EdMenuBar(wx.MenuBar):
         filemenu.Append(ed_glob.ID_PRINT, _("Print") + "\tCtrl+P", 
                         _("Print Current File"))
         filemenu.AppendSeparator()
-        filemenu.Append(ed_glob.ID_EXIT, _("Exit") + "\tAlt+Q", 
+        filemenu.Append(ed_glob.ID_EXIT, _("Exit") + "\tCtrl+Q", 
                         _("Exit the Program"))
         self.Append(filemenu, _("File"))
         return filemenu
@@ -308,6 +308,9 @@ class EdMenuBar(wx.MenuBar):
                         _("Cut Current Line"))
         linemenu.Append(ed_glob.ID_COPY_LINE, _("Copy Line") + "\tCtrl+Y",
                         _("Copy Current Line"))
+        linemenu.Append(ed_glob.ID_DUP_LINE,
+                        _("Duplicate Line") + "\tShift+Ctrl+C",
+                        _("Duplicate the current line"))
         linemenu.AppendSeparator()
         linemenu.Append(ed_glob.ID_JOIN_LINES, _("Join Lines") + "\tCtrl+J",
                         _("Join the Selected Lines"))
@@ -400,6 +403,10 @@ class EdMenuBar(wx.MenuBar):
         formatmenu.Append(ed_glob.ID_TO_LOWER,  _("Lowercase"),
                           _("Convert selected text to all lowercase letters"))
         formatmenu.AppendSeparator()
+        formatmenu.Append(ed_glob.ID_USE_SOFTTABS,
+                          _("Use Soft Tabs") + '\tCtrl+Shift+I',
+                          _("Insert spaces instead of tab "
+                            "characters with tab key"), wx.ITEM_CHECK)
         formatmenu.Append(ed_glob.ID_WORD_WRAP, _("Word Wrap"), 
                                _("Wrap Text Horizontally"), wx.ITEM_CHECK)
         formatmenu.AppendSeparator()
@@ -502,3 +509,33 @@ class EdMenuBar(wx.MenuBar):
             return getattr(self, menu)
         else:
             return None
+
+    def ResetIcons(self):
+        """Walk through each menu item in all of the bars menu and
+        reapply icons where possible.
+        @status: Dont use, sort of works on mac, does nothing on gtk, and causes
+                 graphical glitches on msw.
+
+        """
+        for menu in self.GetMenus():
+            WalkAndSetBitmaps(menu[0])
+
+#-----------------------------------------------------------------------------#
+# Utility Functions
+
+def WalkAndSetBitmaps(menu):
+    """Recursively walk a menu and its submenus setting bitmaps
+    as necessary/available, using the the current theme.
+
+    """
+    for item in menu.GetMenuItems():
+        if item.IsSubMenu():
+            WalkAndSetBitmaps(item.GetSubMenu())
+        else:
+            bmp = wx.ArtProvider.GetBitmap(str(item.GetId()), wx.ART_MENU)
+            if bmp.IsOk():
+                item.SetBitmap(bmp)
+            elif not item.GetBitmap().IsNull():
+                item.SetBitmap(wx.NullBitmap)
+            else:
+                continue
