@@ -25,8 +25,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 49415 2007-10-25 05:06:57Z CJP $"
-__revision__ = "$Revision: 49415 $"
+__svnid__ = "$Id: ed_pages.py 51129 2008-01-09 13:24:59Z CJP $"
+__revision__ = "$Revision: 51129 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -41,6 +41,7 @@ import ed_search
 import util
 import doctools
 from extern import flatnotebook as FNB
+import ed_msg
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -184,7 +185,7 @@ class EdPages(FNB.FlatNotebook):
         self.GetTopLevelParent().Freeze()
         self.pg_num += 1
         self.control = ed_stc.EditraStc(self, wx.ID_ANY)
-        self.LOG("[nb_evt] Page Creation ID: %d" % self.control.GetId())
+        self.LOG("[ed_pages][evt] Page Creation ID: %d" % self.control.GetId())
         self.AddPage(self.control, u"Untitled - %d" % self.pg_num)
         self.SetPageImage(self.GetSelection(), str(self.control.GetLangId()))
         self.GetTopLevelParent().Thaw()
@@ -256,7 +257,7 @@ class EdPages(FNB.FlatNotebook):
         self.frame.SetTitle("%s - file://%s" % (filename, 
                                                 self.control.GetFileName()))
         self.SetPageText(self.GetSelection(), filename)
-        self.LOG("[nb_evt] Opened Page: ID = %d" % self.GetSelection())
+        self.LOG("[ed_pages][evt] Opened Page: ID = %d" % self.GetSelection())
 
         # Setup Document
         self.control.FindLexer()
@@ -282,7 +283,7 @@ class EdPages(FNB.FlatNotebook):
         if current_page < 0:
             return current_page
 
-        self.LOG("[nb_info] Current Page = %d" % current_page)
+        self.LOG("[ed_pages][info] Current Page = %d" % current_page)
 
         control = self.GetPage(current_page)
         control.SetFocus()
@@ -296,7 +297,14 @@ class EdPages(FNB.FlatNotebook):
         @return: the tabs text
 
         """
-        txt = FNB.FlatNotebook.GetPageText(self, pg_num)
+        # Often times this raises an index error in the flatnotebook code
+        # even though the pg_num here is one that is obtained by calling
+        # GetSelection which should return a valid index.
+        try:
+            txt = FNB.FlatNotebook.GetPageText(self, pg_num)
+        except IndexError:
+            txt = ''
+
         if not txt or txt[0] != u"*":
             return txt
         return txt[1:]
@@ -332,7 +340,7 @@ class EdPages(FNB.FlatNotebook):
         # Check file properties and make a "clean" list of file(s) to open
         valid_files = list()
         for fname in files:
-            self.LOG("[fdt_evt] File(s) Dropped: %s" % fname)
+            self.LOG("[ed_pages][evt] File(s) Dropped: %s" % fname)
             if not os.path.exists(fname):
                 self.frame.PushStatusText(_("Invalid file: %s") % fname, \
                                           ed_glob.SB_INFO)
@@ -421,7 +429,10 @@ class EdPages(FNB.FlatNotebook):
 
         """
         evt.Skip()
-        self.LOG("[nb_evt] Page Changed to %d" % evt.GetSelection())
+        pages = (evt.GetOldSelection(), evt.GetSelection())
+        self.LOG("[ed_pages][evt] Control Changing from Page: "
+                  "%d to Page: %d\n" % pages)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGING, (self,) + pages)
 
     def ChangePage(self, pgid):
         """Change the page and focus to the the given page id
@@ -444,13 +455,13 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CHANGED
 
         """
-        self.ChangePage(evt.GetSelection())
-        self.LOG(("[nb_evt] Control Changing from Page: %d to Page: %d\n"
-                  "[nb_info] It has file named: %s" % (evt.GetOldSelection(), 
-                                                       evt.GetSelection(), 
-                                                       self.control.GetFileName())))
-        self.frame.UpdateToolBar()
+        cpage = evt.GetSelection()
+        self.ChangePage(cpage)
+        self.LOG("[ed_pages][evt] Page Changed to %d" % cpage)
+        self.LOG("[ed_pages][info] It has file named: %s" % \
+                 self.control.GetFileName())
         evt.Skip()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED, (self, cpage))
 
     def OnPageClosing(self, evt):
         """Checks page status to flag warnings before closing
@@ -458,11 +469,13 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CLOSING
 
         """
-        self.LOG("[nb_evt] Closing Page: #%d" % self.GetSelection())
+        self.LOG("[ed_pages][evt] Closing Page: #%d" % self.GetSelection())
         page = self.GetCurrentPage()
         if len(page.GetFileName()) > 1:
             self.DocMgr.AddRecord([page.GetFileName(), page.GetCurrentPos()])
         evt.Skip()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSING, 
+                           (self, self.GetSelection()))
 
     def OnPageClosed(self, evt):
         """Handles Paged Closed Event
@@ -470,8 +483,11 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CLOSED
 
         """
-        self.LOG("[nb_evt] Closed Page: #%d" % self.GetSelection())
+        cpage = self.GetSelection()
+        self.LOG("[ed_pages][evt] Closed Page: #%d" % cpage)
         evt.Skip()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSED, (self, cpage))
+
     #---- End Event Handlers ----#
 
     def CloseAllPages(self):
@@ -482,7 +498,7 @@ class EdPages(FNB.FlatNotebook):
         for page in xrange(self.GetPageCount()):
             result = self.ClosePage()
             if result == wx.ID_CANCEL:
-                self.LOG("[nb][closeall] Canceled on page %d" % page)
+                self.LOG("[ed_pages][info] Canceled on page %d" % page)
                 break
             
     def ClosePage(self):
@@ -532,6 +548,23 @@ class EdPages(FNB.FlatNotebook):
                                               GetBitmap(lang_id, wx.ART_MENU))
         FNB.FlatNotebook.SetPageImage(self, pg_num, self._index[lang_id])
 
+    def UpdateAllImages(self):
+        """Reload and Reset all images in the notebook pages and
+        the corresponding imagelist to match those of the current theme
+        @postcondition: all images in control are updated
+
+        """
+        imglst = self.GetImageList()
+        for lang, index in self._index.iteritems():
+            bmp = wx.ArtProvider.GetBitmap(str(lang), wx.ART_MENU)
+            if bmp.IsNull():
+                self._index.setdefault(lang, \
+                                       self._index[synglob.ID_LANG_TXT])
+            else:
+                imglst.Replace(index, bmp)
+
+        self.Refresh()
+
     def UpdatePageImage(self):
         """Updates the page tab image
         @postcondition: page image is updated to reflect any changes in ctrl
@@ -540,7 +573,7 @@ class EdPages(FNB.FlatNotebook):
         pg_num = self.GetSelection()
         ftype = util.GetExtension(self.control.GetFileName())
         ftype = ftype[-1].upper()
-        self.LOG("[nb_info] Updating Page Image: Page %d" % pg_num)
+        self.LOG("[ed_pages][info] Updating Page Image: Page %d" % pg_num)
         self.SetPageImage(pg_num, str(self.control.GetLangId()))
 
     def OnUpdatePageText(self, evt):
@@ -552,12 +585,15 @@ class EdPages(FNB.FlatNotebook):
                the document.
 
         """
-        pg_num = self.GetSelection()
-        title = self.GetPageText(pg_num)
-        if self.control.GetModify():
-            title = u"*" + title
-        if title != FNB.FlatNotebook.GetPageText(self, pg_num):
-            wx.CallAfter(self.SetPageText, pg_num, title)
+        try:
+            pg_num = self.GetSelection()
+            title = self.GetPageText(pg_num)
+            if self.control.GetModify():
+                title = u"*" + title
+            if title != FNB.FlatNotebook.GetPageText(self, pg_num):
+                wx.CallAfter(self.SetPageText, pg_num, title)
+        except wx.PyDeadObjectError:
+            pass
         
     def UpdateTextControls(self):
         """Updates all text controls to use any new settings that have
