@@ -24,8 +24,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 49502 2007-10-28 03:40:01Z CJP $"
-__revision__ = "$Revision: 49502 $"
+__svnid__ = "$Id: ed_stc.py 51270 2008-01-18 07:51:02Z CJP $"
+__revision__ = "$Revision: 51270 $"
 
 #-------------------------------------------------------------------------#
 # Dependencies
@@ -40,6 +40,7 @@ from syntax import syntax
 from autocomp import autocomp
 import util
 import ed_style
+import ed_msg
 
 #-------------------------------------------------------------------------#
 # Globals
@@ -143,28 +144,22 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.Configure()
         self.UpdateBaseStyles()
 
-        # Configure Autocompletion
-        # NOTE: must be done after syntax configuration
-        if self._config['autocomp']:
-            self.ConfigureAutoComp()
-
         ### Folder Marker Styles
         self.DefineMarkers()
 
         #self.Bind(wx.stc.EVT_STC_MACRORECORD, self.OnRecordMacro)
         self.Bind(wx.stc.EVT_STC_MARGINCLICK, self.OnMarginClick)
         self.Bind(wx.stc.EVT_STC_MODIFIED, self.OnModified)
-        self.Bind(wx.EVT_CHAR, self.OnChar)
         self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
+        self.Bind(wx.EVT_CHAR, self.OnChar)
+        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
+        self.Bind(wx.EVT_LEFT_UP, self.OnKeyUp)
 
-        frame = self.GetTopLevelParent()
-        if hasattr(frame, 'OnKeyUp'):
-            self.Bind(wx.EVT_KEY_UP, self.GetTopLevelParent().OnKeyUp)
-            self.Bind(wx.EVT_LEFT_UP, self.GetTopLevelParent().OnKeyUp)
        #---- End Init ----#
 
     __name__ = u"EditraTextCtrl"
 
+    #---- Protected Member Functions ----#
     def _BuildMacro(self):
         """Constructs a macro script from items in the macro
         record list.
@@ -207,6 +202,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 #         code = compile(code_txt, self.__module__, 'exec')
 #         exec code in self.__dict__ # Inject new code into this namespace
 
+    #---- Public Member Functions ----#
     def PlayMacro(self):
         """Send the list of built up macro messages to the editor
         to be played back.
@@ -292,6 +288,27 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         if mark != -1:
             self.GotoLine(mark)
 
+    def BraceBadLight(self, pos):
+        """Highlight the character at the given position
+        @param pos: position of character to highlight with STC_STYLE_BRACEBAD
+
+        """
+        # Check if we are still alive or not, as this may be called
+        # after we have been deleted.
+        if isinstance(self, wx.stc.StyledTextCtrl):
+            wx.stc.StyledTextCtrl.BraceBadLight(self, pos)
+
+    def BraceHighlight(self, pos1, pos2):
+        """Highlight characters at pos1 and pos2
+        @param pos1: position of char 1
+        @param pos2: position of char 2
+
+        """
+        # Check if we are still alive or not, as this may be called
+        # after we have been deleted.
+        if isinstance(self, wx.stc.StyledTextCtrl):
+            wx.stc.StyledTextCtrl.BraceHighlight(self, pos1, pos2)
+
     def GetBookmarks(self):
         """Gets a list of all lines containing bookmarks
         @return: list of line numbers
@@ -313,7 +330,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.SetIndentationGuides(_PGET('GUIDES'))
         self.SetEOLFromString(_PGET('EOL'))
         self.SetViewEOL(_PGET('SHOW_EOL'))
-        self.SyntaxOnOff(_PGET('SYNTAX'))  # <- do before autocomp
         self.SetAutoComplete(_PGET('AUTO_COMP'))
         self.FoldingOnOff(_PGET('CODE_FOLD'))
         self.ToggleAutoIndent(_PGET('AUTO_INDENT'))
@@ -332,12 +348,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             sel = self.GetSelection()
             start = self.LineFromPosition(sel[0])
             end = self.LineFromPosition(sel[1])
-            c_start = self._code['comment'][0] + " "
+            c_start = self._code['comment'][0]
             c_end = u''
             if len(self._code['comment']) > 1:
-                c_end = " " + self._code['comment'][1]
+                c_end = self._code['comment'][1]
             if end > start and self.GetColumn(sel[1]) == 0:
                 end = end - 1
+
             self.BeginUndoAction()
             try:
                 nchars = 0
@@ -347,10 +364,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                     lstart = self.PositionFromLine(line_num)
                     lend = self.GetLineEndPosition(line_num)
                     text = self.GetTextRange(lstart, lend)
-                    if len(text.strip()):
+                    tmp = text.strip()
+                    if len(tmp):
                         if uncomment:
-                            text = text.replace(c_start, u'', 1)
-                            text = text.replace(c_end, u'', 1)
+                            if tmp.startswith(c_start):
+                                text = text.replace(c_start, u'', 1)
+                            if c_end and tmp.endswith(c_end):
+                                text = text.replace(c_end, u'', 1)
                             nchars = nchars - len(c_start + c_end)
                         else:
                             text = c_start + text + c_end
@@ -398,6 +418,14 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         return self._config['autoindent']
 
+    def GetCommentChars(self):
+        """Return the list of characters used to comment a string in the
+        current language.
+        @return: list of strings
+
+        """
+        return self._code['comment']
+
     def GetLangId(self):
         """Returns the language identifer of this control
         @return: language identifier of document
@@ -579,18 +607,17 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             return
         elif k_code == wx.WXK_RETURN:
 
-            if self._config['autoindent']:
+            if self._config['autoindent'] and not self.AutoCompActive():
                 if self.GetSelectedText():
                     self.CmdKeyExecute(wx.stc.STC_CMD_NEWLINE)
-                    return
-                self.AutoIndent()
+                else:
+                    self.AutoIndent()
             else:
                 evt.Skip()
 
-            if self._config['autocomp']:
-                if self.CallTipActive():
-                    self.CallTipCancel()
-                self._code['compsvc'].UpdateNamespace(True)
+            if self.CallTipActive():
+                self.CallTipCancel()
+
         else:
             evt.Skip()
 
@@ -614,7 +641,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         elif key_code in self._code['compsvc'].GetAutoCompKeys():
             if self.AutoCompActive():
                 self.AutoCompCancel()
-            command = self.GetCommandStr() + chr(key_code)
+            command = self.GetCommandStr() + unichr(key_code)
             self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowAutoCompOpt(command)
@@ -622,13 +649,24 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             if self.AutoCompActive():
                 self.AutoCompCancel()
             command = self.GetCommandStr()
-            self.AddText(chr(key_code))
+            self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowCallTip(command)
         else:
             evt.Skip()
         return
 
+    def OnKeyUp(self, evt):
+        """Update status bar of window
+        @param evt: wxEVT_KEY_UP / wxEVT_LEFT_UP
+
+        """
+        evt.Skip()
+        msg = _("Line: %d  Column: %d") % self.GetPos()
+        nevt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.GetId(), 
+                                    msg, ed_glob.SB_ROWCOL)
+        wx.PostEvent(self.GetTopLevelParent(), nevt)
+
     def OnRecordMacro(self, evt):
         """Records macro events
         @param evt: event that called this handler
@@ -705,8 +743,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         lst = self._code['compsvc'].GetAutoCompList(command)
         if len(lst):
-            options = u' '.join(lst)
-            self.AutoCompShow(0, options)
+            self.AutoCompShow(0, u' '.join(lst))
+            self.SetFocus()
 
     def ShowCallTip(self, command):
         """Shows call tip for given command
@@ -715,10 +753,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         if self.CallTipActive():
             self.CallTipCancel()
+
         tip = self._code['compsvc'].GetCallTip(command)
         if len(tip):
             curr_pos = self.GetCurrentPos()
-            tip_pos = curr_pos - (len(command) + 1)
+            tip_pos = curr_pos - (len(command.split('.')[-1]) + 1)
             fail_safe = curr_pos - self.GetColumn(curr_pos)
             self.CallTipShow(max(tip_pos, fail_safe), tip)
 
@@ -772,11 +811,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         if brace_at_caret >= 0:
             brace_opposite = self.BraceMatch(brace_at_caret)
 
+        # CallAfter necessary to reduce CG warnings on Mac
         if brace_at_caret != -1  and brace_opposite == -1:
-            self.BraceBadLight(brace_at_caret)
+            wx.CallAfter(self.BraceBadLight, brace_at_caret)
         else:
-            self.BraceHighlight(brace_at_caret, brace_opposite)
-        evt.Skip()
+            wx.CallAfter(self.BraceHighlight, brace_at_caret, brace_opposite)
 
     def OnMarginClick(self, evt):
         """Open and Close Folders as Needed
@@ -908,7 +947,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                     interp = u''
                 ex_map = { "python" : "py", "wish" : "tcl", "ruby" : "rb",
                            "bash" : "sh", "csh" : "csh", "perl" : "pl",
-                           "ksh" : "ksh", "php" : "php" }
+                           "ksh" : "ksh", "php" : "php", "booi" : "boo",
+                           "pike" : "pike"}
                 self.ConfigureLexer(ex_map.get(interp, interp))
         self.Colourise(0, -1)
 
@@ -932,6 +972,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                   ed_glob.ID_KWHELPER: self.ShowKeywordHelp,
                   ed_glob.ID_CUT_LINE : self.LineCut, 
                   ed_glob.ID_COPY_LINE : self.LineCopy,
+                  ed_glob.ID_DUP_LINE : self.LineDuplicate,
                   ed_glob.ID_BRACKETHL : self.ToggleBracketHL,
                   ed_glob.ID_SYNTAX : self.SyntaxOnOff,
                   ed_glob.ID_UNINDENT : self.BackTab,
@@ -961,7 +1002,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                     ed_glob.ID_ADD_BM    : self.Bookmark,
                     ed_glob.ID_DEL_BM    : self.Bookmark,
                     ed_glob.ID_DEL_ALL_BM : self.Bookmark}
-                    
+
+        if self.CallTipActive():
+            self.CallTipCancel()
+
+        if self.AutoCompActive():
+            self.AutoCompCancel()
+
         if e_obj.GetClassName() == "wxToolBar" or e_map.has_key(e_id):
             if e_map.has_key(e_id):
                 e_map[e_id]()
@@ -985,7 +1032,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.SetIndentationGuides(not bool(self.GetIndentationGuides()))
         elif e_id in syntax.SyntaxIds():
             f_ext = syntax.GetExtFromId(e_id)
-            self.LOG("[stc_evt] Manually Setting Lexer to %s" % str(f_ext))
+            self.LOG("[ed_stc][evt] Manually Setting Lexer to %s" % str(f_ext))
             self.FindLexer(f_ext)
         elif e_id == ed_glob.ID_AUTOCOMP:
             self.SetAutoComplete(not self.GetAutoComplete())
@@ -995,6 +1042,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.AddLine(before=True)
         elif e_id in [ed_glob.ID_TO_UPPER, ed_glob.ID_TO_LOWER]:
             self.ConvertCase(e_id == ed_glob.ID_TO_UPPER)
+        elif e_id == ed_glob.ID_USE_SOFTTABS:
+            self.SetUseTabs(not self.GetUseTabs())
         else:
             evt.Skip()
 
@@ -1029,6 +1078,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             if tmp != eol:
                 mixed = True
                 break
+
         if mixed or diff:
             if mixed:
                 self.SetViewEOL(True)
@@ -1634,11 +1684,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         if (switch is None and not self._config['folding']) or switch:
-            self.LOG("[stc_evt] Code Folding Turned On")
+            self.LOG("[ed_stc][evt] Code Folding Turned On")
             self._config['folding'] = True
             self.SetMarginWidth(FOLD_MARGIN, 12)
         else:
-            self.LOG("[stc_evt] Code Folding Turned Off")
+            self.LOG("[ed_stc][evt] Code Folding Turned Off")
             self._config['folding'] = False
             self.SetMarginWidth(FOLD_MARGIN, 0)
 
@@ -1648,11 +1698,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         if (switch is None and not self._config['highlight']) or switch:
-            self.LOG("[stc_evt] Syntax Highlighting Turned On")
+            self.LOG("[ed_stc][evt] Syntax Highlighting Turned On")
             self._config['highlight'] = True
             self.FindLexer()
         else:
-            self.LOG("[stc_evt] Syntax Highlighting Turned Off")
+            self.LOG("[ed_stc][evt] Syntax Highlighting Turned Off")
             self._config['highlight'] = False
             self.SetLexer(wx.stc.STC_LEX_NULL)
             self.ClearDocumentStyle()
@@ -1675,11 +1725,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         if (switch is None and not self._config['brackethl']) or switch:
-            self.LOG("[stc_evt] Bracket Highlighting Turned On")
+            self.LOG("[ed_stc][evt] Bracket Highlighting Turned On")
             self._config['brackethl'] = True
             self.Bind(wx.stc.EVT_STC_UPDATEUI, self.OnUpdateUI)
         else:
-            self.LOG("[stc_evt] Bracket Highlighting Turned Off")
+            self.LOG("[ed_stc][evt] Bracket Highlighting Turned Off")
             self._config['brackethl'] = False
             self.Unbind(wx.stc.EVT_STC_UPDATEUI)
 
@@ -1689,10 +1739,10 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         if (switch is None and not self.GetMarginWidth(NUM_MARGIN)) or switch:
-            self.LOG("[stc_evt] Showing Line Numbers")
+            self.LOG("[ed_stc][evt] Showing Line Numbers")
             self.SetMarginWidth(NUM_MARGIN, 30)
         else:
-            self.LOG("[stc_evt] Hiding Line Numbers")
+            self.LOG("[ed_stc][evt] Hiding Line Numbers")
             self.SetMarginWidth(NUM_MARGIN, 0)
 
     def WordLeft(self):
@@ -1839,13 +1889,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                 self.EndUndoAction()
                 self.SetSavePoint()
             except (AttributeError, OSError, IOError), msg:
-                self.LOG("[stc][err] Failed to Reload %s" % cfile)
+                self.LOG("[ed_stc][err] Failed to Reload %s" % cfile)
                 return False, str(msg)
             else:
                 self.GotoPos(cpos)
                 return True, ''
         else:
-            self.LOG("[stc][err] %s does not exists, cannot reload it." % cfile)
+            self.LOG("[ed_stc][err] %s does not exists, cant reload." % cfile)
             return False, "%s does not exist" % cfile
 
     def SaveFile(self, path):
@@ -1857,6 +1907,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         result = True
         try:
+            ed_msg.PostMessage(ed_msg.EDMSG_FILE_SAVE, (path, self._code['lang_id']))
             writer = util.GetFileWriter(path, enc=self._finfo['encoding'])
             if self.HasBom():
                 bom = unicode(util.BOM.get(self._finfo['encoding'], u''), 
@@ -1867,8 +1918,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             writer.close()
         except (AttributeError, IOError), msg:
             result = False
-            self.LOG("[stc][err] There was an error saving %s" % path)
-            self.LOG("[stc][err] ERROR: %s" % str(msg))
+            self.LOG("[ed_stc][err] There was an error saving %s" % path)
+            self.LOG("[ed_stc][err] ERROR: %s" % str(msg))
 
         if result:
             self.SetSavePoint()
@@ -1876,6 +1927,9 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.OnModified(wx.stc.StyledTextEvent(wx.stc.wxEVT_STC_MODIFIED))
             self.SetFileName(path)
 
+        wx.CallAfter(ed_msg.PostMessage,
+                    ed_msg.EDMSG_FILE_SAVED,
+                    (path, self._code['lang_id']))
         return result
 
     # With utf-16 encoded text need to remove the BOM prior to setting
@@ -1891,7 +1945,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         bom = unicode(bom, enc)
         if len(txt) > len(bom):
             if txt[:len(bom)] == bom:
-                self.LOG("[stc][info] Stripped BOM from text")
+                self.LOG("[ed_stc][info] Stripped BOM from text")
                 self._finfo['hasbom'] = True
                 txt = txt.replace(bom, u'', 1)
         wx.stc.StyledTextCtrl.SetText(self, txt)
@@ -1925,7 +1979,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self._code['compsvc'].LoadCompProvider(self.GetLexer())
         self.AutoCompSetIgnoreCase(self._code['compsvc'].GetIgnoreCase())
         self.AutoCompStops(self._code['compsvc'].GetAutoCompStops())
-        self._code['compsvc'].UpdateNamespace(True)
 
     def ConfigureLexer(self, file_ext):
         """Sets Lexer and Lexer Keywords for the specifed file extension
@@ -1938,7 +1991,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         try:
             self._code['lang_id'] = syn_data[syntax.LANGUAGE]
         except KeyError:
-            self.LOG("[stc][err] Failed to get Lang Id from Syntax package")
+            self.LOG("[ed_stc][err] Failed to get Lang Id from Syntax package")
             self._code['lang_id'] = 0
 
         lexer = syn_data[syntax.LEXER]
@@ -1958,25 +2011,25 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         try:
             keywords = syn_data[syntax.KEYWORDS]
         except KeyError:
-            self.LOG("[stc][err] No Keywords Data Found")
+            self.LOG("[ed_stc][err] No Keywords Data Found")
             keywords = []
        
         try:
             synspec = syn_data[syntax.SYNSPEC]
         except KeyError:
-            self.LOG("[stc][err] Failed to get Syntax Specifications")
+            self.LOG("[ed_stc][err] Failed to get Syntax Specifications")
             synspec = []
 
         try:
             props = syn_data[syntax.PROPERTIES]
         except KeyError:
-            self.LOG("[stc][err] No Extra Properties to Set")
+            self.LOG("[ed_stc][err] No Extra Properties to Set")
             props = []
 
         try:
             comment = syn_data[syntax.COMMENT]
         except KeyError:
-            self.LOG("[stc][err] No Comment Pattern to set")
+            self.LOG("[ed_stc][err] No Comment Pattern to set")
             comment = []
 
         # Set Lexer
@@ -2075,7 +2128,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         self.Freeze()
         self.StyleClearAll()
-        self.UpdateBaseStyles()
         self.SetSyntax(self._code['syntax_set'])
         self.DefineMarkers()
         self.Thaw()
@@ -2086,7 +2138,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         @postcondition: style is reset to default
 
         """
-        self.StyleResetDefault()
         self.StyleClearAll()
         self.SetCaretForeground(wx.NamedColor("black"))
         self.Colourise(0, -1)
@@ -2110,11 +2161,26 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                           self.GetStyleByName('brace_good'))
         self.StyleSetSpec(wx.stc.STC_STYLE_BRACEBAD, \
                           self.GetStyleByName('brace_bad'))
+        self.StyleSetSpec(wx.stc.STC_STYLE_INDENTGUIDE, \
+                          self.GetStyleByName('guide_style'))
+
+        # wx.stc.STC_STYLE_CALLTIP doesnt seem to do anything
         calltip = self.GetItemByName('calltip')
         self.CallTipSetBackground(calltip.GetBack())
         self.CallTipSetForeground(calltip.GetFore())
-        self.SetSelBackground(True, wx.SystemSettings.\
-                                       GetColour(wx.SYS_COLOUR_HIGHLIGHT))
+
+        sback = self.GetItemByName('select_style')
+        if not sback.IsNull():
+            sback = sback.GetBack()
+        else:
+            sback = wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
+
+        wspace = self.GetItemByName('whitespace_style')
+        if not wspace.IsNull():
+            self.SetWhitespaceBackground(True, wspace.GetBack())
+            self.SetWhitespaceForeground(True, wspace.GetFore())
+
+        self.SetSelBackground(True, sback)
         self.SetCaretForeground(self.GetDefaultForeColour())
         self.DefineMarkers()
         self.Colourise(0, -1)
