@@ -23,8 +23,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 49535 2007-10-30 12:38:01Z CJP $"
-__revision__ = "$Revision: 49535 $"
+__svnid__ = "$Id: ed_style.py 51096 2008-01-08 08:03:46Z CJP $"
+__revision__ = "$Revision: 51096 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -66,14 +66,11 @@ class StyleItem(object):
             %(size)s      = Format string to be swapped at runtime
         """
         object.__init__(self)
-        if fore != wx.EmptyString:
-            self.fore = fore        # Foreground color hex code
-        if face != wx.EmptyString:
-            self.face = face        # Font face name
-        if back != wx.EmptyString:
-            self.back = back        # Background color hex code
-        if size != wx.EmptyString:
-            self.size = size        # Font point size
+        self.null = False
+        self.fore = fore        # Foreground color hex code
+        self.face = face        # Font face name
+        self.back = back        # Background color hex code
+        self.size = size        # Font point size
 
     def __eq__(self, si2):
         """Defines the == operator for the StyleItem Class
@@ -82,57 +79,102 @@ class StyleItem(object):
         @rtype: bool
 
         """
-        if str(self) == str(si2):
-            return True
-        else:
-            return False
+        return str(self) == str(si2)
 
     def __str__(self):
         """Converts StyleItem to a string
+        @note: this return string is in a format that can be accepted by
+               Scintilla. Thus no spaces after the ':'
         @return: string representation of the StyleItem
 
         """
         style_str = wx.EmptyString
-        if hasattr(self, u'fore'):
+        if self.fore:
             style_str = u"fore:%s," % self.fore
-        if hasattr(self, u'back'):
+        if self.back:
             style_str += u"back:%s," % self.back
-        if hasattr(self, u'face'):
+        if self.face:
             style_str += u"face:%s," % self.face
-        if hasattr(self, u'size'):
+        if self.size:
             style_str += u"size:%s," % str(self.size)
+
         if len(style_str) and style_str[-1] == u',':
             style_str = style_str[0:-1]
         return style_str
 
     #---- Get Functions ----#
+    def GetAsList(self):
+        """Returns a list of attr:value strings
+        this style item.
+        @return: list attribute values usable for building stc or ess values
+
+        """
+        retval = list()
+        for attr in ('fore', 'back', 'face', 'size'):
+            val = getattr(self, attr, None)
+            if val not in ( None, wx.EmptyString ):
+                retval.append(attr + ':' + val)
+        return retval
+
     def GetBack(self):
         """Returns the value of the back attribute
         @return: style items background attribute
 
         """
-        return getattr(self, 'back', wx.EmptyString)
+        return self.back.split(',')[0]
 
     def GetFace(self):
         """Returns the value of the face attribute
         @return: style items font face attribute
 
         """
-        return getattr(self, 'face', wx.EmptyString)
+        return self.face.split(',')[0]
 
     def GetFore(self):
         """Returns the value of the fore attribute
         @return: style items foreground attribute
 
         """
-        return getattr(self, 'fore', wx.EmptyString)
+        return self.fore.split(',')[0]
 
     def GetSize(self):
         """Returns the value of the size attribute as a string
         @return: style items font size attribute
 
         """
-        return getattr(self, 'size', wx.EmptyString)
+        return self.size.split(',')[0]
+
+    def GetNamedAttr(self, attr):
+        """Get the value of the named attribute
+        @param attr: named attribute to get value of
+
+        """
+        return getattr(self, attr, None)
+
+    #---- Utilities ----#
+    def IsNull(self):
+        """Return whether the item is null or not
+        @return: bool
+
+        """
+        return self.null
+
+    def IsOk(self):
+        """Check if the style item is ok or not, if it has any of its
+        attributes set it is percieved as ok.
+        @return: bool
+
+        """
+        return len(self.__str__())
+
+    def Nullify(self):
+        """Clear all values and set item as Null
+        @postcondition: item is turned into a NullStyleItem
+
+        """
+        self.null = True
+        for attr in ('fore', 'face', 'back', 'size'):
+            setattr(self, attr, '')
 
     #---- Set Functions ----#
     def SetAttrFromStr(self, style_str):
@@ -144,6 +186,7 @@ class StyleItem(object):
         @type style_str: string
 
         """
+        self.null = False
         last_set = wx.EmptyString
         for atom in style_str.split(u','):
             attrib = atom.split(u':')
@@ -160,45 +203,48 @@ class StyleItem(object):
 
     def SetBack(self, back, ex=wx.EmptyString):
         """Sets the Background Value
-        @param back: hex color string
+        @param back: hex color string, or None to clear attribute
         @keyword ex: extra attribute (i.e bold, italic, underline)
 
         """
-        if ex == wx.EmptyString:
+        self.null = False
+        if back is None or ex == wx.EmptyString:
             self.back = back
         else:
             self.back = u"%s,%s" % (back, ex)
 
     def SetFace(self, face, ex=wx.EmptyString):
         """Sets the Face Value
-        @param face: font name string
+        @param face: font name string, or None to clear attribute
         @keyword ex: extra attribute (i.e bold, italic, underline)
 
         """
-        if ex == wx.EmptyString:
+        if face is None or ex == wx.EmptyString:
             self.face = face
         else:
             self.face = u"%s,%s" % (face, ex)
 
     def SetFore(self, fore, ex=wx.EmptyString):
         """Sets the Foreground Value
-        @param fore: hex color string
+        @param fore: hex color string, or None to clear attribute
         @keyword ex: extra attribute (i.e bold, italic, underline)
 
         """
-        if ex == wx.EmptyString:
+        self.null = False
+        if fore is None or ex == wx.EmptyString:
             self.fore = fore
         else:
             self.fore = u"%s,%s" % (fore, ex)
 
     def SetSize(self, size, ex=wx.EmptyString):
         """Sets the Font Size Value
-        @param size: font point size
+        @param size: font point size, or None to clear attribute
         @type size: string or int
         @keyword ex: extra attribute (i.e bold, italic, underline)
 
         """
-        if ex == wx.EmptyString:
+        self.null = False
+        if size is None or ex == wx.EmptyString:
             self.size = size
         else:
             self.size = u"%s,%s" % (str(size), ex)
@@ -214,12 +260,13 @@ class StyleItem(object):
 
         """
         # Get currently set attributes
+        self.null = False
         cur_str = self.__str__()
         if not add:
             cur_str = cur_str.replace(u',' + ex_attr, wx.EmptyString)
             self.SetAttrFromStr(cur_str)
         else:
-            if ex_attr not in cur_str:
+            if u',' + ex_attr not in cur_str:
                 attr_map = { u"fore" : self.GetFore(),
                              u"back" : self.GetBack(),
                              u"face" : self.GetFace(),
@@ -240,8 +287,9 @@ class StyleItem(object):
         @param value: value to set the attribute to contain
 
         """
-        if hasattr(self, attr):
-            cur_val = getattr(self, attr)
+        self.null = False
+        cur_val = getattr(self, attr, None)
+        if cur_val is not None:
             if u"," in cur_val:
                 tmp = cur_val.split(u",")
                 tmp[0] = value
@@ -277,9 +325,9 @@ class StyleMgr(object):
 
         # Get the Style Set
         if custom != wx.EmptyString and self.LoadStyleSheet(custom):
-            self.LOG("[styles][init] Loaded custom style sheet %s" % custom)
+            self.LOG("[ed_style][info] Loaded custom style sheet %s" % custom)
         else:
-            self.LOG("[styles][err] Failed to import styles from %s" % custom)
+            self.LOG("[ed_style][err] Failed to import styles from %s" % custom)
 
     def BlankStyleDictionary(self):
         """Returns a dictionary of unset style items based on the
@@ -290,14 +338,18 @@ class StyleMgr(object):
         """
         sty_dict = dict()
         for key in self.GetStyleSet().keys():
-            sty_dict[key] = StyleItem("#000000", "#FFFFFF", 
-                                      "%(primary)s", "%(size)d")
+            if key in ('select_style', 'whitespace_style'):
+                sty_dict[key] = NullStyleItem()
+            else:
+                sty_dict[key] = StyleItem("#000000", "#FFFFFF", 
+                                          "%(primary)s", "%(size)d")
         return sty_dict
 
     def GetFontDictionary(self, default=True):
         """Does a system lookup to build a default set of fonts using
-        ten point fonts as the standards size.
-        @keyword default: return default dictionary or custom
+        ten point fonts as the standard size.
+        @keyword default: return the default dictionary of fonts, else return
+                          the current running dictionary of fonts if it exists.
         @type default: bool
         @return: font dictionary (primary, secondary) + (size, size2)
 
@@ -362,8 +414,10 @@ class StyleMgr(object):
         fore = self.GetItemByName('default_style').GetFore()
         if fore == wx.EmptyString:
             fore = u"#000000"
+
         if not as_hex:
-            fore = wx.ColourRGB(int(fore[1:], 16))
+            rgb = util.HexToRGB(fore[1:])
+            fore = wx.Colour(red=rgb[0], green=rgb[1], blue=rgb[2])
         return fore
 
     def GetDefaultBackColour(self, as_hex=False):
@@ -379,8 +433,10 @@ class StyleMgr(object):
         back = self.GetItemByName('default_style').GetBack()
         if back == wx.EmptyString:
             back = u"#FFFFFF"
+
         if not as_hex:
-            back = wx.ColourRGB(int(back[1:], 16))
+            rgb = util.HexToRGB(back[1:])
+            back = wx.Colour(red=rgb[0], green=rgb[1], blue=rgb[2])
         return back
 
     def GetItemByName(self, name):
@@ -391,13 +447,13 @@ class StyleMgr(object):
 
         """
         if self.HasNamedStyle(name):
-            if u"%" in unicode(self.STYLES[self.style_set][name]):
-                val = unicode(self.STYLES[self.style_set][name]) % self.fonts
+            if u"%" in unicode(StyleMgr.STYLES[self.style_set][name]):
+                val = unicode(StyleMgr.STYLES[self.style_set][name]) % self.fonts
                 item = StyleItem()
                 item.SetAttrFromStr(val)
                 return item
             else:
-                return self.STYLES[self.style_set][name]
+                return StyleMgr.STYLES[self.style_set][name]
         else:
             return StyleItem()
 
@@ -435,7 +491,7 @@ class StyleMgr(object):
         @rtype: dict
 
         """
-        return self.STYLES.get(self.style_set, DefaultStyleDictionary())
+        return StyleMgr.STYLES.get(self.style_set, DefaultStyleDictionary())
 
     def HasNamedStyle(self, name):
         """Checks if a style has been set/loaded or not
@@ -456,21 +512,21 @@ class StyleMgr(object):
         """
         if isinstance(style_sheet, basestring) and \
            os.path.exists(style_sheet) and \
-           ((force or not self.STYLES.has_key(style_sheet)) or \
+           ((force or not StyleMgr.STYLES.has_key(style_sheet)) or \
              style_sheet != self.style_set):
             reader = util.GetFileReader(style_sheet)
             if reader == -1:
-                self.LOG("[styles][err] Failed to open style sheet: %s" % style_sheet)
+                self.LOG("[ed_style][err] Failed to open style sheet: %s" % style_sheet)
                 return False
             ret_val = self.SetStyles(style_sheet, self.ParseStyleData(reader.read()))
             reader.close()
             return ret_val
-        elif not self.STYLES.has_key(style_sheet):
-            self.LOG("[styles] Style sheet %s does not exists" % style_sheet)
+        elif not StyleMgr.STYLES.has_key(style_sheet):
+            self.LOG("[ed_style][warn] Style sheet %s does not exists" % style_sheet)
             self.SetStyles('default', DefaultStyleDictionary())
             return False
         else:
-            self.LOG("[styles][info] Using cached style data")
+            self.LOG("[ed_style][info] Using cached style data")
             return True
 
     def PackStyleSet(self, style_set):
@@ -484,6 +540,8 @@ class StyleMgr(object):
         if isinstance(style_set, dict) and style_set.has_key('default_style'):
             default = style_set['default_style']
             for tag in style_set:
+                if style_set[tag].IsNull():
+                    continue
                 if style_set[tag].GetFace() == wx.EmptyString:
                     style_set[tag].SetFace(default.GetFace())
                 if style_set[tag].GetFore() == wx.EmptyString:
@@ -522,10 +580,10 @@ class StyleMgr(object):
         tmp = style_tree
         for style in tmp:
             if len(style) != 2:
-                self.LOG("[styles][error] There was an error parsing "
+                self.LOG("[ed_style][err] There was an error parsing "
                          "the syntax data from " + self.style_set)
-                self.LOG("[styles][error] You are missing a { or } " +
-                        "in Def: " + style[0].split()[0])
+                self.LOG("[ed_style][err] You are missing a { or } " +
+                         "in Def: " + style[0].split()[0])
                 if strict:
                     raise SyntaxError, \
                           "Missing { or } near Def: %s" % style[0].split()[0]
@@ -550,12 +608,12 @@ class StyleMgr(object):
             for leaf in branch[1]:
                 leaf[0] = leaf[0].strip() # Remove any remaining whitespace
                 if len(leaf) != 2:
-                    self.LOG("[styles][error] Missing a : or ; in the "
+                    self.LOG("[ed_style][err] Missing a : or ; in the "
                              "declaration of %s" % tag)
                     if strict:
                         raise SyntaxError, "Missing : or ; in def: %s" % tag
                 elif leaf[0] not in STY_ATTRIBUTES:
-                    self.LOG(("[styles][warning] Unknown style attribute: %s"
+                    self.LOG(("[ed_style][warn] Unknown style attribute: %s"
                              ", In declaration of %s") % (leaf[0], tag))
                     if strict:
                         raise SyntaxWarning, "Unknown attribute %s" % leaf[0]
@@ -564,7 +622,7 @@ class StyleMgr(object):
             style_dict[tag] = value
 
         # Trim the leafless branches from the dictionary
-        tmp = style_dict
+        tmp = style_dict.copy()
         for style_def in tmp:
             if len(tmp[style_def]) == 0:
                 style_dict.pop(style_def)
@@ -572,7 +630,7 @@ class StyleMgr(object):
         # Validate leaf values and format into stylestring
         for style_def in style_dict:
             if not style_def[0][0].isalpha():
-                self.LOG("[styles][syntax_error] The style def %s is not a "
+                self.LOG("[ed_style][err] The style def %s is not a "
                          "valid name" % style_def[0])
                 if strict:
                     raise SyntaxError, "%s is an invalid name" % style_def[0]
@@ -582,7 +640,7 @@ class StyleMgr(object):
                     values = [ val for val in attrib[1].split(u" ") 
                                if val != wx.EmptyString ]
                     if len(values) > 2:
-                        self.LOG("[styles][warning] Only one extra " +
+                        self.LOG("[ed_style][warn] Only one extra " +
                                  "attribute can be set per style. See " +
                                  style_def + " => " + attrib[0])
                         if strict:
@@ -601,14 +659,14 @@ class StyleMgr(object):
                         if match1.match(values[0]) or match2.match(values[0]):
                             v1ok = True
                         else:
-                            self.LOG("[styles] [syntax_warning] Bad value in %s"
+                            self.LOG("[ed_style][warn] Bad value in %s"
                                      " the value %s is invalid." % \
                                      (attrib[0], values[0]))
 
                     if len(values) == 2 and values[1] in STY_EX_ATTRIBUTES:
                         v2ok = True
                     elif len(values) == 2:
-                        self.LOG("[styles][warning] Unknown extra " + \
+                        self.LOG("[ed_style][warn] Unknown extra " + \
                                  "attribute '" + values[1] + \
                                  "' in attribute: " + attrib[0])
 
@@ -626,9 +684,10 @@ class StyleMgr(object):
                     style_dict[style_def] = style_str.strip(u",")
 
         # Build a StyleItem Dictionary
-        for key in style_dict:
+        for key, value in style_dict.iteritems():
             new_item = StyleItem()
-            new_item.SetAttrFromStr(style_dict[key])
+            if isinstance(value, basestring):
+                new_item.SetAttrFromStr(value)
             style_dict[key] = new_item
 
         return style_dict
@@ -668,7 +727,7 @@ class StyleMgr(object):
         @param value: style item to set tag to
 
         """
-        self.STYLES[self.style_set][style_tag] = value
+        StyleMgr.STYLES[self.style_set][style_tag] = value
 
     def SetStyles(self, name, style_dict, nomerge=False):
         """Sets the managers style data and returns True on success.
@@ -681,7 +740,7 @@ class StyleMgr(object):
         """
         if nomerge:
             self.style_set = name
-            self.STYLES[name] = self.PackStyleSet(style_dict)
+            StyleMgr.STYLES[name] = self.PackStyleSet(style_dict)
             return True
 
         # Merge the given style set with the default set to fill in any
@@ -690,15 +749,15 @@ class StyleMgr(object):
             # Check for bad data
             for style in style_dict.values():
                 if not isinstance(style, StyleItem):
-                    self.LOG("[styles][error] Invalid data in style dictionary")
+                    self.LOG("[ed_style][err] Invalid data in style dictionary")
                     return False
 
             self.style_set = name
             tmp = MergeStyles(DefaultStyleDictionary(), style_dict)
-            self.STYLES[name] = self.PackStyleSet(tmp)
+            StyleMgr.STYLES[name] = self.PackStyleSet(tmp)
             return True
         else:
-            self.LOG("[styles][error] SetStyles expects a " \
+            self.LOG("[ed_style][err] SetStyles expects a " \
                      "dictionary of StyleItems")
             return False
 
@@ -732,7 +791,8 @@ def DefaultStyleDictionary():
          'dockey_style' : StyleItem("#0000FF"),
          'error_style' : StyleItem("#DD0101,bold", face="%(secondary)s"),
          'funct_style' : StyleItem("#008B8B,italic"),
-         'global_style' : StyleItem("#007F7F,bold", face="%(secondary)s"), 
+         'global_style' : StyleItem("#007F7F,bold", face="%(secondary)s"),
+         'guide_style' : StyleItem("#838383"),
          'here_style' : StyleItem("#CA61CA,bold", face="%(secondary)s"),
          'ideol_style' : StyleItem("#E0C0E0", face="%(secondary)s"),
          'keyword_style' : StyleItem("#A52B2B,bold"),
@@ -749,10 +809,12 @@ def DefaultStyleDictionary():
          'regex_style' : StyleItem("#008B8B"),
          'scalar_style' : StyleItem("#AB37F2,bold", face="%(secondary)s"),
          'scalar2_style' : StyleItem("#AB37F2", face="%(secondary)s"),
+         'select_style' : NullStyleItem(), # Use system default colour
          'string_style' : StyleItem("#FF3AFF,bold"),
          'stringeol_style' : StyleItem("#000000,bold", "#EEC0EE,eol", \
                                        "%(secondary)s"),
-         'unknown_style' : StyleItem("#FFFFFF,bold", "#DD0101,eol")
+         'unknown_style' : StyleItem("#FFFFFF,bold", "#DD0101,eol"),
+         'whitespace_style' : StyleItem('#838383')
          }
     return def_dict
 
@@ -782,3 +844,12 @@ def MergeStyles(styles1, styles2):
     for style in styles2:
         styles1[style] = styles2[style]
     return styles1
+
+def NullStyleItem():
+    """Create a null style item
+    @return: empty style item that cannot be merged
+
+    """
+    item = StyleItem()
+    item.null = True
+    return item
