@@ -23,8 +23,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: iface.py 49291 2007-10-21 12:59:57Z CJP $"
-__revision__ = "$Revision: 49291 $"
+__svnid__ = "$Id: iface.py 51110 2008-01-08 15:03:38Z CJP $"
+__revision__ = "$Revision: 51110 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -45,7 +45,7 @@ _ = wx.GetTranslation
 class MainWindowI(plugin.Interface):
     """The MainWindow Interface is intended as a simple general purpose
     interface for adding functionality to the main window. It does little
-    managing of how object the implementing it is handled, most is left up to
+    managing of how objects that implement it are handled, most is left up to
     the plugin. Some examples of plugins using this interface are the
     FileBrowser and Calculator plugins.
 
@@ -130,6 +130,16 @@ class ShelfI(plugin.Interface):
 
         """
 
+    def IsStockable(self):
+        """Return whether this item type is stockable. The shelf saves
+        what pages it had open the last time the program was run and then
+        reloads the pages the next time the program starts. If this
+        item can be reloaded between sessions return True otherwise return
+        False.
+
+        """
+        
+
 #-----------------------------------------------------------------------------#
 SHELF_NAME = u'Shelf'
 class Shelf(plugin.Plugin):
@@ -165,7 +175,7 @@ class Shelf(plugin.Plugin):
             try:
                 menu_i = observer.GetMenuEntry(menu)
                 if menu_i:
-                    menu_items.append((menu_i.GetLabel(), menu_i))
+                    menu_items.append((menu_i.GetItemLabel(), menu_i))
             except Exception, msg:
                 self._log("[shelf][err] %s" % str(msg))
         menu_items.sort()
@@ -177,6 +187,33 @@ class Shelf(plugin.Plugin):
             menu.AppendItem(item[1])
         return menu
 
+    def AddItem(self, item, name):
+        """Add an item to the shelfs notebook. This is usefull for interacting
+        with the Shelf from outside its interface. It may be necessary to
+        call L{EnsureShelfVisible} before or after adding an item if you wish
+        the shelf to be shown when the item is added.
+        @param item: A panel like instance to add to the shelfs notebook
+        @param name: Items name used for page text in notebook
+
+        """
+        self._shelf.AddPage(item, u"%s - %d" % (name, self._open.get(name, 0)))
+        self._open[name] = self._open.get(name, 0) + 1
+
+    def CanStockItem(self, item_name):
+        """See if a named item can be stocked or not, meaning if it
+        can be saved and opened in the next session or not.
+        @param item_name: name of item to check
+        @return: bool whether item can be stocked or not
+
+        """
+        for item in self.observers:
+            if item_name == item.GetName():
+                if hasattr(item, 'IsStockable'):
+                    return item.IsStockable()
+                else:
+                    break
+        return False
+
     def Init(self, parent):
         """Mixes the shelf into the parent window
         @param parent: Reference to MainWindow
@@ -250,7 +287,7 @@ class Shelf(plugin.Plugin):
 
         for page in xrange(self._shelf.GetPageCount()):
             if item_name == re.sub(PGNUM_PAT, u'', 
-                                  self._shelf.GetPageText(page), 1):
+                                   self._shelf.GetPageText(page), 1):
                 count = count + 1
         return count
 
@@ -279,6 +316,21 @@ class Shelf(plugin.Plugin):
                         self._shelf.GetPageText(page), 1))
         return rval
 
+    def GetOwnerWindow(self):
+        """Return the L{ed_main.MainWindow} instance that owns/created
+        this Shelf.
+        @return: reference to ed_main.MainWindow or None
+
+        """
+        return self._parent
+
+    def GetWindow(self):
+        """Return reference to the Shelfs window component
+        @return: FlatnoteBook
+
+        """
+        return self._shelf
+
     def Hide(self):
         """Hide the shelf
         @postcondition: Shelf is hidden by aui manager
@@ -333,13 +385,17 @@ class Shelf(plugin.Plugin):
     def OnPutShelfItemAway(self, evt):
         """Handles when an item is closed
         @param evt: event that called this handler
+        @todo: is this needed?
 
         """
         print "OnPutShelfItemAway Not implemented"
         evt.Skip()
 
     def PutItemOnShelf(self, shelfid):
-        """Put an item on the shelf
+        """Put an item on the shelf by using its unique shelf id.
+        This is only for use with loading items implementing the
+        L{ShelfI} interface. See L{AddItem} if you wish to pass
+        a panel to the shelf to add.
         @param shelfid: id of the ShelfItem to open
 
         """
@@ -359,9 +415,7 @@ class Shelf(plugin.Plugin):
             return
         else:
             self.EnsureShelfVisible()
-            self._shelf.AddPage(item.CreateItem(self._shelf), 
-                                u"%s - %d" % (name, self._open.get(name, 0)))
-            self._open[name] = self._open.get(name, 0) + 1
+            self.AddItem(item.CreateItem(self._shelf), name)
 
     def ItemIsOnShelf(self, item_name):
         """Check if at least one instance of a given item
@@ -373,7 +427,7 @@ class Shelf(plugin.Plugin):
             return False
 
         for page in xrange(self._shelf.GetPageCount()):
-            if item_name in self._shelf.GetPageText(page):
+            if self._shelf.GetPageText(page).startswith(item_name):
                 return True
         return False
 
@@ -384,8 +438,9 @@ class Shelf(plugin.Plugin):
 
         """
         for item in i_list:
-            itemid = self.GetItemId(item)
-            if itemid:
-                self.PutItemOnShelf(itemid)
+            if self.CanStockItem(item):
+                itemid = self.GetItemId(item)
+                if itemid:
+                    self.PutItemOnShelf(itemid)
 
 #--------------------------------------------------------------------------#
