@@ -19,8 +19,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: perspective.py 49400 2007-10-24 14:22:22Z CJP $"
-__revision__ = "$Revision: 49400 $"
+__cvsid__ = "$Id: perspective.py 51096 2008-01-08 08:03:46Z CJP $"
+__revision__ = "$Revision: 51096 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -28,15 +28,20 @@ import os
 import wx
 import util
 import ed_menu
-from profiler import Profile_Get
+from profiler import Profile_Get, Profile_Set
 
 #--------------------------------------------------------------------------#
 # Globals
+AUTO_PERSPECTIVE = u'Automatic'
 DATA_FILE = u'perspectives'
 LAST_KEY = u'**LASTVIEW**'
-ID_DELETE_PERSPECTIVE = wx.NewId()
-ID_SAVE_PERSPECTIVE = wx.NewId()
 
+# ID's
+ID_SAVE_PERSPECTIVE = wx.NewId()
+ID_DELETE_PERSPECTIVE = wx.NewId()
+ID_AUTO_PERSPECTIVE = wx.NewId()
+
+# Aliases
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
@@ -63,14 +68,18 @@ class PerspectiveManager(object):
         self._base = os.path.join(base, DATA_FILE)  # Path to config
         self._viewset = dict()                      # Set of Views
         self.LoadPerspectives()
-        self._menu = ed_menu.EdMenu()              # Control menu
-        self._currview = None                       # Currently used view
+        self._menu = ed_menu.EdMenu()               # Control menu
+        self._currview = Profile_Get('DEFAULT_VIEW')    # Currently used view
 
         # Setup Menu
         self._menu.Append(ID_SAVE_PERSPECTIVE, _("Save Current View"),
                     _("Save the current window layout"))
         self._menu.Append(ID_DELETE_PERSPECTIVE, _("Delete Saved View"))
         self._menu.AppendSeparator()
+        self._menu.Append(ID_AUTO_PERSPECTIVE, _(AUTO_PERSPECTIVE),
+                          _("Automatically save/use window state from last session"), 
+                          wx.ITEM_CHECK)
+        self._menu.AppendSeparator()
         for name in self._viewset:
             self.AddPerspectiveMenuEntry(name)
 
@@ -79,8 +88,6 @@ class PerspectiveManager(object):
         self._window.SetTransparent(Profile_Get('ALPHA', default=255))
         if Profile_Get('SET_WPOS') and Profile_Get('WPOS', "size_tuple", False):
             self._window.SetPosition(Profile_Get('WPOS'))
-        else:
-            self._window.CenterOnParent()
 
         # Event Handlers
         self._window.Bind(wx.EVT_MENU, self.OnPerspectiveMenu)
@@ -92,20 +99,22 @@ class PerspectiveManager(object):
         @keyword p_data: perspective data from auimgr
 
         """
-        if not len(name):
+        # Don't allow empty keys or ones that override the automatic
+        # settings to be added
+        name = name.strip()
+        if not len(name) or name == AUTO_PERSPECTIVE:
             return
-        if self.HasPerspective(name.strip()):
-            menu = False
-        else:
-            menu = True
 
-        if not p_data:
-            self._viewset[name.strip()] = self._mgr.SavePerspective()
+        domenu = not self.HasPerspective(name)
+        if p_data is None:
+            self._viewset[name] = self._mgr.SavePerspective()
         else:
-            self._viewset[name.strip()] = p_data
-        self._currview = name.strip()
-        if menu:
+            self._viewset[name] = p_data
+        self._currview = name
+
+        if name != AUTO_PERSPECTIVE and domenu:
             self.AddPerspectiveMenuEntry(name)
+
         self.SavePerspectives()
 
     def AddPerspectiveMenuEntry(self, name):
@@ -113,12 +122,14 @@ class PerspectiveManager(object):
         @param name: name of perspective to add to menu
 
         """
-        if not len(name):
+        name = name.strip()
+        if not len(name) or name == AUTO_PERSPECTIVE:
             return
+
         per_id = wx.NewId()
         self._ids.append(per_id)
         self._menu.InsertAlpha(per_id, name, _("Change view to \"%s\"") % name, 
-                               after=ID_DELETE_PERSPECTIVE)
+                               kind=wx.ITEM_CHECK, after=ID_AUTO_PERSPECTIVE)
 
     def GetPerspectiveControls(self):
         """Returns the control menu for the manager
@@ -140,7 +151,15 @@ class PerspectiveManager(object):
 
         """
         return self._viewset.get(name, None)
-            
+
+    def GetPersectiveHandlers(self):
+        """Gets a list of ID to UIHandlers for the perspective Menu
+        @return: list of [(ID, HandlerFunction)]
+
+        """
+        handlers = [(m_id, self.OnUpdatePerspectiveMenu) for m_id in self._ids]
+        return handlers + [(ID_AUTO_PERSPECTIVE, self.OnUpdatePerspectiveMenu)]
+
     def GetPerspectiveList(self):
         """Returns a list of all the loaded perspectives. The
         returned list only provides the names of the perspectives
@@ -148,9 +167,7 @@ class PerspectiveManager(object):
         @return: list of all managed perspectives
 
         """
-        views = self._viewset.keys()
-        views.sort()
-        return views
+        return sorted(self._viewset.keys())
 
     def HasPerspective(self, name):
         """Returns True if there is a perspective by the given name
@@ -169,14 +186,17 @@ class PerspectiveManager(object):
         """
         reader = util.GetFileReader(self._base)
         if reader == -1:
+            util.Log("[perspective][err] Failed to get " +
+                     "file reader for %s" % self._base)
             return 0
 
         try:
             for line in reader.readlines():
                 label, val = line.split(u"=", 1)
-                if not len(label.strip()):
+                label = label.strip()
+                if not len(label):
                     continue
-                self._viewset[label.strip()] = val.strip()
+                self._viewset[label] = val.strip()
             reader.close()
         finally:
             if self._viewset.has_key(LAST_KEY):
@@ -185,40 +205,63 @@ class PerspectiveManager(object):
             return len(self._viewset)
 
     def OnPerspectiveMenu(self, evt):
-        """Handles menu events generated by the managers control
-        menu.
+        """Handles menu events generated by the managers control menu.
         @param evt: event that called this handler
         
         """
         e_id = evt.GetId()
-        if e_id in [ID_SAVE_PERSPECTIVE, ID_DELETE_PERSPECTIVE]:
-            if e_id == ID_SAVE_PERSPECTIVE:
-                name = wx.GetTextFromUser(_("Perspective Name"), \
-                                          _("Save Perspective"))
-                if name:
-                    self.AddPerspective(name, p_data=None)
-                    self.SavePerspectives()
+        if e_id == ID_SAVE_PERSPECTIVE:
+            name = wx.GetTextFromUser(_("Perspective Name"), \
+                                      _("Save Perspective"))
+            if name:
+                self.AddPerspective(name, p_data=None)
+                self.SavePerspectives()
+                Profile_Set('DEFAULT_VIEW', name)
 
-                    for mainw in wx.GetApp().GetMainWindows():
-                        mainw.AddPerspective(name, self._viewset[name])
+                # It may make sense to update all windows to use this
+                # perspective at this point but it may be an unexpected
+                # event to happen when there is many windows open. Will
+                # leave this to future concideration.
+                for mainw in wx.GetApp().GetMainWindows():
+                    mainw.AddPerspective(name, self._viewset[name])
 
-            elif e_id == ID_DELETE_PERSPECTIVE:
-                name = wx.GetSingleChoice(_("Perspective to Delete"), 
-                                          _("Delete Perspective"),
-                                          self._viewset.keys())
-                if name:
-                    self.RemovePerspective(name)
-                    self.SavePerspectives()
-                    for mainw in wx.GetApp().GetMainWindows():
-                        mainw.RemovePerspective(name)
+        elif e_id == ID_DELETE_PERSPECTIVE:
+            views = [ view for view in self._viewset.keys()
+                      if view != AUTO_PERSPECTIVE ]
+            name = wx.GetSingleChoice(_("Perspective to Delete"), 
+                                      _("Delete Perspective"), views)
+
+            if name:
+                self.RemovePerspective(name)
+                self.SavePerspectives()
+                for mainw in wx.GetApp().GetMainWindows():
+                    mainw.RemovePerspective(name)
             else:
                 pass
 
             # Update all windows data sets
             for mainw in wx.GetApp().GetMainWindows():
                 mainw.LoadPerspectives()
+
         elif e_id in self._ids:
-            self.SetPerspectiveById(e_id)
+            if e_id == ID_AUTO_PERSPECTIVE:
+                Profile_Set('DEFAULT_VIEW', AUTO_PERSPECTIVE)
+                self.SetAutoPerspective()
+            else:
+                self.SetPerspectiveById(e_id)
+        else:
+            evt.Skip()
+
+    def OnUpdatePerspectiveMenu(self, evt):
+        """Update the perspective menu's check mark states
+        @param evt: UpdateUI event that called this handler
+
+        """
+        e_id = evt.GetId()
+        if e_id in self._ids + [ID_AUTO_PERSPECTIVE]:
+            evt.Check(self._menu.GetLabel(e_id) == self._currview)
+            evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+            evt.SetUpdateInterval(500)
         else:
             evt.Skip()
 
@@ -233,6 +276,14 @@ class PerspectiveManager(object):
             if rem_id:
                 self._ids.remove(rem_id)
 
+    def SetAutoPerspective(self):
+        """Set the current perspective mangagement into automatic mode
+        @postcondition: window is set into 
+
+        """
+        self._currview = AUTO_PERSPECTIVE
+        self.UpdateAutoPerspective()
+
     def SavePerspectives(self):
         """Writes the perspectives out to disk. Returns
         True if all data was written and False if there
@@ -243,6 +294,7 @@ class PerspectiveManager(object):
         """
         writer = util.GetFileWriter(self._base)
         if writer == -1:
+            util.Log("[perspective][err] Failed to save %s" % self._base)
             return False
 
         try:
@@ -251,6 +303,7 @@ class PerspectiveManager(object):
                 writer.write(u"%s=%s\n" % (perspect, self._viewset[perspect]))
             del self._viewset[LAST_KEY]
         except (IOError, OSError):
+            util.Log("[perspective][err] Write error: %s" % self._base)
             return False
         else:
             return True
@@ -264,14 +317,19 @@ class PerspectiveManager(object):
 
         """
         if self._viewset.has_key(name):
+
+            if name == AUTO_PERSPECTIVE:
+                self._viewset[name] = self._viewset[self._currview]
+            else:
+                self._mgr.LoadPerspective(self._viewset[name])
+                self._mgr.Update()
+
             self._currview = name
-            self._mgr.LoadPerspective(self._viewset[name])
-            self._mgr.Update()
             self.SavePerspectives()
             return True
-        elif name == u"Default":
-            self.AddPerspective(name)
         else:
+            # Fall back to automatic mode
+            self.SetAutoPerspective()
             return False
 
     def SetPerspectiveById(self, per_id):
@@ -282,12 +340,23 @@ class PerspectiveManager(object):
 
         """
         name = None
-        for pos in range(self._menu.GetMenuItemCount()):
+        for pos in xrange(self._menu.GetMenuItemCount()):
             item = self._menu.FindItemByPosition(pos)
             if per_id == item.GetId():
                 name = item.GetLabel()
                 break
-        if name:
+
+        if name is not None:
             return self.SetPerspective(name)
         else:
             return False
+
+    def UpdateAutoPerspective(self):
+        """Update the value of the auto-perspectives current saved state
+        @postcondition: The perspective data for the Automatic setting is
+                        updated to have data for the current state of the
+                        window.
+
+        """
+        self._viewset[AUTO_PERSPECTIVE] = self._mgr.SavePerspective()
+        self.SavePerspectives()
