@@ -20,19 +20,18 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: plugdlg.py 49578 2007-11-01 08:10:04Z CJP $"
-__revision__ = "$Revision: 49578 $"
+__cvsid__ = "$Id: plugdlg.py 50881 2007-12-22 02:42:52Z CJP $"
+__revision__ = "$Revision: 50881 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import sys
 import os
 import re
-import urllib
+import urllib2
 import wx
 import wx.lib.delayedresult as delayedresult
-# import wx.lib.mixins.listctrl as listmix
-import extern.listctrl as listmix
+import wx.lib.mixins.listctrl as listmix
 import ed_glob
 from profiler import Profile_Get, Profile_Set
 import ed_event
@@ -96,9 +95,6 @@ class PluginDialog(wx.Frame):
         wx.Frame.__init__(self, parent, fid, title, pos, size, style)
         util.SetWindowIcon(self)
 
-        if wx.Platform == '__WXMAC__' and Profile_Get('METAL', 'bool', False):
-            self.SetExtraStyle(wx.FRAME_EX_METAL)
-
         # Attributes
         sizer = wx.BoxSizer(wx.VERTICAL)
         self.SetStatusBar(DownloadStatusBar(self))
@@ -692,7 +688,15 @@ def _GetPluginListData(url=PLUGIN_REPO):
     text = u''
     try:
         try:
-            h_file = urllib.urlopen(url)
+            if Profile_Get('USE_PROXY', default=False):
+                proxy_set = Profile_Get('PROXY_SETTINGS',
+                                        default=dict(uname='', url='',
+                                                     port='80', passwd=''))
+                proxy = util.GetProxyOpener(proxy_set)
+                h_file = proxy.open(url)
+            else:
+                h_file = urllib2.urlopen(url)
+
             text = h_file.read()
             h_file.close()
         except (IOError, OSError), msg:
@@ -711,7 +715,15 @@ def _DownloadPlugin(*args):
     egg = None
     try:
         try:
-            h_file = urllib.urlopen(url)
+            if Profile_Get('USE_PROXY', default=False):
+                proxy_set = Profile_Get('PROXY_SETTINGS',
+                                        default=dict(uname='', url='',
+                                                     port='80', passwd=''))
+                proxy = util.GetProxyOpener(proxy_set)
+                h_file = proxy.open(url)
+            else:
+                h_file = urllib2.urlopen(url)
+
             egg = h_file.read()
             h_file.close()
         except (IOError, OSError), msg:
@@ -788,7 +800,7 @@ class InstallPanel(wx.Panel):
             inst_loc = ed_glob.CONFIG['SYS_PLUGIN_DIR']
 
         for item in items:
-            egg_name = item.split("/")[-1]
+            egg_name = item.split(os.sep)[-1]
             if os.path.isabs(item):
                 try:
                     reader = file(item, "rb")
