@@ -20,19 +20,21 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 49575 2007-11-01 07:50:27Z CJP $"
-__revision__ = "$Revision: 49575 $"
+__svnid__ = "$Id: prefdlg.py 51275 2008-01-18 09:53:04Z CJP $"
+__revision__ = "$Revision: 51275 $"
 
 #----------------------------------------------------------------------------#
 # Dependancies
 import wx
 import wx.lib.mixins.listctrl as listmix
+import os
 import sys
 import ed_glob
 import profiler
 from profiler import Profile_Get, Profile_Set
 import ed_i18n
 import ed_event
+import ed_crypt
 import updater
 import util
 import syntax.syntax as syntax
@@ -49,44 +51,20 @@ ID_LATE_BOX     = wx.NewId()
 _ = wx.GetTranslation
 #----------------------------------------------------------------------------#
 # Class Globals
-from wx import ImageFromStream, BitmapFromImage, EmptyIcon
-import cStringIO, zlib
+from extern.embeddedimage import PyEmbeddedImage
 
-def getData():
-    """Gets the data of the button background"""
-    return zlib.decompress(
-'x\xda\x01A\x02\xbe\xfd\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00 \x00\
-\x00\x00 \x08\x06\x00\x00\x00szz\xf4\x00\x00\x00\x04sBIT\x08\x08\x08\x08|\
-\x08d\x88\x00\x00\x01\xf8IDATX\x85\xed\x97?n\x1aA\x14\x87\xbf\xd9Y\xb0\xdd\
-\x10\x05\x87\xd2[Q`"\x91\x8a\x03P\xc6\x11R\xdcE.R9F\xb2\x94\x0b87\xc8\x01\\D\
-\x91S\xc5\xce\t\x9c.\x17\xa0\n\x8a%\xba\x95\x90\xe8b\x1cX\x8b\xc4\xec<&\x05\
-\x7fb\xc0`b0n\xf8I[\xcc\xe8\xcd\xfb}\xf3\xdehF\xab\x18T\x1cx\x96\xc9d\x0e\
-\xb5\xd6)\xe6(\x11)\x97J\xa5}\xe0;P\xeb\xcd\xabk1\xbb\xa9T*\x97\xcdfw\xb6\
-\x9eo\xb1\xfed}\x9e\xfe\x9c\xff<\xe7\xf4\xeb)\xc5b\xf1\xb8\\.\xbf\x06\xa4\
-\x07\xe0D"\x917\x85\xbd\xc2\xfbd2\x19\xf3<o\xae\xc6\xc3\xaaT*\x1c\xbc;\xf8\
-\xd0l6\xf7\x01Q@n\xfb\xe5\xf6\xb7|>\x7f\xaf\xc6\xd7U\xadV9\xfatt\xec\xfb\xfe\
-[wc\xc3\xdb\xf3<\x8f \xb8\x04\xecB\x00b\xb1Gln\xa6w|\xdf\xff\xe8F\\\xf7U"\
-\x91 \x08\x1a\xb7.\xb4\xd6b\xadE)\x85R\xea\xd6\xf8Iz\x9aNsv\xf6\xe3\xd05\xc6\
-P\xffU\x9fl\xdc\xad\x8cb6\xd3\x91\xbcm\x9br\x8d\x18\x1aS\xec~|\x16\xb8+\x97\
-\x11\x83+"4\x1a\xc1\xdd\x01f\x90\x88\xd0iA}\xb0\x05\x137d\xa7\t\x9a\xbc\xb4\
-\'c\x0c\xae\x18\xe12x\xa0\n\x18\xc1\r\x8d\xa1V\xbb@k=\xf3\xc9\x9eV\xd6ZD\x84\
-\xd0\x18\x14\xdd\xca\xc4\x1f\xc7\x89F\xa2\xfd \xad5\xd1\xe8\xca\\\x0c[\xad+D\
-\xe4\xdf8lQ\xbb\xe8<\x07}\x80ai\xadY\x89\xae\x0e\xcc)\xa5X[]\xc3q\x9c\x91\
-\xf8\xb6m\x13\x86!\xc6\x84\xd8\xa1\x8cW\xad?\x03\x00\x039\xc7\x01\x8c\x93\
-\xab#@\xe7B\x1a\xa2\xc3ZK\xbb}\xb3\xd18\xfd7\xc0\xbc5Z\xcb%\xc0\x12`\t\xb0\
-\x04X\x02,V/\x1c\xe0\xcb\x03\x02\xfc\x06\xc8\xd1y\x90\x16\xfd}\xa6\xf3/\x8a\
-\x03\x14\x80\xfa\x02\xcd\xeb\xc0\xeep9\n\x0b\x04(\xdc\xd4\x0f\xa7\xdb\x8e\
-\x93{4>\xe9z\xf4\x0f\xff_\x17\xe2 \x06n\xab\xb1\xf7\x00\x00\x00\x00IEND\xaeB\
-`\x82\x11\x87\x03\xa2' )
-
-def getBitmap():
-    """Gets the bitmap of the button base"""
-    return BitmapFromImage(getImage())
-
-def getImage():
-    """Gets the image of the button base"""
-    stream = cStringIO.StringIO(getData())
-    return ImageFromStream(stream)
+ButtonBackground = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAAfhJ"
+    "REFUWIXtlz9uGkEUh7/ZWbDdEAWH0ltRYCKRigNQxhFS3EUuUjlGspQLODfIAVxEkVPFzgmc"
+    "LhegCoolupWQ6GIcWIvE7DwmBX9iwGBiMG74SVvM6M37ffPeaEarGFQceJbJZA611inmKBEp"
+    "l0qlfeA7UOvNq2sxu6lUKpfNZne2nm+x/mR9nv6c/zzn9OspxWLxuFwuvwakB+BEIpE3hb3C"
+    "+2QyGfM8b67Gw6pUKhy8O/jQbDb3AVFAbvvl9rd8Pn+vxtdVrVY5+nR07Pv+W3djw9vzPI8g"
+    "uATsQgBisUdsbqZ3fN//6EZc91UikSAIGrcutNZirUUphVLq1vhJeppOc3b249A1xlD/VZ9s"
+    "3K2MYjbTkbxtm3KNGBpT7H58FrgrlxGDKyI0GsHdAWaQiNBpQX2wBRM3ZKcJmry0J2MMrhjh"
+    "MnigChjBDY2hVrtAaz3zyZ5W1lpEhNAYFN3KxB/HiUai/SCtNdHoylwMW60rROTfOGxRu+g8"
+    "B32AYWmtWYmuDswppVhbXcNxnJH4tm0ThiHGhNihjFetPwMAAznHAYyTqyNA50IaosNaS7t9"
+    "s9E4/TfAvDVayyXAEmAJsARYAixWLxzgywMC/AbI0XmQFv19pvMvigMUgPoCzevA7nA5CgsE"
+    "KNzUD6fbjpN7ND7pevQP/18X4iAGbqux9wAAAABJRU5ErkJggg==")
 
 def MakeThemeTool(tool_id):
     """Makes a themed bitmap for the tool book of the pref dialog.
@@ -98,7 +76,7 @@ def MakeThemeTool(tool_id):
     Profile_Set('ICON_SZ', (24, 24))
     over = wx.ArtProvider.GetBitmap(str(tool_id), wx.ART_TOOLBAR)
     Profile_Set('ICON_SZ', osize)
-    mdc = wx.MemoryDC(getBitmap())
+    mdc = wx.MemoryDC(ButtonBackground.GetBitmap())
     if over.IsOk():
         # Create overlay
         if over.GetSize() != (24, 24):
@@ -132,8 +110,6 @@ class PreferencesDialog(wx.Frame):
 
         # Extra Styles
         self.SetTransparent(Profile_Get('ALPHA', 'int', 255))
-        if wx.Platform == '__WXMAC__' and Profile_Get('METAL', 'bool', False):
-            self.SetExtraStyle(wx.DIALOG_EX_METAL)
 
         # Attributes
         self._tbook = PrefTools(self)
@@ -155,7 +131,14 @@ class PreferencesDialog(wx.Frame):
         @param evt: Event that called this handler
 
         """
-        wx.GetApp().UnRegisterWindow(repr(self))
+        # XXX More strange wx is None errors have been reported here
+        #     really need to find the cause of this!
+        if wx is not None:
+            wx.GetApp().UnRegisterWindow(repr(self))
+
+        # Save profile settings
+        profiler.Profile().Write(profiler.Profile_Get('MYPROFILE'))
+
         evt.Skip()
 
     def OnShow(self, evt):
@@ -207,7 +190,7 @@ class PrefTools(wx.Toolbook):
                      imageId=self.APPEAR_PG)
         self.AddPage(DocumentPanel(self), _("Document"), 
                      imageId=self.DOC_PG)
-        self.AddPage(UpdatePanel(self), _("Update"),
+        self.AddPage(NetworkPanel(self), _("Network"),
                      imageId=self.UPDATE_PG)
 #         self.AddPage(PrefPanelBase(self), _("Advanced"),
 #                      imageId=self.ADV_PG)
@@ -221,7 +204,7 @@ class PrefTools(wx.Toolbook):
         @todo: animate the resizing so its smoother
 
         """
-        self.LOG("[prefdlg][toolbook][evt] page changed")
+        self.LOG("[prefdlg][evt] toolbook page changed")
         page = self.GetSelection()
         page = self.GetPage(page)
         page.SetInitialSize()
@@ -250,7 +233,7 @@ class PrefTools(wx.Toolbook):
         dc = wx.PaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
         col1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
-        col2 = util.AdjustColour(col1, -20)
+        col2 = util.AdjustColour(col1, -10)
         rect = self.GetToolBar().GetRect()
 
         # Create the background path
@@ -264,6 +247,8 @@ class PrefTools(wx.Toolbook):
 
         evt.Skip()
 
+#-----------------------------------------------------------------------------#
+
 class PrefPanelBase(wx.Panel):
     """Base of all preference panels
     @summary: Provides a panel with a painted background
@@ -278,7 +263,8 @@ class PrefPanelBase(wx.Panel):
                                               wx.SUNKEN_BORDER)
 
         # Event Handlers
-        self.Bind(wx.EVT_PAINT, self.OnPaint)
+        if wx.Platform == '__WXMAC__':
+            self.Bind(wx.EVT_PAINT, self.OnPaint)
 
     def OnPaint(self, evt):
         """Paints the panels background
@@ -295,20 +281,21 @@ class PrefPanelBase(wx.Panel):
         dc = wx.PaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
         col1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
-        col1 = util.AdjustColour(col1, 30)
-        brush = gc.CreateBrush(wx.Brush(col1))
+        brush = gc.CreateBrush(wx.Brush(util.AdjustColour(col1, 5)))
         rect = self.GetRect()
 
         # Create the background path
         path = gc.CreatePath()
         path.AddRectangle(0, 0, rect.width, rect.height)
 
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -60), 1))
+        gc.SetPen(wx.Pen(util.AdjustColour(col1, -10), 1))
         gc.SetBrush(brush)
         gc.DrawPath(path)
 
         evt.Skip()
 
+#-----------------------------------------------------------------------------#
+
 class GeneralPanel(PrefPanelBase):
     """Creates a panel with controls for Editra's general settings
     @summary: Panel with a number of controls that affect the users
@@ -345,17 +332,21 @@ class GeneralPanel(PrefPanelBase):
         start_lbl = wx.StaticText(self, label=_("Startup Settings") + u": ")
         mode_lbl = wx.StaticText(self, label=_("Editor Mode") + u": ")
         mode_ch = ExChoice(self, ed_glob.ID_PREF_MODE,
-                           choices=['CODE', 'DEBUG', 'GUI_DEBUG'],
+                           choices=['CODE', 'DEBUG'],
                            default=Profile_Get('MODE'))
         msizer = wx.BoxSizer(wx.HORIZONTAL)
-        msizer.AddMany([(mode_lbl, 0), ((5, 5), 0), (mode_ch, 0)])
+        msizer.AddMany([(mode_lbl, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
+                        (mode_ch, 0, wx.ALIGN_CENTER_VERTICAL)])
+
         pmode_lbl = wx.StaticText(self, label=_("Printer Mode") + u": ")
         pmode_ch = ExChoice(self, ed_glob.ID_PRINT_MODE,
                             choices=['Black/White', 'Colour/White', 
                                      'Colour/Default', 'Inverse', 'Normal'],
                             default=Profile_Get('PRINT_MODE'))
         psizer = wx.BoxSizer(wx.HORIZONTAL)
-        psizer.AddMany([(pmode_lbl, 0), ((5, 5), 0), (pmode_ch, 0)])
+        psizer.AddMany([(pmode_lbl, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
+                        (pmode_ch, 0, wx.ALIGN_CENTER_VERTICAL)])
+
         reporter_cb = wx.CheckBox(self, ed_glob.ID_REPORTER, 
                                   _("Disable Error Reporter"))
         reporter_cb.SetValue(not Profile_Get('REPORTER'))
@@ -375,7 +366,9 @@ class GeneralPanel(PrefPanelBase):
                          choices=['1','2','3','4','5','6','7','8','9'],
                          default=Profile_Get('FHIST_LVL', 'str'))
         fhsizer = wx.BoxSizer(wx.HORIZONTAL)
-        fhsizer.AddMany([(fh_lbl, 0), ((5, 5), 0), (fh_ch, 0)])
+        fhsizer.AddMany([(fh_lbl, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
+                         (fh_ch, 0, wx.ALIGN_CENTER_VERTICAL)])
+
         win_cb = wx.CheckBox(self, ed_glob.ID_NEW_WINDOW,
                              _("Open files in new windows by default"))
         win_cb.SetValue(Profile_Get('OPEN_NW'))
@@ -393,22 +386,23 @@ class GeneralPanel(PrefPanelBase):
         lang_lbl = wx.StaticText(self, label=_("Language") + u": ")
         lang_c = ed_i18n.LangListCombo(self, ed_glob.ID_PREF_LANG, 
                                        Profile_Get('LANG'))
-        lsizer.AddMany([(lang_lbl, 0), ((5, 5), 0), (lang_c, 0)])
+        lsizer.AddMany([(lang_lbl, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
+                        (lang_c, 0, wx.ALIGN_CENTER_VERTICAL)])
 
         # Layout items
         sizer = wx.GridBagSizer(5, 5)
         sizer.AddMany([((5, 5), (0, 0)),
-                       (start_lbl, (1, 1)),
+                       (start_lbl, (1, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL),
                        (msizer, (1, 2), (1, 2)),
                        (psizer, (2, 2), (1, 2)),
                        (reporter_cb, (3, 2), (1, 2)),
                        (session_cb, (4, 2), (1, 2)),
-                       (splash_cb, (5, 2), (1, 2))])
-        sizer.AddMany([(file_lbl, (7, 1)),
+                       (splash_cb, (5, 2), (1, 2)),
+                       (file_lbl, (7, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL),
                        (fhsizer, (7, 2), (1, 2)), (win_cb, (8, 2), (1, 3)),
                        (pos_cb, (9, 2), (1, 3)),
-                       (chkmod_cb, (10, 2), (1, 2))])
-        sizer.AddMany([(locale, (12, 1)),
+                       (chkmod_cb, (10, 2), (1, 2)),
+                       (locale, (12, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL),
                        (lsizer, (12, 2), (1, 3)),
                        ((15, 15), (13, 0))])
         self.SetSizer(sizer)
@@ -418,7 +412,7 @@ class GeneralPanel(PrefPanelBase):
         @param evt: event that called this handler
 
         """
-        self.LOG("[prefdlg][gen][evt] Check box clicked")
+        self.LOG("[prefdlg][evt] General Page: Check box clicked")
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
         if e_id in [ed_glob.ID_APP_SPLASH, ed_glob.ID_PREF_SPOS,
@@ -437,7 +431,7 @@ class GeneralPanel(PrefPanelBase):
         @note: Also handles the Language ComboBox
 
         """
-        self.LOG("[prefldg][gen][evt] Choice event caught")
+        self.LOG("[prefldg][evt] General Page: Choice event caught")
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
         if e_id in [ed_glob.ID_PREF_MODE, ed_glob.ID_PRINT_MODE,
@@ -448,6 +442,8 @@ class GeneralPanel(PrefPanelBase):
         else:
             evt.Skip()
 
+#-----------------------------------------------------------------------------#
+
 class DocumentPanel(PrefPanelBase):
     """Creates a panel with controls for Editra's editing settings
     @summary: Conatains a L{wx.Notebook} that contains a number of pages
@@ -560,15 +556,17 @@ class DocGenPanel(wx.Panel):
         sizer = wx.GridBagSizer(5, 5)
         sizer.Add((5, 5), (1, 0))
         sizer.Add(wx.StaticText(self, label=_("Format") + u": "), (1, 1))
-        sizer.AddMany([(ut_cb, (1, 2), wx.GBSpan(1, 2)),
-                       (tabw_lbl, (2, 2)), (tw_ch, (2, 3), wx.GBSpan(1, 3)),
-                       (eol_lbl, (3, 2)), (eol_ch, (3, 3), wx.GBSpan(1, 2)),
+        sizer.AddMany([(ut_cb, (1, 2), (1, 2)),
+                       (tabw_lbl, (2, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (tw_ch, (2, 3), (1, 3), wx.ALIGN_CENTER_VERTICAL),
+                       (eol_lbl, (3, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (eol_ch, (3, 3), (1, 2), wx.ALIGN_CENTER_VERTICAL),
                        ((5, 5), (5, 0)), (view_lbl, (5, 1)),
                        (aa_cb, (5, 2)), (seol_cb, (6, 2)), (sln_cb, (7, 2)),
                        (sws_cb, (8, 2)), (ww_cb, (9, 2)),
-                       (font_lbl, (11, 1)), 
+                       (font_lbl, (11, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL), 
                        (fpicker, (11, 2), (1, 3), wx.EXPAND), 
-                       (font_lbl2, (12, 1)),
+                       (font_lbl2, (12, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL),
                        (fpicker2, (12, 2), (1, 3), wx.EXPAND),
                        ((5, 5), (13, 0))
                        ])
@@ -600,6 +598,15 @@ class DocGenPanel(wx.Panel):
         @param evt: Event that called this handler
 
         """
+        # XXX why is happening in this dialog on windows this and other
+        #     imports randomly become None. I have been unable to reproduce
+        #     this behavior myself but have recieved enough error reports about
+        #     it to beleive it. If they were actually NoneTypes the dialog would
+        #     not be able to be shown so this is very strange!!
+        global ed_glob
+        if ed_glob is None:
+            import ed_glob
+
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
         if e_id in [ed_glob.ID_PREF_TABS, ed_glob.ID_PREF_TABW,
@@ -631,7 +638,7 @@ class DocCodePanel(wx.Panel):
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_CHOICE, self.OnCheck)
-        self.Bind(wx.EVT_SLIDER, self.OnSlide)
+        self.Bind(wx.EVT_SPINCTRL, self.OnSpin)
 
     def _DoLayout(self):
         """Layout the page
@@ -648,15 +655,11 @@ class DocCodePanel(wx.Panel):
         edge_cb = wx.CheckBox(self, ed_glob.ID_SHOW_EDGE, _("Edge Guide"))
         edge_cb.SetValue(Profile_Get('SHOW_EDGE'))
 
-        edge_size = (-1, -1)
-        if wx.Platform == '__WXGTK__':
-            edge_size = (150, 15)
-        edge_sl = wx.Slider(self, ed_glob.ID_PREF_EDGE, Profile_Get('EDGE'),
-                            0, 100, size=edge_size, style=wx.SL_HORIZONTAL | \
-                            wx.SL_AUTOTICKS | wx.SL_LABELS)
-        edge_sl.SetTickFreq(5, 1)
-        if wx.Platform == '__WXMAC__':
-            edge_sl.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
+        edge_sp = wx.SpinCtrl(self, ed_glob.ID_PREF_EDGE,
+                              Profile_Get('EDGE', 'str'), min=0, max=120)
+        edge_col = wx.BoxSizer(wx.HORIZONTAL)
+        edge_col.AddMany([(wx.StaticText(self, label=_("Column") + u": "), 0, wx.ALIGN_CENTER_VERTICAL),
+                          ((5, 5), 0), (edge_sp, 0, wx.ALIGN_CENTER_VERTICAL)])
         ind_cb = wx.CheckBox(self, ed_glob.ID_INDENT_GUIDES, 
                              _("Indentation Guides"))
         ind_cb.SetValue(Profile_Get('GUIDES'))
@@ -674,9 +677,9 @@ class DocCodePanel(wx.Panel):
         sizer.Add((5, 5), (1, 0))
         sizer.Add(vis_lbl, (1, 1))
         sizer.AddMany([(br_cb, (1, 2)), (fold_cb, (2, 2)),
-                       (edge_cb, (3, 2)),
-                       (wx.StaticText(self, label=_("Column") + u": "), (3, 3)),
-                       (edge_sl, (3, 4), (2, 2), wx.EXPAND), (ind_cb, (4, 2))])
+                       (edge_cb, (3, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (edge_col, (3, 4), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (ind_cb, (4, 2))])
         sizer.Add(wx.StaticText(self, label=_("Input Helpers") + u": "), (6, 1))
         sizer.AddMany([(comp_cb, (6, 2)), (ai_cb, (7, 2)), (vi_cb, (8, 2))])
         self.SetSizer(sizer)
@@ -699,7 +702,7 @@ class DocCodePanel(wx.Panel):
         else:
             evt.Skip()
 
-    def OnSlide(self, evt):
+    def OnSpin(self, evt):
         """Catch actions from a slider
         @param evt: Event that called this handler.
 
@@ -763,9 +766,10 @@ class DocSyntaxPanel(wx.Panel):
 
         # Layout the controls
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
-        hsizer.AddMany([(syn_cb, 0, wx.ALIGN_LEFT), ((5, 5), 1, wx.EXPAND),
-                        (wx.StaticText(self, label=_("Color Scheme") + u": "), \
-                        0, wx.ALIGN_LEFT),
+        hsizer.AddMany([(syn_cb, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
+                        ((5, 5), 1, wx.EXPAND),
+                        (wx.StaticText(self, label=_("Color Scheme") + u": "),
+                        0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                         (syntheme, 0, wx.EXPAND), ((5, 5), 0)])
 
         sizer.AddMany([((15, 15), 0), 
@@ -775,18 +779,14 @@ class DocSyntaxPanel(wx.Panel):
                        ((15, 15), 0),
                        (lst_lbl, 0, wx.ALIGN_LEFT)])
 
-
         hsizer2 = wx.BoxSizer(wx.HORIZONTAL)
-        hsizer2.Add((5, 5), 0)
-        hsizer2.Add(self._elist, 1, wx.EXPAND)
-        hsizer2.Add((5, 5), 0)
+        hsizer2.AddMany([((5, 5), 0), (self._elist, 1, wx.EXPAND), ((5, 5), 0)])
 
-        sizer.Add((10, 10), 0)
-        sizer.Add(hsizer2, 1, wx.EXPAND)
-        sizer.Add((20, 20), 0)
-        sizer.Add(wx.Button(self, wx.ID_DEFAULT, 
-                  _("Revert to Default")), 0, wx.ALIGN_LEFT)
-        sizer.Add((10, 10), 0)
+        default_btn = wx.Button(self, wx.ID_DEFAULT,_("Revert to Default"))
+        if wx.Platform == '__WXMAC__':
+            default_btn.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
+        sizer.AddMany([((10, 10), 0), (hsizer2, 1, wx.EXPAND), ((15, 15), 0),
+                       (default_btn, 0, wx.ALIGN_LEFT), ((10, 10), 0)])
 
         msizer = wx.BoxSizer(wx.HORIZONTAL)
         msizer.AddMany([((5, 5), 0), (sizer, 1, wx.EXPAND), ((5, 5), 0)])
@@ -813,14 +813,24 @@ class DocSyntaxPanel(wx.Panel):
         """
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
-        if e_id in [ed_glob.ID_SYNTAX, ed_glob.ID_PREF_SYNTHEME]:
+        if e_id == ed_glob.ID_PREF_SYNTHEME:
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetValue())
 
             for mainw in wx.GetApp().GetMainWindows():
                 mainw.nb.UpdateTextControls()
+
+        elif e_id == ed_glob.ID_SYNTAX:
+            val = e_obj.GetValue()
+            Profile_Set(ed_glob.ID_2_PROF[e_id], val)
+
+            for mainw in wx.GetApp().GetMainWindows():
+                mainw.nb.UpdateTextControls('SyntaxOnOff', [val])
+
         else:
             evt.Skip()
 
+#-----------------------------------------------------------------------------#
+
 class AppearancePanel(PrefPanelBase):
     """Creates a panel with controls for Editra's appearance settings
     @summary: contains all the controls for configuring the appearance
@@ -852,19 +862,17 @@ class AppearancePanel(PrefPanelBase):
         from ed_theme import BitmapProvider
         icons = ['Default']
         icons.extend(BitmapProvider(wx.GetApp().GetPluginManager()).GetThemes())
-        tb_icont = wx.StaticText(self, wx.ID_ANY, _("Icon Theme") + u": ")
+        tb_icont = wx.StaticText(self, label=_("Icon Theme") + u": ")
         tb_icon = ExChoice(self, ed_glob.ID_PREF_ICON,
                             choices=icons, 
                             default=Profile_Get('ICONS', 'str').title())
-        tb_isz_lbl = wx.StaticText(self, wx.ID_ANY, \
-                                   _("Toolbar Icon Size") + u": ")
+        tb_isz_lbl = wx.StaticText(self, label=_("Toolbar Icon Size") + u": ")
         tb_isz_ch = ExChoice(self, ed_glob.ID_PREF_ICONSZ,
                               choices=['16', '24', '32'],
                               default=str(Profile_Get('ICON_SZ', 'size_tuple')[0]))
 
         # Layout Section
-        perspect_lbl = wx.StaticText(self, wx.ID_ANY, \
-                                     _("Default Perspective") + u": ")
+        perspect_lbl = wx.StaticText(self, label=_("Default Perspective") + u": ")
         mainw = wx.GetApp().GetActiveWindow()
         if mainw is not None:
             pchoices = mainw.GetPerspectiveList()
@@ -892,33 +900,25 @@ class AppearancePanel(PrefPanelBase):
         tsizer = wx.BoxSizer(wx.HORIZONTAL)
         tsizer.AddMany([(trans_lbl, 0), ((5, 5), 0), (trans, 0)])
 
-        # Activate Metal Style for OSX
-        if wx.Platform == '__WXMAC__':
-            m_cb = wx.CheckBox(self, ed_glob.ID_PREF_METAL, \
-                               _("Use Metal Style"))
-            m_cb.SetValue(Profile_Get('METAL', 'bool', False))
-            m_cb.SetToolTip(wx.ToolTip(_("Restart Required")))
-        else:
-            m_cb = (0, 0)
-
         # Layout
         sizer = wx.GridBagSizer(5, 4)
-        sizer.AddMany([((5, 5), (0, 1)),
-                       (wx.StaticText(self, label=_("Icons") + u": "), (1, 1)),
-                       (tb_icont, (1, 2)), (tb_icon, (1, 3)),
-                       (tb_isz_lbl, (2, 2)), (tb_isz_ch, (2, 3)),
+        sizer.AddMany([((2, 2), (0, 1)),
+                       (wx.StaticText(self, label=_("Icons") + u": "), (1, 1),
+                        (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (tb_icont, (1, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (tb_icon, (1, 3), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (tb_isz_lbl, (2, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (tb_isz_ch, (2, 3), (1, 1), wx.ALIGN_CENTER_VERTICAL),
                        ((5, 5), (4, 0)),
-                       (wx.StaticText(self, label=_("Layout") + u": "), (4, 1))
+                       (wx.StaticText(self, label=_("Layout") + u": "), (4, 1),
+                        (1, 1), wx.ALIGN_CENTER_VERTICAL)
                       ])
-        sizer.AddMany([(perspect_lbl, (4, 2)),
-                       (perspect_ch, (4, 3), (1, 2)),
-                       (ws_cb, (5, 2), (1, 2)), 
-                       (wp_cb, (6, 2), (1, 2))])
-        sizer.Add((5, 5), (8, 0))
-        sizer.Add(wx.StaticText(self, label=_("Misc") + u": "), (8, 1))
-        sizer.AddMany([(tsizer, (8, 2), (1, 2)),
-                       (m_cb, (9, 2), (1, 2)),
-                       ((15, 15), (10, 2))])
+        sizer.AddMany([(perspect_lbl, (4, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL),
+                       (perspect_ch, (4, 3), (1, 2), wx.ALIGN_CENTER_VERTICAL),
+                       (ws_cb, (5, 2), (1, 2)), (wp_cb, (6, 2), (1, 2)),
+                       ((5, 5), (8, 0)),
+                       (wx.StaticText(self, label=_("Misc") + u": "), (8, 1)),
+                       (tsizer, (8, 2), (1, 2)), ((15, 15), (10, 2))])
         self.SetSizer(sizer)
 
     def OnCheck(self, evt):
@@ -930,8 +930,6 @@ class AppearancePanel(PrefPanelBase):
         val = evt.GetEventObject().GetValue()
         if e_id in [ed_glob.ID_PREF_WPOS, ed_glob.ID_PREF_WSIZE]:
             Profile_Set(ed_glob.ID_2_PROF[e_id], val)
-        elif e_id == ed_glob.ID_PREF_METAL:
-            Profile_Set(ed_glob.ID_2_PROF[e_id], val)
         else:
             evt.Skip()
 
@@ -954,14 +952,15 @@ class AppearancePanel(PrefPanelBase):
                    (toolbar.GetToolTheme() != Profile_Get('ICONS')) \
                    or (toolbar.GetToolBitmapSize() != Profile_Get('ICON_SZ')):
                     toolbar.ReInit()
-                mainw.UpdateToolBar()
 
-                # Update Search Bar
-                # TODO for some reason this doesnt work with 2.8.5
+                # Update Commandbar icons
                 sbar = mainw.FindWindowById(ed_glob.ID_COMMAND_BAR)
                 if sbar:
                     sbar.UpdateIcons()
 
+                # Update Notebook Images
+                mainw.GetNotebook().UpdateAllImages()
+
         elif e_id == ed_glob.ID_PERSPECTIVES:
             Profile_Set('DEFAULT_VIEW', e_obj.GetValue())
             for main_win in wx.GetApp().GetMainWindows():
@@ -986,7 +985,167 @@ class AppearancePanel(PrefPanelBase):
         else:
             evt.Skip()
 
-class UpdatePanel(PrefPanelBase):
+#-----------------------------------------------------------------------------#
+
+class NetworkPanel(PrefPanelBase):
+    """Network related configration options"""
+    def __init__(self, parent):
+        """Create the panel"""
+        PrefPanelBase.__init__(self, parent)
+
+        # Attributes
+        self._nb = wx.Notebook(self)
+        self.SetAutoLayout(True)
+
+        # Layout
+        self.__DoLayout()
+
+    def __DoLayout(self):
+        """Do the layout of the panel
+        @note: Do not call this after __init__
+
+        """
+        sizer = wx.GridBagSizer()
+        self._nb.AddPage(NetConfigPage(self._nb), _("Configuration"))
+        self._nb.AddPage(UpdatePage(self._nb), _("Update"))
+        sizer.Add(self._nb, (1, 1))
+        sizer.Add((5, 5), (2, 2))
+        self.SetSizer(sizer)
+
+#-----------------------------------------------------------------------------#
+
+ID_USE_PROXY = wx.NewId()
+ID_URL = wx.NewId()
+ID_PORT = wx.NewId()
+ID_USERNAME = wx.NewId()
+ID_PASSWORD = wx.NewId()
+
+class NetConfigPage(wx.Panel):
+    """Configuration page for network and proxy settings"""
+    def __init__(self, parent):
+        wx.Panel.__init__(self, parent)
+        
+        # Attributes
+        
+        # Layout
+        self.__DoLayout()
+
+        # Event Handlers
+        self.Bind(wx.EVT_CHECKBOX, self.OnCheck, id=ID_USE_PROXY)
+        self.Bind(wx.EVT_BUTTON, self.OnApply, id=wx.ID_APPLY)
+
+    def __DoLayout(self):
+        """Layout the controls in the panel"""
+        msizer = wx.BoxSizer(wx.VERTICAL)
+
+        sbox = wx.StaticBox(self, label=_("Proxy Settings"))
+        sboxsz = wx.StaticBoxSizer(sbox, wx.VERTICAL)
+        flexg = wx.FlexGridSizer(4, 2, 10, 5)
+        flexg.AddGrowableCol(1, 1)
+
+        proxy_val = Profile_Get('PROXY_SETTINGS', default=dict())
+        use_proxy = wx.CheckBox(self, ID_USE_PROXY, _("Use Proxy"))
+        use_proxy.SetValue(Profile_Get('USE_PROXY', 'bool', False))
+        sboxsz.Add(use_proxy, 0, wx.ALIGN_LEFT)
+        sboxsz.Add((10, 10), 0)
+
+        url_sz = wx.BoxSizer(wx.HORIZONTAL)
+        url_lbl = wx.StaticText(self, label=_("Proxy URL") + u":")
+        url_txt = wx.TextCtrl(self, ID_URL, proxy_val.get('url', ''))
+        port_sep = wx.StaticText(self, label=":")
+        port_txt = wx.TextCtrl(self, ID_PORT, proxy_val.get('port', ''))
+        port_txt.SetToolTipString(_("Port Number"))
+        url_sz.AddMany([(url_txt, 1, wx.EXPAND), ((2, 2)),
+                        (port_sep, 0, wx.ALIGN_CENTER_VERTICAL),
+                        ((2, 2)), (port_txt, 0, wx.ALIGN_CENTER_VERTICAL)])
+        flexg.Add(url_lbl, 0, wx.ALIGN_CENTER_VERTICAL)
+        flexg.Add(url_sz, 0, wx.EXPAND)
+
+        usr_sz = wx.BoxSizer(wx.HORIZONTAL)
+        usr_lbl = wx.StaticText(self, label=_("Username") + u":")
+        usr_txt = wx.TextCtrl(self, ID_USERNAME, proxy_val.get('uname', ''))
+        usr_sz.Add(usr_txt, 1, wx.EXPAND)
+        flexg.Add(usr_lbl, 0, wx.ALIGN_CENTER_VERTICAL)
+        flexg.Add(usr_sz, 0, wx.EXPAND)
+
+        pass_sz = wx.BoxSizer(wx.HORIZONTAL)
+        pass_lbl = wx.StaticText(self, label=_("Password") + u":")
+        pass_txt = wx.TextCtrl(self, ID_PASSWORD,
+                               ed_crypt.Decrypt(proxy_val.get('passwd', ''),
+                                                proxy_val.get('pid', '')),
+                               style=wx.TE_PASSWORD)
+        pass_sz.Add(pass_txt, 1, wx.EXPAND)
+        flexg.Add(pass_lbl, 0, wx.ALIGN_CENTER_VERTICAL)
+        flexg.Add(pass_sz, 0, wx.EXPAND)
+
+        flexg.Add((5, 5))
+        apply_b = wx.Button(self, wx.ID_APPLY)
+        flexg.Add(apply_b, 0, wx.ALIGN_RIGHT)
+
+        if wx.Platform == '__WXMAC__':
+            for lbl in (use_proxy, url_txt, port_sep, port_txt, url_lbl, 
+                        usr_lbl, usr_txt, pass_lbl, pass_txt, apply_b):
+                lbl.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
+
+        self.EnableControls(use_proxy.GetValue())
+        sboxsz.Add(flexg, 1, wx.EXPAND)
+        hsizer = wx.BoxSizer(wx.HORIZONTAL)
+        hsizer.AddMany([((5, 5)), (sboxsz, 1, wx.EXPAND), ((5, 5))])
+        msizer.AddMany([((10, 10)), (hsizer, 1, wx.EXPAND), ((10, 10))])
+        self.SetSizer(msizer)
+
+    def OnApply(self, evt):
+        """Apply the changes to the proxy settings
+        @param evt: wx.EVT_BUTTON
+
+        """
+        if evt.GetId() == wx.ID_APPLY:
+            key_map = { ID_USERNAME : 'uname', ID_URL : 'url',
+                        ID_PORT : 'port', ID_PASSWORD : 'passwd' }
+            proxy_dict = dict(uname='', passwd='', url='', port='')
+            for val in (ID_URL, ID_PORT, ID_USERNAME, ID_PASSWORD):
+                win = self.FindWindowById(val)
+                if win is not None:
+                    winval = win.GetValue()
+                    if val == ID_PASSWORD:
+                        # This is not the most secure method of saving a
+                        # sensitive data but it is definitely better than plain
+                        # text. ALso as to be able to obtain this info from the
+                        # user profile the intruder would already have had to
+                        # compromise the users system account making anymore
+                        # such security rather moot by that point.
+                        pid = os.urandom(8)
+                        winval = ed_crypt.Encrypt(winval, pid)
+                        proxy_dict['pid'] = pid
+                    proxy_dict[key_map[val]] = winval
+
+            Profile_Set('PROXY_SETTINGS', proxy_dict)
+        else:
+            evt.Skip()
+
+    def EnableControls(self, enable=True):
+        """Enable the controls in the box or disable them
+        @keyword enable: Enable or Disable
+
+        """
+        for child in self.GetChildren():
+            if isinstance(child, wx.StaticText) or \
+               isinstance(child, wx.TextCtrl) or \
+               isinstance(child, wx.Button):
+                child.Enable(enable)
+
+    def OnCheck(self, evt):
+        """Enable the use of the proxy settings or not
+        @param evt: wx.EVT_CHECKBOX
+
+        """
+        e_val = evt.GetEventObject().GetValue()
+        Profile_Set('USE_PROXY', e_val)
+        self.EnableControls(e_val)
+
+#-----------------------------------------------------------------------------#
+
+class UpdatePage(wx.Panel):
     """Creates a panel with controls for updating Editra
     @summary: Panel with three controls, L{updater.UpdateProgress} bar,
               and two L{wx.Button} for checking and downloading updates.
@@ -997,7 +1156,7 @@ class UpdatePanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
         
         # Attributes
         self.LOG = wx.GetApp().GetLog()
@@ -1053,7 +1212,7 @@ class UpdatePanel(PrefPanelBase):
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
         if e_id == ID_CHECK_UPDATE:
-            self.LOG("[prefdlg_evt] Check Update Clicked")
+            self.LOG("[prefdlg][evt] Update Page: Check Update Clicked")
             e_obj.Disable()
             prog_bar = self.FindWindowById(ed_glob.ID_PREF_UPDATE_BAR)
             # Note this function returns right away but its result is
@@ -1061,7 +1220,7 @@ class UpdatePanel(PrefPanelBase):
             # via a custom event being posted by the control.
             prog_bar.CheckForUpdates()
         elif e_id == ID_DOWNLOAD:
-            self.LOG("[prefdlg_evt] Download Updates Clicked")
+            self.LOG("[prefdlg][evt] Update Page: Download Updates Clicked")
             e_obj.Disable()
             chk_bt = self.FindWindowById(ID_CHECK_UPDATE)
             chk_bt.Disable()
@@ -1076,7 +1235,7 @@ class UpdatePanel(PrefPanelBase):
     def OnUpdateText(self, evt):
         """Handles text update events"""
         e_id = evt.GetId()
-        self.LOG("[prefdlg_evt] Updating version status text")
+        self.LOG("[prefdlg][evt] Update Page: Updating version status text")
         txt = self.FindWindowById(ID_UPDATE_MSG)
         upd = self.FindWindowById(ed_glob.ID_PREF_UPDATE_BAR)
         if None not in [txt, upd]:
@@ -1205,8 +1364,8 @@ class ExtListCtrl(wx.ListCtrl,
             self.SetStringItem(index, self.EXT_COL, \
                                u'  %s' % u' '.join(self._extreg[key]))
             if not index % 2:
-                syscolor = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
-                color = util.AdjustColour(syscolor, 45)
+                color = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
+                color = util.AdjustColour(color, 15)
                 self.SetItemBackgroundColour(index, color)
 
         self.SetColumnWidth(self.FILE_COL, wx.LIST_AUTOSIZE)
@@ -1295,9 +1454,9 @@ class PyFontPicker(wx.Panel):
         
         # Layout
         self._sizer = wx.BoxSizer(wx.HORIZONTAL)
-        self._sizer.Add(self._text, 1, wx.ALIGN_LEFT)
+        self._sizer.Add(self._text, 1, wx.ALIGN_CENTER_VERTICAL)
         self._sizer.Add((5, 5), 0)
-        self._sizer.Add(self._button, 0, wx.ALIGN_RIGHT)
+        self._sizer.Add(self._button, 0, wx.ALIGN_CENTER_VERTICAL)
         self.SetSizer(self._sizer)
         self.SetAutoLayout(True)
         
@@ -1375,4 +1534,4 @@ class PyFontPicker(wx.Panel):
 def DoUpdates():
     """Update all open text controls"""
     for mainw in wx.GetApp().GetMainWindows():
-        mainw.nb.UpdateTextControls()
\ No newline at end of file
+        mainw.nb.UpdateTextControls()
