@@ -21,8 +21,8 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: style_editor.py 49458 2007-10-26 12:26:57Z CJP $"
-__revision__ = "$Revision: 49458 $"
+__svnid__ = "$Id: style_editor.py 51173 2008-01-12 02:08:10Z CJP $"
+__revision__ = "$Revision: 51173 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -71,9 +71,6 @@ class StyleEditor(wx.Dialog):
         """
         wx.Dialog.__init__(self, parent, id_, title, style=style)
 
-        if wx.Platform == '__WXMAC__' and Profile_Get('METAL', 'bool', False):
-            self.SetExtraStyle(wx.DIALOG_EX_METAL)
-
         # Attributes
         self.LOG = wx.GetApp().GetLog()
         self.preview = ed_stc.EditraStc(self, wx.ID_ANY, size=(-1, 200),
@@ -82,6 +79,12 @@ class StyleEditor(wx.Dialog):
         self.styles_new = DuplicateStyleDict(self.styles_orig)
         self.preview.SetStyles('preview', self.styles_new, True)
         self.OpenPreviewFile('cpp')
+        # XXX On Windows the settings pane must be made before the
+        #     sizer it is to be put in or it becomes unable to recieve
+        #     focus. But is the exact opposite on mac/gtk. This is really
+        #     a pain or possibly a bug?
+        if wx.Platform == '__WXMSW__':
+            self._settings = SettingsPanel(self)
 
         # Main Sizer
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -92,6 +95,12 @@ class StyleEditor(wx.Dialog):
         left_colum = wx.BoxSizer(wx.VERTICAL)    # Left Column
         right_colum = wx.BoxSizer(wx.VERTICAL)   # Right Column
 
+        # XXX On Mac/GTK if panel is created before sizer all controls in
+        #     it become unable to recieve focus from clicks, but it is the
+        #     exact opposite on windows!
+        if wx.Platform != '__WXMSW__':
+            self._settings = SettingsPanel(self)
+
         # Control Panel Left Column
         left_colum.AddMany([((10, 10), 0), 
                             (self.__StyleSheets(), 0, wx.ALIGN_LEFT),
@@ -109,7 +118,7 @@ class StyleEditor(wx.Dialog):
         ctrl_sizer.Add((5, 5), 0)
 
         # Control Panel Right Column
-        right_colum.Add(self.__Settings(), 1, wx.ALIGN_LEFT | wx.EXPAND)
+        right_colum.Add(self._settings, 1, wx.ALIGN_LEFT | wx.EXPAND)
         ctrl_sizer.AddMany([(right_colum, 1, wx.ALIGN_RIGHT | wx.EXPAND),
                             ((5, 5), 0)])
 
@@ -172,100 +181,6 @@ class StyleEditor(wx.Dialog):
                            ((10, 10))])
         return lex_sizer
 
-    def __Settings(self):
-        """Returns a sizer object holding all the settings controls
-        @return: main panel of configuration controls
-        @todo: this needs to be reworked, it also doesn't look as nice on 
-               msw/gtk as it does on wxmac.
-
-        """
-        setting_sizer = wx.BoxSizer(wx.VERTICAL)
-        setting_top = wx.BoxSizer(wx.HORIZONTAL)
-
-        # Settings top
-        setting_sizer.Add((10, 10))
-        cbox_sizer = wx.StaticBoxSizer(wx.StaticBox(self.ctrl_pane, 
-                                          label=_("Color") + u":"), wx.VERTICAL)
-
-        # Foreground
-        fground_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        fground_lbl = wx.StaticText(self.ctrl_pane, 
-                                    label=_("Foreground") + u": ")
-        fground_sel = ColourSetter(self.ctrl_pane, ID_FORE_COLOR, "#000000")
-        fground_sizer.AddMany([((5, 5)), 
-                               (fground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                               ((2, 2), 1, wx.EXPAND),
-                               (fground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
-                               ((5, 5))])
-        cbox_sizer.AddMany([(fground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND),
-                            ((10, 10))])
-
-        # Background
-        bground_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        bground_lbl = wx.StaticText(self.ctrl_pane, 
-                                    label=_("Background") + u": ")
-        bground_sel = ColourSetter(self.ctrl_pane, ID_BACK_COLOR, "#FFFFFF")
-        bground_sizer.AddMany([((5, 5)), 
-                               (bground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                               ((2, 2), 1, wx.EXPAND),
-                               (bground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
-                               ((5, 5))])
-        cbox_sizer.Add(bground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)
-        setting_top.AddMany([(cbox_sizer, 0, wx.ALIGN_TOP), ((10, 10))])
-
-        # Attrib Box
-        attrib_box = wx.StaticBox(self.ctrl_pane, label=_("Attributes") + u":")
-        abox_sizer = wx.StaticBoxSizer(attrib_box, wx.VERTICAL)
-
-        # Attributes
-        bold_cb = wx.CheckBox(self.ctrl_pane, ID_BOLD, _("bold"))
-        eol_cb = wx.CheckBox(self.ctrl_pane, ID_EOL, _("eol"))
-        ital_cb = wx.CheckBox(self.ctrl_pane, ID_ITALIC, _("italic"))
-        uline_cb = wx.CheckBox(self.ctrl_pane, ID_ULINE, _("underline"))
-        abox_sizer.AddMany([(bold_cb, 0, wx.ALIGN_CENTER_VERTICAL),
-                            (eol_cb, 0, wx.ALIGN_CENTER_VERTICAL),
-                            (ital_cb, 0, wx.ALIGN_CENTER_VERTICAL),
-                            (uline_cb, 0, wx.ALIGN_CENTER_VERTICAL)])
-        setting_top.Add(abox_sizer, 0, wx.ALIGN_TOP)
-
-        # Font
-        fh_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        font_box = wx.StaticBox(self.ctrl_pane, label=_("Font Settings") + u":")
-        fbox_sizer = wx.StaticBoxSizer(font_box, wx.VERTICAL)
-
-        # Font Face Name
-        fsizer = wx.BoxSizer(wx.HORIZONTAL)
-        flbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY, _("Font") + u": ")
-        fontenum = wx.FontEnumerator()
-        fontenum.EnumerateFacenames(fixedWidthOnly=True)
-        font_lst = ["%(primary)s", "%(secondary)s"]
-        font_lst.extend(sorted(fontenum.GetFacenames()))
-        fchoice = wx.Choice(self.ctrl_pane, ID_FONT, choices=font_lst)
-        fsizer.AddMany([((5, 5), 0), (flbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                        (fchoice, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5))])
-        fbox_sizer.Add(fsizer, 0, wx.ALIGN_LEFT)
-
-        # Font Size
-        fsize_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        fsize_lbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY, _("Size") + u": ")
-        fsizes = ['%(size)d', '%(size2)d']
-        fsizes.extend([ str(x) for x in xrange(4, 21) ])
-        fs_choice = wx.Choice(self.ctrl_pane, ID_FONT_SIZE, choices=fsizes)
-        fsize_sizer.AddMany([((5, 5), 0), 
-                             (fsize_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                             (fs_choice, 1, wx.EXPAND | wx.ALIGN_RIGHT), 
-                             ((5, 5), 0)])
-        fbox_sizer.AddMany([((5, 5)), 
-                            (fsize_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)])
-        fh_sizer.AddMany([(fbox_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL),
-                          ((10, 10))])
-
-        # Build Section
-        setting_sizer.AddMany([(setting_top, 0, wx.ALIGN_CENTER_HORIZONTAL),
-                               ((10, 10), 1, wx.EXPAND),
-                               (fh_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL)])
-        return setting_sizer
-
     def __StyleSheets(self):
         """Returns a sizer item that contains a choice control with
         all the available style sheets listed in it.
@@ -278,7 +193,7 @@ class StyleEditor(wx.Dialog):
         ss_lst = util.GetResourceFiles(u'styles', get_all=True)
         ss_choice = wx.Choice(self.ctrl_pane, ed_glob.ID_PREF_SYNTHEME,
                               choices=sorted(ss_lst))
-        ss_choice.SetToolTip(wx.ToolTip(_("Base new theme of existing one")))
+        ss_choice.SetToolTip(wx.ToolTip(_("Base new theme on existing one")))
         ss_choice.SetStringSelection(Profile_Get('SYNTHEME', 'str'))
         ss_new = wx.CheckBox(self.ctrl_pane, wx.ID_NEW, _("New"))
         ss_new.SetToolTip(wx.ToolTip(_("Start a blank new style")))
@@ -347,9 +262,8 @@ class StyleEditor(wx.Dialog):
         @keyword enable: whether to enable/disable settings controls
 
         """
-        for sid in SETTINGS_IDS:
-            ctrl = self.FindWindowById(sid)
-            ctrl.Enable(enable)
+        for child in self._settings.GetChildren():
+            child.Enable(enable)
 
     def ExportStyleSheet(self):
         """Writes the style sheet data out to a style sheet
@@ -376,7 +290,7 @@ class StyleEditor(wx.Dialog):
 
         if result == wx.ID_OK:
             sheet_path = dlg.GetPath()
-            if u'ess' != sheet_path.split(u'.')[-1]:
+            if sheet_path.split(u'.')[-1] != u'ess':
                 sheet_path += u".ess"
 
             try:
@@ -388,7 +302,7 @@ class StyleEditor(wx.Dialog):
                 self.LOG('[style_editor][err] %s' % str(msg))
             else:
                 # Update Style Sheet Control
-                sheet = os.path.basename(sheet_path).split(u'.')[0]
+                sheet = ".".join(os.path.basename(sheet_path).split(u'.')[:-1])
                 ss_c = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
                 ss_c.SetItems(util.GetResourceFiles(u'styles', get_all=True))
                 ss_c.SetStringSelection(sheet)
@@ -397,7 +311,7 @@ class StyleEditor(wx.Dialog):
 
                 if sheet_path.startswith(ed_glob.CONFIG['STYLES_DIR']) or \
                    sheet_path.startswith(ed_glob.CONFIG['SYS_STYLES_DIR']):
-                    # Update editor windows to use new style sheet
+                    # Update editor windows/buffer to use new style sheet
                     Profile_Set('SYNTHEME', sheet)
                     for mainw in wx.GetApp().GetMainWindows():
                         mainw.nb.UpdateTextControls()
@@ -413,18 +327,29 @@ class StyleEditor(wx.Dialog):
 
         """
         sty_sheet = list()
-        for tag in self.styles_new:
-            sty_sheet.append(tag + u" {\n")
-            sdat = unicode(self.styles_new[tag]).split(u",")
+        ditem = self.styles_new.get('default_style', StyleItem())
+        dvals = ';\n\t\t'.join([item.replace(',', ' ') 
+                                for item in ditem.GetAsList() ]) + ';'
+        sty_sheet.append(''.join(['default_style {\n\t\t', dvals, '\n\n}\n\n']))
+        for tag, item in self.styles_new.iteritems():
+            if item.IsNull() or tag == 'default_style':
+                continue
+
             stage1 = wx.EmptyString
-            for atom in sdat:
-                if atom in "bold eol italic underline":
-                    stage1 = stage1[0:-1] + u" " + atom + u";"
-                else:
-                    stage1 = stage1 + atom + u";"
-            stage2 = u"\t\t" + stage1[0:-1].replace(u";", u";\n\t\t") + u";"
-            sty_sheet.append(stage2)
-            sty_sheet.append(u"\n}\n\n")
+            for attr in ('fore', 'back', 'face', 'size'):
+                ival = item.GetNamedAttr(attr)
+                if ival is None or ival == ditem.GetNamedAttr(attr):
+                    continue
+
+                stage1 = ''.join((stage1, attr, u':', 
+                                  ival.replace(',', ' '), u';'))
+
+            if len(stage1):
+                sty_sheet.append(tag + u" {\n")
+                stage2 = u"\t\t" + stage1[0:-1].replace(u";", u";\n\t\t") + u";"
+                sty_sheet.append(stage2)
+                sty_sheet.append(u"\n}\n\n")
+
         return u"".join(sty_sheet)
 
     def OnCancel(self, evt):
@@ -433,7 +358,7 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][cancel] Cancel Clicked Closing Window')
+        self.LOG('[style_editor][evt] Cancel Clicked Closing Window')
         evt.Skip()
 
     def OnCheck(self, evt):
@@ -448,14 +373,19 @@ class StyleEditor(wx.Dialog):
             choice = self.ctrl_pane.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
             choice.Enable(not val)
             if val:
-                self.preview.StyleDefault()
                 self.styles_orig = self.preview.BlankStyleDictionary()
                 self.styles_new = DuplicateStyleDict(self.styles_orig)
                 self.preview.SetStyles('preview', self.styles_new, nomerge=True)
-                self.preview.DefineMarkers()
+                self.preview.UpdateBaseStyles()
+
+                # For some reason this causes the text display to refresh
+                # properly when nothing else would work.
+                self.OnTextRegion()
             else:
                 scheme = choice.GetStringSelection().lower()
                 self.preview.UpdateAllStyles(scheme)
+                self.styles_orig = self.preview.GetStyleSet()
+                self.styles_new = DuplicateStyleDict(self.styles_orig)
         elif e_id in [ID_BOLD, ID_EOL, ID_ULINE, ID_ITALIC]:
             self.UpdateStyleSet(e_id)
         else:
@@ -501,13 +431,16 @@ class StyleEditor(wx.Dialog):
         # Update The Style data for current tag
         self.UpdateStyleSet(evt.GetId())
 
-    def OnTextRegion(self, evt):
+    def OnTextRegion(self, evt=None):
         """Processes clicks in the preview control and sets the style
         selection in the style tags list to the style tag of the area
         the cursor has moved into.
         @param evt: event that called this handler
 
         """
+        if evt is not None:
+            evt.Skip()
+
         style_id = self.preview.GetStyleAt(self.preview.GetCurrentPos())
         tag_lst = self.FindWindowById(ID_STYLES)
         data = self.preview.FindTagById(style_id)
@@ -517,7 +450,6 @@ class StyleEditor(wx.Dialog):
                 tag_lst.SetFirstItemStr(data)
             self.UpdateSettingsPane(self.styles_new[data])
             self.EnableSettings()
-        evt.Skip()
 
     def OnListBox(self, evt):
         """Catches the selection of a style tag in the listbox
@@ -538,7 +470,7 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][info] Ok Clicked Closing Window')
+        self.LOG('[style_editor][evt] Ok Clicked Closing Window')
         result = self.DiffStyles()
         if result == wx.ID_NO:
             evt.Skip()
@@ -554,7 +486,7 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][export] Saving style changes')
+        self.LOG('[style_editor][evt] Export Clicked')
         self.ExportStyleSheet()
 
     def OpenPreviewFile(self, file_lbl):
@@ -593,6 +525,9 @@ class StyleEditor(wx.Dialog):
                     ID_FONT       : syntax_data.GetFace(),
                     ID_FONT_SIZE  : syntax_data.GetSize()
                   }
+
+        # Fall back to defaults for color values
+        # that we may not be able to understand
         if u"#" not in val_map[ID_FORE_COLOR]:
             val_map[ID_FORE_COLOR] = self.preview.GetDefaultForeColour(as_hex=True)
         if u"#" not in val_map[ID_BACK_COLOR]:
@@ -658,6 +593,110 @@ class StyleEditor(wx.Dialog):
         self.preview.SetStyleTag(tag, self.styles_new[tag])
         self.preview.RefreshStyles()
 
+#-----------------------------------------------------------------------------#
+
+class SettingsPanel(wx.Panel):
+    """Panel holding all settings controls for changing the font,
+    colors, styles, ect.. in the style set.
+
+    """
+    def __init__(self, parent):
+        """Create the settings panel"""
+        wx.Panel.__init__(self, parent)
+
+        # Attributes
+        
+        # Layout
+        self.__DoLayout()
+
+    def __DoLayout(self):
+        """Layout the controls in the panel"""
+        setting_sizer = wx.BoxSizer(wx.VERTICAL)
+        setting_top = wx.BoxSizer(wx.HORIZONTAL)
+
+        # Settings top
+        setting_sizer.Add((10, 10))
+        cbox_sizer = wx.StaticBoxSizer(wx.StaticBox(self, 
+                                          label=_("Color") + u":"), wx.VERTICAL)
+
+        # Foreground
+        fground_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        fground_lbl = wx.StaticText(self, label=_("Foreground") + u": ")
+        fground_sel = ColourSetter(self, ID_FORE_COLOR, "#000000")
+        fground_sizer.AddMany([((5, 5)), 
+                               (fground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
+                               ((2, 2), 1, wx.EXPAND),
+                               (fground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
+                               ((5, 5))])
+        cbox_sizer.AddMany([(fground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND),
+                            ((10, 10))])
+
+        # Background
+        bground_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        bground_lbl = wx.StaticText(self, label=_("Background") + u": ")
+        bground_sel = ColourSetter(self, ID_BACK_COLOR, "#FFFFFF")
+        bground_sizer.AddMany([((5, 5)), 
+                               (bground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
+                               ((2, 2), 1, wx.EXPAND),
+                               (bground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
+                               ((5, 5))])
+        cbox_sizer.Add(bground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)
+        setting_top.AddMany([(cbox_sizer, 0, wx.ALIGN_TOP), ((10, 10))])
+
+        # Attrib Box
+        attrib_box = wx.StaticBox(self, label=_("Attributes") + u":")
+        abox_sizer = wx.StaticBoxSizer(attrib_box, wx.VERTICAL)
+
+        # Attributes
+        bold_cb = wx.CheckBox(self, ID_BOLD, _("bold"))
+        eol_cb = wx.CheckBox(self, ID_EOL, _("eol"))
+        ital_cb = wx.CheckBox(self, ID_ITALIC, _("italic"))
+        uline_cb = wx.CheckBox(self, ID_ULINE, _("underline"))
+        abox_sizer.AddMany([(bold_cb, 0, wx.ALIGN_CENTER_VERTICAL),
+                            (eol_cb, 0, wx.ALIGN_CENTER_VERTICAL),
+                            (ital_cb, 0, wx.ALIGN_CENTER_VERTICAL),
+                            (uline_cb, 0, wx.ALIGN_CENTER_VERTICAL)])
+        setting_top.Add(abox_sizer, 0, wx.ALIGN_TOP)
+
+        # Font
+        fh_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        font_box = wx.StaticBox(self, label=_("Font Settings") + u":")
+        fbox_sizer = wx.StaticBoxSizer(font_box, wx.VERTICAL)
+
+        # Font Face Name
+        fsizer = wx.BoxSizer(wx.HORIZONTAL)
+        flbl = wx.StaticText(self, label=_("Font") + u": ")
+        fontenum = wx.FontEnumerator()
+        fontenum.EnumerateFacenames(fixedWidthOnly=True)
+        font_lst = ["%(primary)s", "%(secondary)s"]
+        font_lst.extend(sorted(fontenum.GetFacenames()))
+        fchoice = wx.Choice(self, ID_FONT, choices=font_lst)
+        fsizer.AddMany([((5, 5), 0), (flbl, 0, wx.ALIGN_CENTER_VERTICAL),
+                        (fchoice, 0, wx.ALIGN_CENTER_VERTICAL), ((5, 5))])
+        fbox_sizer.Add(fsizer, 0, wx.ALIGN_LEFT)
+
+        # Font Size
+        fsize_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        fsize_lbl = wx.StaticText(self, label=_("Size") + u": ")
+        fsizes = ['%(size)d', '%(size2)d']
+        fsizes.extend([ str(x) for x in xrange(4, 21) ])
+        fs_choice = wx.Choice(self, ID_FONT_SIZE, choices=fsizes)
+        fsize_sizer.AddMany([((5, 5), 0), 
+                             (fsize_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
+                             (fs_choice, 1, wx.EXPAND | wx.ALIGN_RIGHT), 
+                             ((5, 5), 0)])
+        fbox_sizer.AddMany([((5, 5)), 
+                            (fsize_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)])
+        fh_sizer.AddMany([(fbox_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL),
+                          ((10, 10))])
+
+        # Build Section
+        setting_sizer.AddMany([(setting_top, 0, wx.ALIGN_CENTER_HORIZONTAL),
+                               ((10, 10), 1, wx.EXPAND),
+                               (fh_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL)])
+        self.SetSizer(setting_sizer)
+        self.SetAutoLayout(True)
+
 #-----------------------------------------------------------------------------#
 class ColourSetter(wx.Panel):
     """Control for setting and selecting a color to describe the
@@ -674,7 +713,9 @@ class ColourSetter(wx.Panel):
 
         # Attributes
         self._label = label
-        self._txt = wx.TextCtrl(self, value=label, style=wx.TE_CENTER)
+        self._txt = wx.TextCtrl(self, value=label,
+                                style=wx.TE_CENTER,
+                                validator=HexValidator())
         txtheight = self._txt.GetTextExtent('#000000')[1]
         self._txt.SetMaxSize((-1, txtheight + 4))
         self._txt.SetToolTip(wx.ToolTip(_("Enter a hex color value")))
@@ -698,9 +739,9 @@ class ColourSetter(wx.Panel):
     def _DoLayout(self):
         """Layout the controls"""
         sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(self._txt, 0, wx.EXPAND)
+        sizer.Add(self._txt, 0, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
         sizer.Add((5, 5), 0)
-        sizer.Add(self._cbtn, 0, wx.ALIGN_LEFT)
+        sizer.Add(self._cbtn, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
         self.SetSizer(sizer)
 
     def GetColour(self):
@@ -745,11 +786,21 @@ class ColourSetter(wx.Panel):
         @keyword evt: event that called this handler
 
         """
-        hexstr = self._txt.GetValue().replace('#', '')
-        if len(hexstr) > 6:
-            self._txt.SetValue(u'#' + hexstr[:6])
-        hexstr = '#' + hexstr + (u'0' * (6 - len(hexstr)))
-        self._cbtn.SetValue(util.HexToRGB(hexstr))
+        cpos = self._txt.GetInsertionPoint()
+        hexstr = self._txt.GetValue().replace('#', '').strip()
+        valid = ''
+        for char in hexstr:
+            if char in '0123456789abcdefABCDEF':
+                valid = valid + char
+
+        if len(valid) > 6:
+            valid = valid[:6]
+
+        valid = '#' + valid
+        self._txt.SetValue(valid)
+        self._txt.SetInsertionPoint(cpos)
+        valid = valid + (u'0' * (6 - len(valid)))
+        self._cbtn.SetValue(util.HexToRGB(valid))
         self.__PostEvent()
 
     def OnUpdateUI(self, evt):
@@ -761,7 +812,11 @@ class ColourSetter(wx.Panel):
         for char in self._txt.GetValue().replace('#', ''):
             if char in '0123456789abcdefABCDEF':
                 ret = ret + char
-        self._txt.SetValue(u'#' + ret)
+
+        if ret != self._txt.GetValue():
+            cpos = self._txt.GetInsertionPoint()
+            self._txt.SetValue(u'#' + ret)
+            self._txt.SetInsertionPoint(cpos)
 
     def OnValidateTxt(self, evt):
         """Validate text to ensure only valid hex characters are entered
@@ -775,6 +830,10 @@ class ColourSetter(wx.Panel):
             return
 
         key = unichr(code)
+        if (key.isdigit() and evt.ShiftDown()) or \
+           evt.AltDown() or evt.MetaDown():
+            return
+
         if key in "0123456789ABCDEFabcdef#" and \
            (len(self._txt.GetValue().lstrip("#")) < 6 or \
             self._txt.GetStringSelection()):
@@ -801,6 +860,47 @@ class ColourSetter(wx.Panel):
         self._txt.SetValue(hex_str)
         self.__PostEvent()
 
+class HexValidator(wx.PyValidator):
+    """Validate Hex strings for the color setter"""
+    def __init__(self):
+        """Initialize the validator
+
+        """
+        wx.PyValidator.__init__(self)
+
+        # Event Handlers
+        self.Bind(wx.EVT_CHAR, self.OnChar)
+
+    def Clone(self):
+        """Clones the current validator
+        @return: clone of this object
+
+        """
+        return HexValidator()
+
+    def Validate(self, win):
+        """Validate an window value
+        @param win: window to validate
+
+        """
+        return win.GetValue() in '#0123456789abcdefABCDEF'
+
+    def OnChar(self, event):
+        """Process values as they are entered into the control
+        @param event: event that called this handler
+
+        """
+        key = event.GetKeyCode()
+        if event.CmdDown() or key < wx.WXK_SPACE or key == wx.WXK_DELETE or \
+           key > 255 or chr(key) in '0123456789abcdefABCDEF':
+            event.Skip()
+            return
+
+        if not wx.Validator_IsSilent():
+            wx.Bell()
+
+        return
+
 #-----------------------------------------------------------------------------#
 # Utility funtcions
 def DuplicateStyleDict(style_dict):
@@ -815,5 +915,7 @@ def DuplicateStyleDict(style_dict):
     new_dict = dict()
     for tag in style_dict:
         new_dict[tag] = StyleItem()
-        new_dict[tag].SetAttrFromStr(unicode(style_dict[tag]))
+        ok = new_dict[tag].SetAttrFromStr(unicode(style_dict[tag]))
+        if not ok:
+            new_dict[tag].null = True
     return new_dict
