@@ -26,18 +26,21 @@
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 49551 2007-10-31 12:01:14Z CJP $"
-__revision__ = "$Revision: 49551 $"
+__svnid__ = "$Id: util.py 51220 2008-01-15 07:52:30Z CJP $"
+__revision__ = "$Revision: 51220 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import sys
+import re
 import codecs
 import mimetypes
+import urllib2
 import wx
-import ed_event
 import ed_glob
+import ed_event
+import ed_crypt
 from syntax.syntax import GetFileExtensions
 import dev_tool
 
@@ -106,16 +109,20 @@ class DropTargetFT(wx.PyDropTarget):
         @return: result of drop object entering window
 
         """
-        try:
-            if self.GetData():
-                files = self._data['fdata'].GetFilenames()
-                text = self._data['tdata'].GetText()
-            else:
-                return drag_result
-        except wx.PyAssertionError:
-            return wx.DragError
+        # GetData seems to happen automatically on msw, calling it again
+        # causes this to fail the first time.
+        if wx.Platform in ['__WXGTK__', '__WXMSW__']:
+            return wx.DragCopy
+
+        if wx.Platform == '__WXMAC__':
+            try:
+                self.GetData()
+            except wx.PyAssertionError:
+                return wx.DragError
 
         self._lastp = (x_cord, y_cord)
+        files = self._data['fdata'].GetFilenames()
+        text = self._data['tdata'].GetText()
         if len(files):
             self.window.SetCursor(wx.StockCursor(wx.CURSOR_COPY_ARROW))
         else:
@@ -163,6 +170,9 @@ class DropTargetFT(wx.PyDropTarget):
 
         """
         self.window.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
+        if self.window.HasCapture():
+            self.window.ReleaseMouse()
+
         try:
             data = self.GetData()
         except wx.PyAssertionError:
@@ -185,6 +195,8 @@ class DropTargetFT(wx.PyDropTarget):
                     if pos != wx.stc.STC_INVALID_POSITION:
                         win.SetSelection(pos, pos)
                         win.Paste()
+                    else:
+                        drag_result = wx.DragCancel
         self.InitObjects()
         return drag_result
 
@@ -194,8 +206,14 @@ class DropTargetFT(wx.PyDropTarget):
 
         """
         self.window.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
+        if self.window.HasCapture():
+            self.window.ReleaseMouse()
+
         if self._tmp is not None:
-            self._tmp.EndDrag()
+            try:
+                self._tmp.EndDrag()
+            except wx.PyAssertionError, msg:
+                Log("[droptargetft][err] %s" % str(msg))
 
 #---- End FileDropTarget ----#
 
@@ -225,6 +243,10 @@ BOM = { 'utf-8' : codecs.BOM_UTF8,
 ENC = [ 'utf-8', 'utf-7', 'latin-1', 'utf-16-be', 'utf-16-le', 'ascii']
   #      'utf-32-be', 'utf-32-le', 
 
+# Regex for extracting magic comments from source files
+# i.e *-* coding: utf-8 *-*
+RE_MAGIC_COMMENT = re.compile("coding[:=]\s*([-\w.]+)")
+
 def DecodeString(str2decode):
     """Decode a given string if possible and return that string
     @param str2decode: the string to decode
@@ -395,20 +417,25 @@ def GetPathChar():
 def GetUniqueName(path, name):
     """Make a file name that will be unique in case a file of the
     same name already exists at that path.
-    @param path: Root path to download folder
+    @param path: Root path to folder of files destination
     @param name: desired file name base
     @return: string
 
     """
     tmpname = os.path.join(path, name)
     if os.path.exists(tmpname):
-        ext = name.split('.')[-1]
-        fbase = name[:-1 * len(ext) - 1]
+        if '.' not in name:
+            ext = ''
+            fbase = name
+        else:
+            ext = '.' + name.split('.')[-1]
+            fbase = name[:-1 * len(ext)]
+
         inc = len([x for x in os.listdir(path) if x.startswith(fbase)])
-        tmpname = os.path.join(path, "%s-%d.%s" % (fbase, inc, ext))
+        tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
         while os.path.exists(tmpname):
             inc = inc + 1
-            tmpname = os.path.join(path, "%s-%d.%s" % (fbase, inc, ext))
+            tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
 
     return tmpname
 
@@ -439,6 +466,45 @@ def GetExtension(file_str):
     extension = pieces[-1]
     return extension
 
+def MakeNewFile(path, name):
+    """Make a new file at the given path with the given name.
+    If the file already exists, the given name will be changed to
+    a unique name in the form of name + -NUMBER + .extension
+    @param path: path to directory to create file in
+    @param name: desired name of file
+    @return: Tuple of (success?, Path of new file OR Error message)
+
+    """
+    if not os.path.isdir(path):
+        path = os.path.dirname(path)
+    fname = GetUniqueName(path, name)
+
+    try:
+        open(fname, 'w').close()
+    except (IOError, OSError), msg:
+        return (False, str(msg))
+
+    return (True, fname)
+
+def MakeNewFolder(path, name):
+    """Make a new folder at the given path with the given name.
+    If the folder already exists, the given name will be changed to
+    a unique name in the form of name + -NUMBER.
+    @param path: path to create folder on
+    @param name: desired name for folder
+    @return: Tuple of (success?, new dirname OR Error message)
+
+    """
+    if not os.path.isdir(path):
+        path = os.path.dirname(path)
+    folder = GetUniqueName(path, name)
+    try:
+        os.mkdir(folder)
+    except (OSError, IOError), msg:
+        return (False, str(msg))
+
+    return (True, folder)
+
 def ResolvAbsPath(rel_path):
     """Takes a relative path and converts it to an
     absolute path.
@@ -637,9 +703,13 @@ def GetResourceFiles(resource, trim=True, get_all=False):
         for rec in recs:
             if os.path.isfile(rec_dir + rec) or \
               (get_all and os.path.isfile(rec_dir2 + rec)):
+                # Trim the last part of an extension if one exists
                 if trim:
-                    rec = rec.split(u".")[0]
-                rec_list.append(rec.title())
+                    rec = ".".join(rec.split(u".")[:-1]).strip()
+
+                if len(rec):
+                    rec = rec[0].upper() + rec[1:]
+                    rec_list.append(rec)
         rec_list.sort()
         return list(set(rec_list))
 
@@ -650,6 +720,32 @@ def Log(msg):
     """
     wx.GetApp().GetLog()(msg)
 
+def GetProxyOpener(proxy_set):
+    """Get a urlopener for use with a proxy
+    @param proxy_set: proxy settings to use
+
+    """
+    Log("[util][info] Making proxy opener with %s" % str(proxy_set))
+    proxy_info = dict(proxy_set)
+    auth_str = "%(uname)s:%(passwd)s@%(url)s"
+    url = proxy_info['url']
+    if url.startswith('http://'):
+        auth_str = "http://" + auth_str
+        proxy_info['url'] = url.replace('http://', '')
+    else:
+        pass
+
+    if len(proxy_info.get('port', '')):
+        auth_str = auth_str + ":%(port)s"
+
+    proxy_info['passwd'] = ed_crypt.Decrypt(proxy_info['passwd'],
+                                            proxy_info['pid'])
+    Log("[util][info] Formatted proxy request: %s" % \
+        (auth_str.replace('%(passwd)s', '****') % proxy_info))
+    proxy = urllib2.ProxyHandler({"http" : auth_str % proxy_info})
+    opener = urllib2.build_opener(proxy, urllib2.HTTPHandler)
+    return opener
+
 #---- GUI helper functions ----#
 def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
     """ Brighten/Darken input colour by percent and adjust alpha
@@ -658,20 +754,21 @@ def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
     @type color: wx.Color
     @param percent: percent to adjust +(brighten) or -(darken)
     @type percent: int
-    @keyword alpha: amount to adjust alpha channel
+    @keyword alpha: Value to adjust alpha channel to
 
-    """ 
-    end_color = wx.WHITE
-    rdif = end_color.Red() - color.Red()
-    gdif = end_color.Green() - color.Green()
-    bdif = end_color.Blue() - color.Blue()
-    high = 100
+    """
+    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
+                         for val in color.Get() ]
 
-    # We take the percent way of the color from color -. white
-    red = color.Red() + ((percent * rdif) / high)
-    green = color.Green() + ((percent * gdif) / high)
-    blue = color.Blue() + ((percent * bdif) / high)
-    return wx.Colour(max(red, 0), max(green, 0), max(blue, 0), alpha)
+    if percent < 0:
+        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
+    else:
+        radj, gadj, badj = [ val or percent for val in [radj, gadj, badj] ]
+
+    red = min(color.Red() + radj, 255)
+    green = min(color.Green() + gadj, 255)
+    blue = min(color.Blue() + badj, 255)
+    return wx.Colour(red, green, blue, alpha)
 
 def HexToRGB(hex_str):
     """Returns a list of red/green/blue values from a
